@@ -232,6 +232,7 @@
   mrc p15, 0, \tmp, c1, c0, 0 /* Read SCTLR */
   orr \tmp, \tmp, #(0x1 << 2) /* Enable D cache */
   mcr p15, 0, \tmp, c1, c0, 0 /* Update the SCTLR */
+  isb
 .endm
 
 /****************************************************************************
@@ -252,6 +253,7 @@
   mrc p15, 0, \tmp, c1, c0, 0 /* Read SCTLR */
   bic \tmp, \tmp, #(0x1 << 2) /* Disable D cache */
   mcr p15, 0, \tmp, c1, c0, 0 /* Update the SCTLR */
+  isb
 .endm
 
 /****************************************************************************
@@ -272,6 +274,7 @@
   mrc p15, 0, \tmp, c1, c0, 0  /* Read SCTLR */
   orr \tmp, \tmp, #(0x1 << 12) /* Enable I cache */
   mcr p15, 0, \tmp, c1, c0, 0  /* Update the SCTLR */
+  isb
 .endm
 
 /****************************************************************************
@@ -292,6 +295,7 @@
   mrc p15, 0, \tmp, c1, c0, 0  /* Read SCTLR */
   bic \tmp, \tmp, #(0x1 << 12) /* Disable I cache */
   mcr p15, 0, \tmp, c1, c0, 0  /* Update the SCTLR */
+  isb
 .endm
 
 /****************************************************************************
@@ -311,6 +315,7 @@
 .macro cp15_invalidate_icache_inner_sharable, tmp
   mov \tmp, #0
   mrc p15, 0, \tmp, c7, c1, 0 /* ICIALLUIS */
+  isb
 .endm
 
 /****************************************************************************
@@ -330,10 +335,11 @@
 .macro cp15_invalidate_btb_inner_sharable, tmp
   mov \tmp, #0
   mrc p15, 0, \tmp, c7, c1, 6 /* BPIALLIS */
+  isb
 .endm
 
 /****************************************************************************
- * Name: cp15_invalidate_icache
+ * Name: cp15_invalidate_icache_all
  *
  * Description:
  *   Invalidate all instruction caches to PoU, also flushes branch target
@@ -347,7 +353,7 @@
  *
  ****************************************************************************/
 
-.macro cp15_invalidate_icache, tmp
+.macro cp15_invalidate_icache_all, tmp
   mov \tmp, #0
   mrc p15, 0, \tmp, c7, c5, 0 /* ICIALLU */
   isb
@@ -369,6 +375,7 @@
 
 .macro cp15_invalidate_icache_bymva, va
   mrc p15, 0, \va, c7, c5, 1 /* ICIMVAU */
+  isb
 .endm
 
 /****************************************************************************
@@ -388,6 +395,7 @@
 .macro cp15_flush_btb, tmp
   mov \tmp, #0
   mrc p15, 0, \tmp, c7, c5, 6 /* BPIALL */
+  isb
 .endm
 
 /****************************************************************************
@@ -404,9 +412,9 @@
  *
  ****************************************************************************/
 
-.macro cp15_flush_btb_bymva, tmp
-  mov \tmp, #0
-  mrc p15, 0, \tmp, c7, c5, 7 /* BPIMVA */
+.macro cp15_flush_btb_bymva, va
+  mrc p15, 0, \va, c7, c5, 7 /* BPIMVA */
+  isb
 .endm
 
 /****************************************************************************
@@ -425,6 +433,7 @@
 
 .macro cp15_invalidate_dcacheline_bymva, va
   mrc p15, 0, \va, c7, c6, 1 /* DCIMVAC */
+  isb
 .endm
 
 /****************************************************************************
@@ -443,6 +452,7 @@
 
 .macro cp15_invalidate_dcacheline_bysetway, setway
   mrc p15, 0, \setway, c7, c6, 2 /* DCISW */
+  isb
 .endm
 
 /****************************************************************************
@@ -461,6 +471,7 @@
 
 .macro cp15_clean_dcache_bymva, va
   mrc p15, 0, \va, c7, c10, 1 /* DCCMVAC */
+  isb
 .endm
 
 /****************************************************************************
@@ -479,6 +490,7 @@
 
 .macro cp15_clean_dcache_bysetway, setway
   mrc p15, 0, \setway, c7, c10, 2 /* DCCSW */
+  isb
 .endm
 
 /****************************************************************************
@@ -495,8 +507,9 @@
  *
  ****************************************************************************/
 
-.macro cp15_clean_ucache_bymva, setway
-  mrc p15, 0, \setway, c7, c11, 1 /* DCCMVAU */
+.macro cp15_clean_ucache_bymva, va
+  mrc p15, 0, \va, c7, c11, 1 /* DCCMVAU */
+  isb
 .endm
 
 /****************************************************************************
@@ -515,6 +528,7 @@
 
 .macro cp15_cleaninvalidate_dcacheline_bymva, va
   mrc p15, 0, \va, c7, c14, 1 /* DCCIMVAC */
+  isb
 .endm
 
 /****************************************************************************
@@ -533,6 +547,7 @@
 
 .macro cp15_cleaninvalidate_dcacheline, setway
   mrc p15, 0, \setway, c7, c14, 2 /* DCCISW */
+  isb
 .endm
 
 #endif /* __ASSEMBLY__ */
@@ -564,6 +579,7 @@ static inline void cp15_enable_dcache(void)
   sctlr = CP15_GET(SCTLR);
   sctlr |= SCTLR_C;
   CP15_SET(SCTLR, sctlr);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -587,6 +603,7 @@ static inline void cp15_disable_dcache(void)
   sctlr = CP15_GET(SCTLR);
   sctlr &= ~SCTLR_C;
   CP15_SET(SCTLR, sctlr);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -610,6 +627,7 @@ static inline void cp15_enable_icache(void)
   sctlr = CP15_GET(SCTLR);
   sctlr |= SCTLR_I;
   CP15_SET(SCTLR, sctlr);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -633,6 +651,7 @@ static inline void cp15_disable_icache(void)
   sctlr = CP15_GET(SCTLR);
   sctlr &= ~SCTLR_I;
   CP15_SET(SCTLR, sctlr);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -652,6 +671,7 @@ static inline void cp15_disable_icache(void)
 static inline void cp15_invalidate_icache_inner_sharable(void)
 {
   CP15_SET(ICIALLUIS, 0);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -671,10 +691,11 @@ static inline void cp15_invalidate_icache_inner_sharable(void)
 static inline void cp15_invalidate_btb_inner_sharable(void)
 {
   CP15_SET(BPIALLIS, 0);
+  ARM_ISB();
 }
 
 /****************************************************************************
- * Name: cp15_invalidate_icache
+ * Name: cp15_invalidate_icache_all
  *
  * Description:
  *   Invalidate all instruction caches to PoU, also flushes branch target
@@ -688,7 +709,7 @@ static inline void cp15_invalidate_btb_inner_sharable(void)
  *
  ****************************************************************************/
 
-static inline void cp15_invalidate_icache(void)
+static inline void cp15_invalidate_icache_all(void)
 {
   CP15_SET(ICIALLU, 0);
   ARM_ISB();
@@ -711,6 +732,7 @@ static inline void cp15_invalidate_icache(void)
 static inline void cp15_invalidate_icache_bymva(unsigned int va)
 {
   CP15_SET(ICIMVAU, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -730,6 +752,7 @@ static inline void cp15_invalidate_icache_bymva(unsigned int va)
 static inline void cp15_flush_btb(void)
 {
   CP15_SET(BPIALL, 0);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -739,16 +762,17 @@ static inline void cp15_flush_btb(void)
  *   Invalidate branch predictor array entry by MVA
  *
  * Input Parameters:
- *   None
+ *   va - 32-bit value with VA format
  *
  * Returned Value:
  *   None
  *
  ****************************************************************************/
 
-static inline void cp15_flush_btb_bymva(void)
+static inline void cp15_flush_btb_bymva(unsigned int va)
 {
-  CP15_SET(BPIMVA, 0);
+  CP15_SET(BPIMVA, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -770,6 +794,7 @@ static inline void cp15_flush_btb_bymva(void)
 static inline void cp15_invalidate_dcacheline_bymva(unsigned int va)
 {
   CP15_SET(DCIMVAC, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -791,6 +816,7 @@ static inline void cp15_invalidate_dcacheline_bymva(unsigned int va)
 static inline void cp15_invalidate_dcacheline_bysetway(unsigned int setway)
 {
   CP15_SET(DCISW, setway);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -812,6 +838,7 @@ static inline void cp15_invalidate_dcacheline_bysetway(unsigned int setway)
 static inline void cp15_clean_dcache_bymva(unsigned int va)
 {
   CP15_SET(DCCMVAC, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -831,6 +858,7 @@ static inline void cp15_clean_dcache_bymva(unsigned int va)
 static inline void cp15_clean_dcache_bysetway(unsigned int setway)
 {
   CP15_SET(DCCSW, setway);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -840,16 +868,17 @@ static inline void cp15_clean_dcache_bysetway(unsigned int setway)
  *   Clean unified cache line by MVA
  *
  * Input Parameters:
- *   setway - 32-bit value with VA format
+ *   va - 32-bit value with VA format
  *
  * Returned Value:
  *   None
  *
  ****************************************************************************/
 
-static inline void cp15_clean_ucache_bymva(unsigned int setway)
+static inline void cp15_clean_ucache_bymva(unsigned int va)
 {
-  CP15_SET(DCCMVAU, setway);
+  CP15_SET(DCCMVAU, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -869,6 +898,7 @@ static inline void cp15_clean_ucache_bymva(unsigned int setway)
 static inline void cp15_cleaninvalidate_dcacheline_bymva(unsigned int va)
 {
   CP15_SET(DCCIMVAC, va);
+  ARM_ISB();
 }
 
 /****************************************************************************
@@ -888,6 +918,7 @@ static inline void cp15_cleaninvalidate_dcacheline_bymva(unsigned int va)
 static inline void cp15_cleaninvalidate_dcacheline(unsigned int setway)
 {
   CP15_SET(DCCISW, setway);
+  ARM_ISB();
 }
 
 #endif /* __ASSEMBLY__ */
@@ -909,6 +940,23 @@ extern "C"
  * Public Function Prototypes
  ****************************************************************************/
 
+/****************************************************************************
+ * Name: cp15_invalidate_icache
+ *
+ * Description:
+ *   Invalidate the instruction cache within the specified region.
+ *
+ * Input Parameters:
+ *   start - virtual start address of region
+ *   end   - virtual end address of region + 1
+ *
+ * Returned Value:
+ *   None
+ *
+ ****************************************************************************/
+
+void cp15_invalidate_icache(uintptr_t start, uintptr_t end);
+
 /****************************************************************************
  * Name: cp15_dcache_op_level
  *
@@ -1065,6 +1113,22 @@ void cp15_flush_dcache_all(void);
 
 uint32_t cp15_cache_size(void);
 
+/****************************************************************************
+ * Name: cp15_cache_linesize
+ *
+ * Description:
+ *   Get cp15 cache linesize in byte
+ *
+ * Input Parameters:
+ *   None
+ *
+ * Returned Value:
+ *   Cache linesize in byte
+ *
+ ****************************************************************************/
+
+uint32_t cp15_cache_linesize(void);
+
 #undef EXTERN
 #ifdef __cplusplus
 }
