@@ -100,10 +100,68 @@ static inline uint32_t arm_clz(unsigned int value)
 }
 #endif
 
+/****************************************************************************
+ * Private Functions
+ ****************************************************************************/
+
+/****************************************************************************
+ * Name: up_get_cache_linesize
+ *
+ * Description:
+ *   Get cache linesize
+ *
+ * Input Parameters:
+ *   None
+ *
+ * Returned Value:
+ *   Cache line size
+ *
+ ****************************************************************************/
+
+#if defined(CONFIG_ARMV8M_ICACHE) || defined(CONFIG_ARMV8M_DCACHE)
+static size_t up_get_cache_linesize(void)
+{
+  static uint32_t clsize;
+
+  if (clsize == 0)
+    {
+      uint32_t ccsidr;
+      uint32_t sshift;
+
+      ccsidr = getreg32(NVIC_CCSIDR);
+      sshift = CCSIDR_LSSHIFT(ccsidr) + 4;   /* log2(cache-line-size-in-bytes) */
+      clsize = 1 << sshift;
+    }
+
+  return clsize;
+}
+#endif
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
 
+/****************************************************************************
+ * Name: up_get_icache_linesize
+ *
+ * Description:
+ *   Get icache linesize
+ *
+ * Input Parameters:
+ *   None
+ *
+ * Returned Value:
+ *   Cache line size
+ *
+ ****************************************************************************/
+
+#ifdef CONFIG_ARMV8M_ICACHE
+size_t up_get_icache_linesize(void)
+{
+  return up_get_cache_linesize();
+}
+#endif
+
 /****************************************************************************
  * Name: up_enable_icache
  *
@@ -196,20 +254,10 @@ void up_disable_icache(void)
 #ifdef CONFIG_ARMV8M_ICACHE
 void up_invalidate_icache(uintptr_t start, uintptr_t end)
 {
-  uint32_t ccsidr;
-  uint32_t sshift;
-  uint32_t ssize;
+  uint32_t ssize = up_get_icache_linesize();
 
-  /* Get the characteristics of the I-Cache */
-
-  ccsidr = getreg32(NVIC_CCSIDR);
-  sshift = CCSIDR_LSSHIFT(ccsidr) + 4;   /* log2(cache-line-size-in-bytes) */
-
-  /* Invalidate the I-Cache containing this range of addresses */
-
-  ssize  = (1 << sshift);
-
-  /* Round down the start address to the nearest cache line boundary.
+  /* Invalidate the I-Cache containing this range of addresses.
+   * Round down the start address to the nearest cache line boundary.
    *
    *   sshift = 5      : Offset to the beginning of the set field
    *   (ssize - 1)  = 0x007f : Mask of the set field
@@ -279,6 +327,27 @@ void up_invalidate_icache_all(void)
 }
 #endif
 
+/****************************************************************************
+ * Name: up_get_dcache_linesize
+ *
+ * Description:
+ *   Get dcache linesize
+ *
+ * Input Parameters:
+ *   None
+ *
+ * Returned Value:
+ *   Cache line size
+ *
+ ****************************************************************************/
+
+#ifdef CONFIG_ARMV8M_DCACHE
+size_t up_get_dcache_linesize(void)
+{
+  return up_get_cache_linesize();
+}
+#endif
+
 /****************************************************************************
  * Name: up_enable_dcache
  *
@@ -459,20 +528,10 @@ void up_disable_dcache(void)
 #ifdef CONFIG_ARMV8M_DCACHE
 void up_invalidate_dcache(uintptr_t start, uintptr_t end)
 {
-  uint32_t ccsidr;
-  uint32_t sshift;
-  uint32_t ssize;
+  uint32_t ssize = up_get_dcache_linesize();
 
-  /* Get the characteristics of the D-Cache */
-
-  ccsidr = getreg32(NVIC_CCSIDR);
-  sshift = CCSIDR_LSSHIFT(ccsidr) + 4;   /* log2(cache-line-size-in-bytes) */
-
-  /* Invalidate the D-Cache containing this range of addresses */
-
-  ssize  = (1 << sshift);
-
-  /* Round down the start address to the nearest cache line boundary.
+  /* Invalidate the D-Cache containing this range of addresses
+   * Round down the start address to the nearest cache line boundary.
    *
    *   sshift = 5      : Offset to the beginning of the set field
    *   (ssize - 1)  = 0x007f : Mask of the set field
