@@ -134,9 +134,9 @@ int arm_svcall(int irq, void *context, void *arg)
    */
 
 #ifdef CONFIG_DEBUG_SYSCALL_INFO
-# ifndef CONFIG_DEBUG_SVCALL
+#  ifndef CONFIG_DEBUG_SVCALL
   if (cmd > SYS_switch_context)
-# endif
+#  endif
     {
       svcinfo("SVCALL Entry: regs: %p cmd: %d\n", regs, cmd);
       svcinfo("  R0: %08x %08x %08x %08x %08x %08x %08x %08x\n",
@@ -145,12 +145,12 @@ int arm_svcall(int irq, void *context, void *arg)
       svcinfo("  R8: %08x %08x %08x %08x %08x %08x %08x %08x\n",
               regs[REG_R8],  regs[REG_R9],  regs[REG_R10], regs[REG_R11],
               regs[REG_R12], regs[REG_R13], regs[REG_R14], regs[REG_R15]);
-# ifdef REG_EXC_RETURN
+#  ifdef REG_EXC_RETURN
       svcinfo(" PSR: %08x EXC_RETURN: %08x\n",
               regs[REG_XPSR], regs[REG_EXC_RETURN]);
-# else
+#  else
       svcinfo(" PSR: %08x\n", regs[REG_XPSR]);
-# endif
+#  endif
     }
 #endif
 
@@ -158,26 +158,6 @@ int arm_svcall(int irq, void *context, void *arg)
 
   switch (cmd)
     {
-      /* R0=SYS_save_context:  This is a save context command:
-       *
-       *   int up_saveusercontext(void *saveregs);
-       *
-       * At this point, the following values are saved in context:
-       *
-       *   R0 = SYS_save_context
-       *   R1 = saveregs
-       *
-       * In this case, we simply need to copy the current registers to the
-       * save register space references in the saved R1 and return.
-       */
-
-      case SYS_save_context:
-        {
-          DEBUGASSERT(regs[REG_R1] != 0);
-          memcpy((uint32_t *)regs[REG_R1], regs, XCPTCONTEXT_SIZE);
-        }
-        break;
-
       /* R0=SYS_restore_context:  This a restore context command:
        *
        *   void arm_fullcontextrestore(uint32_t *restoreregs)
@@ -457,11 +437,11 @@ int arm_svcall(int irq, void *context, void *arg)
    */
 
 #ifdef CONFIG_DEBUG_SYSCALL_INFO
-# ifndef CONFIG_DEBUG_SVCALL
+#  ifndef CONFIG_DEBUG_SVCALL
   if (cmd > SYS_switch_context)
-# else
+#  else
   if (regs != CURRENT_REGS)
-# endif
+#  endif
     {
       svcinfo("SVCall Return:\n");
       svcinfo("  R0: %08x %08x %08x %08x %08x %08x %08x %08x\n",
@@ -474,19 +454,19 @@ int arm_svcall(int irq, void *context, void *arg)
               CURRENT_REGS[REG_R10], CURRENT_REGS[REG_R11],
               CURRENT_REGS[REG_R12], CURRENT_REGS[REG_R13],
               CURRENT_REGS[REG_R14], CURRENT_REGS[REG_R15]);
-# ifdef REG_EXC_RETURN
+#  ifdef REG_EXC_RETURN
       svcinfo(" PSR: %08x EXC_RETURN: %08x\n",
               CURRENT_REGS[REG_XPSR], CURRENT_REGS[REG_EXC_RETURN]);
-# else
+#  else
       svcinfo(" PSR: %08x\n", CURRENT_REGS[REG_XPSR]);
-# endif
+#  endif
     }
-# ifdef CONFIG_DEBUG_SVCALL
+#  ifdef CONFIG_DEBUG_SVCALL
   else
     {
       svcinfo("SVCall Return: %d\n", regs[REG_R0]);
     }
-# endif
+#  endif
 #endif
 
   return OK;
