@@ -204,64 +204,72 @@ static int cisif_intc_handler(int irq, void *context, void *arg);
 
 /* video image data operations */
 
-static int cxd56_cisif_init(void);
-static int cxd56_cisif_uninit(void);
+static int cxd56_cisif_init(struct imgdata_s *data);
+static int cxd56_cisif_uninit(struct imgdata_s *data);
 static int cxd56_cisif_validate_frame_setting
-             (uint8_t nr_datafmt,
+             (struct imgdata_s *data,
+              uint8_t nr_datafmt,
               imgdata_format_t *datafmt,
               imgdata_interval_t *interval);
 static int cxd56_cisif_start_capture
-             (uint8_t nr_datafmt,
+             (struct imgdata_s *data,
+              uint8_t nr_datafmt,
               imgdata_format_t *datafmt,
               imgdata_interval_t *interval,
               imgdata_capture_t callback);
-static int cxd56_cisif_stop_capture(void);
-static int cxd56_cisif_validate_buf(uint8_t *addr, uint32_t size);
-static int cxd56_cisif_set_buf(uint8_t *addr, uint32_t size);
+static int cxd56_cisif_stop_capture(struct imgdata_s *data);
+static int cxd56_cisif_validate_buf(struct imgdata_s *data,
+                                    uint8_t *addr, uint32_t size);
+static int cxd56_cisif_set_buf(struct imgdata_s *data,
+                               uint8_t *addr, uint32_t size);
 
-const intc_func_table g_intcomp_func[] =
-  {
-    cisif_vs_int,            /* VS_INT */
-    NULL,                    /* EOY_INT */
-    NULL,                    /* SOY_INT */
-    NULL,                    /* EOI_INT */
-    NULL,                    /* SOI_INT */
-    NULL,                    /* YCC_VACT_END_INT */
-    NULL,                    /* JPG_VACT_END_INT */
-    cisif_ycc_axi_trdn_int,  /* YCC_AXI_TRDN_INT */
-    cisif_ycc_nstorage_int,  /* YCC_NSTORAGE_INT */
-    NULL,                    /* YCC_DAREA_END_INT */
-    cisif_jpg_axi_trdn_int,  /* JPG_AXI_TRDN_INT */
-    cisif_jpg_nstorage_int,  /* JPG_NSTORAGE_INT */
-    NULL,                    /* JPG_DAREA_END_INT */
-    NULL,                    /* reserve */
-    NULL,                    /* reserve */
-    NULL,                    /* VLATCH_INT */
-    cisif_ycc_err_int,       /* SIZE_OVER_INT */
-    cisif_ycc_err_int,       /* SIZE_UNDER_INT */
-    cisif_ycc_err_int,       /* YCC_MARKER_ERR_INT */
-    cisif_ycc_err_int,       /* YCC_AXI_TRERR_INT */
-    cisif_ycc_err_int,       /* YCC_FIFO_OVF_INT */
-    cisif_ycc_err_int,       /* YCC_MEM_OVF_INT */
-    NULL,                    /* reserve */
-    NULL,                    /* reserve */
-    cisif_jpg_err_int,       /* JPG_MARKER_ERR_INT */
-    cisif_jpg_err_int,       /* JPG_AXI_TRERR_INT */
-    cisif_jpg_err_int,       /* JPG_FIFO_OVF_INT */
-    cisif_jpg_err_int,       /* JPG_MEM_OVF_INT */
-    cisif_jpg_err_int,       /* JPG_ERR_STATUS_INT */
-  };
+static const intc_func_table g_intcomp_func[] =
+{
+  cisif_vs_int,            /* VS_INT */
+  NULL,                    /* EOY_INT */
+  NULL,                    /* SOY_INT */
+  NULL,                    /* EOI_INT */
+  NULL,                    /* SOI_INT */
+  NULL,                    /* YCC_VACT_END_INT */
+  NULL,                    /* JPG_VACT_END_INT */
+  cisif_ycc_axi_trdn_int,  /* YCC_AXI_TRDN_INT */
+  cisif_ycc_nstorage_int,  /* YCC_NSTORAGE_INT */
+  NULL,                    /* YCC_DAREA_END_INT */
+  cisif_jpg_axi_trdn_int,  /* JPG_AXI_TRDN_INT */
+  cisif_jpg_nstorage_int,  /* JPG_NSTORAGE_INT */
+  NULL,                    /* JPG_DAREA_END_INT */
+  NULL,                    /* reserve */
+  NULL,                    /* reserve */
+  NULL,                    /* VLATCH_INT */
+  cisif_ycc_err_int,       /* SIZE_OVER_INT */
+  cisif_ycc_err_int,       /* SIZE_UNDER_INT */
+  cisif_ycc_err_int,       /* YCC_MARKER_ERR_INT */
+  cisif_ycc_err_int,       /* YCC_AXI_TRERR_INT */
+  cisif_ycc_err_int,       /* YCC_FIFO_OVF_INT */
+  cisif_ycc_err_int,       /* YCC_MEM_OVF_INT */
+  NULL,                    /* reserve */
+  NULL,                    /* reserve */
+  cisif_jpg_err_int,       /* JPG_MARKER_ERR_INT */
+  cisif_jpg_err_int,       /* JPG_AXI_TRERR_INT */
+  cisif_jpg_err_int,       /* JPG_FIFO_OVF_INT */
+  cisif_jpg_err_int,       /* JPG_MEM_OVF_INT */
+  cisif_jpg_err_int,       /* JPG_ERR_STATUS_INT */
+};
 
-const struct imgdata_ops_s g_cxd56_cisif_ops =
-  {
-    .init                   = cxd56_cisif_init,
-    .uninit                 = cxd56_cisif_uninit,
-    .validate_buf           = cxd56_cisif_validate_buf,
-    .set_buf                = cxd56_cisif_set_buf,
-    .validate_frame_setting = cxd56_cisif_validate_frame_setting,
-    .start_capture          = cxd56_cisif_start_capture,
-    .stop_capture           = cxd56_cisif_stop_capture,
-  };
+static const struct imgdata_ops_s g_cxd56_cisif_ops =
+{
+  .init                   = cxd56_cisif_init,
+  .uninit                 = cxd56_cisif_uninit,
+  .set_buf                = cxd56_cisif_set_buf,
+  .validate_frame_setting = cxd56_cisif_validate_frame_setting,
+  .start_capture          = cxd56_cisif_start_capture,
+  .stop_capture           = cxd56_cisif_stop_capture,
+};
+
+static struct imgdata_s g_cxd56_cisif =
+{
+  &g_cxd56_cisif_ops
+};
 
 /****************************************************************************
  * Private Functions
@@ -365,7 +373,7 @@ static void cisif_callback_for_intlev(uint8_t code)
 
   /* Notify and get next addr */
 
-  g_cxd56_cisif_complete_capture(0, size);
+  g_cxd56_cisif_complete_capture(0, size, NULL);
 
   g_jpgint_receive = false;
 
@@ -405,7 +413,7 @@ static void cisif_ycc_axi_trdn_int(uint8_t code)
   else
     {
       size = cisif_reg_read(CISIF_YCC_DSTRG_CONT);
-      g_cxd56_cisif_complete_capture(0, size);
+      g_cxd56_cisif_complete_capture(0, size, NULL);
       cisif_reg_write(CISIF_YCC_DREAD_CONT, 0);
     }
 }
@@ -455,7 +463,7 @@ static void cisif_jpg_axi_trdn_int(uint8_t code)
   else
     {
       size = cisif_reg_read(CISIF_JPG_DSTRG_CONT);
-      g_cxd56_cisif_complete_capture(0, size);
+      g_cxd56_cisif_complete_capture(0, size, NULL);
       cisif_reg_write(CISIF_JPG_DREAD_CONT, 0);
     }
 }
@@ -487,7 +495,7 @@ static void cisif_ycc_err_int(uint8_t code)
 #endif
 
   size = cisif_reg_read(CISIF_YCC_DSTRG_CONT);
-  g_cxd56_cisif_complete_capture(code, size);
+  g_cxd56_cisif_complete_capture(code, size, NULL);
   cisif_reg_write(CISIF_YCC_DREAD_CONT, 0);
   g_errint_receive = true;
 }
@@ -505,7 +513,7 @@ static void cisif_jpg_err_int(uint8_t code)
 #endif
 
   size = cisif_reg_read(CISIF_JPG_DSTRG_CONT);
-  g_cxd56_cisif_complete_capture(code, size);
+  g_cxd56_cisif_complete_capture(code, size, NULL);
   cisif_reg_write(CISIF_JPG_DREAD_CONT, 0);
   g_errint_receive = true;
 }
@@ -765,7 +773,7 @@ static int cisif_chk_yuvfrmsize(int w, int h)
  * cxd56_cisif_init
  ****************************************************************************/
 
-static int cxd56_cisif_init(void)
+static int cxd56_cisif_init(struct imgdata_s *data)
 {
   if (g_state != STATE_STANDBY)
     {
@@ -803,7 +811,7 @@ static int cxd56_cisif_init(void)
  * cxd56_cisif_uninit
  ****************************************************************************/
 
-static int cxd56_cisif_uninit(void)
+static int cxd56_cisif_uninit(struct imgdata_s *data)
 {
   if (g_state != STATE_READY)
     {
@@ -838,7 +846,8 @@ static int cxd56_cisif_uninit(void)
  ****************************************************************************/
 
 static int cxd56_cisif_start_capture
-             (uint8_t nr_fmt,
+             (struct imgdata_s *data,
+              uint8_t nr_fmt,
               imgdata_format_t *fmt,
               imgdata_interval_t *interval,
               imgdata_capture_t callback)
@@ -937,7 +946,7 @@ static int cxd56_cisif_start_capture
   return OK;
 }
 
-static int cxd56_cisif_stop_capture(void)
+static int cxd56_cisif_stop_capture(struct imgdata_s *data)
 {
   g_state = STATE_READY;
   cisif_reg_write(CISIF_DIN_ENABLE, 0);
@@ -947,7 +956,8 @@ static int cxd56_cisif_stop_capture(void)
   return OK;
 }
 
-static int cxd56_cisif_validate_buf(uint8_t *addr, uint32_t size)
+static int cxd56_cisif_validate_buf(struct imgdata_s *data,
+                                    uint8_t *addr, uint32_t size)
 {
   if (ILLEGAL_BUFADDR_ALIGNMENT(addr) ||
       size == 0)
@@ -958,7 +968,8 @@ static int cxd56_cisif_validate_buf(uint8_t *addr, uint32_t size)
   return OK;
 }
 
-static int cxd56_cisif_set_buf(uint8_t *addr, uint32_t size)
+static int cxd56_cisif_set_buf(struct imgdata_s *data,
+                               uint8_t *addr, uint32_t size)
 {
   int      ret;
   uint32_t mode;
@@ -966,7 +977,7 @@ static int cxd56_cisif_set_buf(uint8_t *addr, uint32_t size)
   uint16_t w;
   uint16_t h;
 
-  ret = cxd56_cisif_validate_buf(addr, size);
+  ret = cxd56_cisif_validate_buf(data, addr, size);
   if (ret != OK)
     {
       return ret;
@@ -1010,7 +1021,8 @@ static int cxd56_cisif_set_buf(uint8_t *addr, uint32_t size)
 }
 
 static int cxd56_cisif_validate_frame_setting
-             (uint8_t nr_datafmt,
+             (struct imgdata_s *data,
+              uint8_t nr_datafmt,
               imgdata_format_t *datafmt,
               imgdata_interval_t *interval)
 {
@@ -1080,7 +1092,7 @@ static int cxd56_cisif_validate_frame_setting
 
 int cxd56_cisif_initialize(void)
 {
-  imgdata_register(&g_cxd56_cisif_ops);
+  imgdata_register(&g_cxd56_cisif);
   return OK;
 }
 
