@@ -170,6 +170,7 @@ struct cxd56_gnss_dev_s
   mutex_t                         ioctllock;
   sem_t                           apiwait;
   int                             apiret;
+  bool                            has_event;
 };
 
 /****************************************************************************
@@ -317,10 +318,9 @@ static const struct file_operations g_gnssfops =
   cxd56_gnss_write, /* write */
   NULL,             /* seek */
   cxd56_gnss_ioctl, /* ioctl */
+  NULL,             /* mmap */
+  NULL,             /* truncate */
   cxd56_gnss_poll   /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL            /* unlink */
-#endif
 };
 
 /* GNSS ioctl command list */
@@ -1578,7 +1578,7 @@ static int cxd56_gnss_set_signal(struct file *filep, unsigned long arg)
     }
 
   sig = NULL;
-  pid = getpid();
+  pid = nxsched_getpid();
   for (i = 0; i < CONFIG_CXD56_GNSS_NSIGNALRECEIVERS; i++)
     {
       checksig = &priv->sigs[i];
@@ -2179,6 +2179,23 @@ static int cxd56_gnss_get_1pps_output(struct file *filep,
  *  CXD56_GNSS signal handler and utils
  */
 
+/****************************************************************************
+ * Name: cxd56_gnss_pollnotify
+ *
+ * Description:
+ *   Notify POLLIN event for poll
+ *
+ * Input Parameters:
+ *   dev - Gnss device structure pointer
+ *
+ ****************************************************************************/
+
+static void cxd56_gnss_pollnotify(struct cxd56_gnss_dev_s *dev)
+{
+  poll_notify(dev->fds, CONFIG_CXD56_GNSS_NPOLLWAITERS, POLLIN);
+  dev->has_event = true;
+}
+
 /****************************************************************************
  * Name: cxd56_gnss_wait_notify
  *
@@ -2493,7 +2510,7 @@ static void cxd56_gnss_default_sighandler(uint32_t data, void *userdata)
       return;
     }
 
-  poll_notify(priv->fds, CONFIG_CXD56_GNSS_NPOLLWAITERS, POLLIN);
+  cxd56_gnss_pollnotify(priv);
 
   nxmutex_unlock(&priv->devlock);
 
@@ -3020,6 +3037,13 @@ static int cxd56_gnss_poll(struct file *filep, struct pollfd *fds,
           ret       = -EBUSY;
           goto errout;
         }
+
+      /* Should we immediately notify on any of the requested events? */
+
+      if (priv->has_event)
+        {
+          cxd56_gnss_pollnotify(priv);
+        }
     }
   else if (fds->priv)
     {
@@ -3031,6 +3055,7 @@ static int cxd56_gnss_poll(struct file *filep, struct pollfd *fds,
 
       *slot                = NULL;
       fds->priv            = NULL;
+      priv->has_event      = false;
     }
 
 errout:
