@@ -1202,7 +1202,11 @@ static int kinetis_lock(struct sdio_dev_s *dev, bool lock)
    * bus is part of board support package.
    */
 
-  kinetis_muxbus_sdio_lock(lock);
+  /* FIXME: Implement the below function to support bus share:
+   *
+   * kinetis_muxbus_sdio_lock(lock);
+   */
+
   return OK;
 }
 #endif
