@@ -1045,11 +1045,11 @@ struct i2s_dev_s *lc823450_i2sdev_initialize(void)
 
   /* Backup the current affinity */
 
-  nxsched_get_affinity(gettid(), sizeof(cpuset0), &cpuset0);
+  nxsched_get_affinity(nxsched_gettid(), sizeof(cpuset0), &cpuset0);
 
   /* Set the new affinity which assigns to CPU0 */
 
-  nxsched_set_affinity(gettid(), sizeof(cpuset1), &cpuset1);
+  nxsched_set_affinity(nxsched_gettid(), sizeof(cpuset1), &cpuset1);
   nxsig_usleep(10 * 1000);
 #endif
 
@@ -1062,7 +1062,7 @@ struct i2s_dev_s *lc823450_i2sdev_initialize(void)
 #ifdef CONFIG_SMP
   /* Restore the original affinity */
 
-  nxsched_set_affinity(gettid(), sizeof(cpuset0), &cpuset0);
+  nxsched_set_affinity(nxsched_gettid(), sizeof(cpuset0), &cpuset0);
   nxsig_usleep(10 * 1000);
 #endif
 
