@@ -24,6 +24,7 @@
 
 #include "amebaz_depend.h"
 #include <nuttx/mqueue.h>
+#include <nuttx/semaphore.h>
 #include <nuttx/syslog/syslog.h>
 
 /****************************************************************************
@@ -168,7 +169,7 @@ void rtw_init_sema(void **sema, int init_val)
       return;
     }
 
-  if (sem_init(_sema, 0, init_val))
+  if (nxsem_init(_sema, 0, init_val))
     {
       free(_sema);
       return;
@@ -179,7 +180,7 @@ void rtw_init_sema(void **sema, int init_val)
 
 void rtw_free_sema(void **sema)
 {
-  sem_destroy(*sema);
+  nxsem_destroy(*sema);
   free(*sema);
   *sema = NULL;
 }
