@@ -189,7 +189,7 @@
                  DMA_CCR_PSIZE_8BITS   | \
                  DMA_CCR_MSIZE_8BITS   | \
                  CONFIG_STM32_HCIUART_RXDMAPRIO)
-# endif
+#  endif
 #endif
 
 /* All interrupts */
@@ -352,9 +352,9 @@ static int  hciuart_pm_prepare(struct pm_callback_s *cb, int domain,
 
 static uint8_t g_usart1_rxbuffer[CONFIG_STM32_HCIUART1_RXBUFSIZE];
 static uint8_t g_usart1_txbuffer[CONFIG_STM32_HCIUART1_TXBUFSIZE];
-# ifdef CONFIG_STM32_HCIUART1_RXDMA
+#  ifdef CONFIG_STM32_HCIUART1_RXDMA
 static uint8_t g_usart1_rxdmabuffer[RXDMA_BUFFER_SIZE];
-# endif
+#  endif
 
 /* HCI USART1 variable state information */
 
@@ -381,26 +381,26 @@ static const struct hciuart_config_s g_hciusart1_config =
 
   .rxbuffer      = g_usart1_rxbuffer,
   .txbuffer      = g_usart1_txbuffer,
-#ifdef CONFIG_STM32_HCIUART1_RXDMA
+#  ifdef CONFIG_STM32_HCIUART1_RXDMA
   .rxdmabuffer   = g_usart1_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART1_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART1_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART1_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART1_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_USART1_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_USART1,
   .baud          = CONFIG_STM32_HCIUART1_BAUD,
-#if defined(CONFIG_STM32_STM32F33XX)
+#  if defined(CONFIG_STM32_STM32F33XX)
   .apbclock      = STM32_PCLK1_FREQUENCY, /* Errata 2.5.1 */
-#else
+#  else
   .apbclock      = STM32_PCLK2_FREQUENCY,
-#endif
+#  endif
   .usartbase     = STM32_USART1_BASE,
   .tx_gpio       = GPIO_USART1_TX,
   .rx_gpio       = GPIO_USART1_RX,
@@ -416,9 +416,9 @@ static const struct hciuart_config_s g_hciusart1_config =
 
 static uint8_t g_usart2_rxbuffer[CONFIG_STM32_HCIUART2_RXBUFSIZE];
 static uint8_t g_usart2_txbuffer[CONFIG_STM32_HCIUART2_TXBUFSIZE];
-# ifdef CONFIG_STM32_HCIUART2_RXDMA
+#  ifdef CONFIG_STM32_HCIUART2_RXDMA
 static uint8_t g_usart2_rxdmabuffer[RXDMA_BUFFER_SIZE];
-# endif
+#  endif
 
 /* HCI USART2 variable state information */
 
@@ -445,18 +445,18 @@ static const struct hciuart_config_s g_hciusart2_config =
 
   .rxbuffer      = g_usart2_rxbuffer,
   .txbuffer      = g_usart2_txbuffer,
-#ifdef CONFIG_STM32_HCIUART2_RXDMA
+#  ifdef CONFIG_STM32_HCIUART2_RXDMA
   .rxdmabuffer   = g_usart2_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART2_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART2_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART2_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART2_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_USART2_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_USART2,
   .baud          = CONFIG_STM32_HCIUART2_BAUD,
@@ -476,9 +476,9 @@ static const struct hciuart_config_s g_hciusart2_config =
 
 static uint8_t g_usart3_rxbuffer[CONFIG_STM32_HCIUART3_RXBUFSIZE];
 static uint8_t g_usart3_txbuffer[CONFIG_STM32_HCIUART3_TXBUFSIZE];
-#ifdef CONFIG_STM32_HCIUART3_RXDMA
+#  ifdef CONFIG_STM32_HCIUART3_RXDMA
 static uint8_t g_usart3_rxdmabuffer[RXDMA_BUFFER_SIZE];
-#endif
+#  endif
 
 /* HCI USART3 variable state information */
 
@@ -505,18 +505,18 @@ static const struct hciuart_config_s g_hciusart3_config =
 
   .rxbuffer      = g_usart3_rxbuffer,
   .txbuffer      = g_usart3_txbuffer,
-#ifdef CONFIG_STM32_HCIUART3_RXDMA
+#  ifdef CONFIG_STM32_HCIUART3_RXDMA
   .rxdmabuffer   = g_usart3_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART3_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART3_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART3_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART3_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_USART3_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_USART3,
   .baud          = CONFIG_STM32_HCIUART3_BAUD,
@@ -534,13 +534,11 @@ static const struct hciuart_config_s g_hciusart3_config =
 #ifdef CONFIG_STM32_USART6_HCIUART
 /* I/O buffers */
 
-#ifdef CONFIG_STM32_USART6_HCIUART
 static uint8_t g_usart6_rxbuffer[CONFIG_STM32_HCIUART6_RXBUFSIZE];
 static uint8_t g_usart6_txbuffer[CONFIG_STM32_HCIUART6_TXBUFSIZE];
-# ifdef CONFIG_STM32_HCIUART6_RXDMA
+#  ifdef CONFIG_STM32_HCIUART6_RXDMA
 static uint8_t g_usart6_rxdmabuffer[RXDMA_BUFFER_SIZE];
-# endif
-#endif
+#  endif
 
 /* HCI USART6 variable state information */
 
@@ -567,18 +565,18 @@ static const struct hciuart_config_s g_hciusart6_config =
 
   .rxbuffer      = g_usart6_rxbuffer,
   .txbuffer      = g_usart6_txbuffer,
-#ifdef CONFIG_STM32_HCIUART6_RXDMA
+#  ifdef CONFIG_STM32_HCIUART6_RXDMA
   .rxdmabuffer   = g_usart6_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART6_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART6_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART6_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART6_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_USART6_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_USART6,
   .baud          = CONFIG_STM32_HCIUART6_BAUD,
@@ -598,9 +596,9 @@ static const struct hciuart_config_s g_hciusart6_config =
 
 static uint8_t g_uart7_rxbuffer[CONFIG_STM32_HCIUART7_RXBUFSIZE];
 static uint8_t g_uart7_txbuffer[CONFIG_STM32_HCIUART7_TXBUFSIZE];
-#ifdef CONFIG_STM32_HCIUART7_RXDMA
+#  ifdef CONFIG_STM32_HCIUART7_RXDMA
 static uint8_t g_uart7_rxdmabuffer[RXDMA_BUFFER_SIZE];
-#endif
+#  endif
 
 /* HCI UART7 variable state information */
 
@@ -627,18 +625,18 @@ static const struct hciuart_config_s g_hciuart7_config =
 
   .rxbuffer      = g_uart7_rxbuffer,
   .txbuffer      = g_uart7_txbuffer,
-#ifdef CONFIG_STM32_HCIUART7_RXDMA
+#  ifdef CONFIG_STM32_HCIUART7_RXDMA
   .rxdmabuffer   = g_uart7_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART7_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART7_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART7_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART7_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_UART7_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_UART7,
   .baud          = CONFIG_STM32_HCIUART7_BAUD,
@@ -658,9 +656,9 @@ static const struct hciuart_config_s g_hciuart7_config =
 
 static uint8_t g_uart8_rxbuffer[CONFIG_STM32_HCIUART8_RXBUFSIZE];
 static uint8_t g_uart8_txbuffer[CONFIG_STM32_HCIUART8_TXBUFSIZE];
-#ifdef CONFIG_STM32_HCIUART8_RXDMA
+#  ifdef CONFIG_STM32_HCIUART8_RXDMA
 static uint8_t g_uart8_rxdmabuffer[RXDMA_BUFFER_SIZE];
-#endif
+#  endif
 
 /* HCI UART8 variable state information */
 
@@ -687,18 +685,18 @@ static const struct hciuart_config_s g_hciuart8_config =
 
   .rxbuffer      = g_uart8_rxbuffer,
   .txbuffer      = g_uart8_txbuffer,
-#ifdef CONFIG_STM32_HCIUART8_RXDMA
+#  ifdef CONFIG_STM32_HCIUART8_RXDMA
   .rxdmabuffer   = g_uart8_rxdmabuffer,
-#endif
+#  endif
   .rxbufsize     = CONFIG_STM32_HCIUART8_RXBUFSIZE,
   .txbufsize     = CONFIG_STM32_HCIUART8_TXBUFSIZE,
-#ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
+#  ifdef CONFIG_STM32_HCIUART_SW_RXFLOW
   .rxupper       = RXFLOW_UPPER(CONFIG_STM32_HCIUART8_RXBUFSIZE),
   .rxlower       = RXFLOW_LOWER(CONFIG_STM32_HCIUART8_RXBUFSIZE),
-#endif
-#ifdef CONFIG_STM32_HCIUART_RXDMA
+#  endif
+#  ifdef CONFIG_STM32_HCIUART_RXDMA
   .rxdmachan     = DMAMAP_UART8_RX,
-#endif
+#  endif
 
   .irq           = STM32_IRQ_UART8,
   .baud          = CONFIG_STM32_HCIUART8_BAUD,
