@@ -183,6 +183,10 @@ void __start(void)
 
   showprogress('C');
 
+#ifdef CONFIG_SCHED_IRQMONITOR
+  up_perf_init((void *)STM32_SYSCLK_FREQUENCY);
+#endif
+
   /* Perform early serial initialization */
 
 #ifdef USE_EARLYSERIALINIT
