@@ -178,11 +178,6 @@ void arm64_chip_boot(void)
 {
   /* MAP IO and DRAM, enable MMU. */
 
-  uint64_t cpumpid;
-  cpumpid = read_sysreg(mpidr_el1);
-
-  sinfo("Main CPU 0x%-16"PRIx64"", cpumpid);
-
   arm64_mpu_init(true);
 
 #if defined(CONFIG_SMP) && defined(CONFIG_ARCH_HAVE_PCSI)
