@@ -66,11 +66,8 @@
 
 int up_shmat(uintptr_t *pages, unsigned int npages, uintptr_t vaddr)
 {
-  struct tcb_s          *tcb = nxsched_self();
-  struct arch_addrenv_s *addrenv;
-  uintptr_t              ptlast;
-  uintptr_t              ptlevel;
-  uintptr_t              paddr;
+  struct tcb_s          *tcb     = nxsched_self();
+  struct arch_addrenv_s *addrenv = &tcb->addrenv_own->addrenv;
 
   /* Sanity checks */
 
@@ -79,33 +76,9 @@ int up_shmat(uintptr_t *pages, unsigned int npages, uintptr_t vaddr)
   DEBUGASSERT(vaddr >= CONFIG_ARCH_SHM_VBASE && vaddr < ARCH_SHM_VEND);
   DEBUGASSERT(MM_ISALIGNED(vaddr));
 
-  addrenv = &tcb->addrenv_own->addrenv;
-  ptlevel =  RV_MMU_PT_LEVELS;
+  /* Let riscv_map_pages do the work */
 
-  /* Add the references to pages[] into the caller's address environment */
-
-  for (; npages > 0; npages--)
-    {
-      /* Get the address of the last level page table */
-
-      ptlast = riscv_pgvaddr(riscv_get_pgtable(addrenv, vaddr));
-      if (!ptlast)
-        {
-          return -ENOMEM;
-        }
-
-      /* Then add the reference */
-
-      paddr = *pages++;
-      mmu_ln_setentry(ptlevel, ptlast, paddr, vaddr, MMU_UDATA_FLAGS);
-      vaddr += MM_PGSIZE;
-    }
-
-  /* Flush the data cache, so the changes are committed to memory */
-
-  __DMB();
-
-  return OK;
+  return riscv_map_pages(addrenv, pages, npages, vaddr, MMU_UDATA_FLAGS);
 }
 
 /****************************************************************************
@@ -127,12 +100,8 @@ int up_shmat(uintptr_t *pages, unsigned int npages, uintptr_t vaddr)
 
 int up_shmdt(uintptr_t vaddr, unsigned int npages)
 {
-  struct tcb_s          *tcb = nxsched_self();
-  struct arch_addrenv_s *addrenv;
-  uintptr_t              ptlast;
-  uintptr_t              ptprev;
-  uintptr_t              ptlevel;
-  uintptr_t              paddr;
+  struct tcb_s          *tcb     = nxsched_self();
+  struct arch_addrenv_s *addrenv = &tcb->addrenv_own->addrenv;
 
   /* Sanity checks */
 
@@ -141,40 +110,9 @@ int up_shmdt(uintptr_t vaddr, unsigned int npages)
   DEBUGASSERT(vaddr >= CONFIG_ARCH_SHM_VBASE && vaddr < ARCH_SHM_VEND);
   DEBUGASSERT(MM_ISALIGNED(vaddr));
 
-  addrenv = &tcb->addrenv_own->addrenv;
-  ptlevel =  ARCH_SPGTS;
-  ptprev  =  riscv_pgvaddr(addrenv->spgtables[ARCH_SPGTS - 1]);
-  if (!ptprev)
-    {
-      /* Something is very wrong */
+  /* Let riscv_unmap_pages do the work */
 
-      return -EFAULT;
-    }
-
-  /* Remove the references from the caller's address environment */
-
-  for (; npages > 0; npages--)
-    {
-      /* Get the current final level entry corresponding to this vaddr */
-
-      paddr = mmu_pte_to_paddr(mmu_ln_getentry(ptlevel, ptprev, vaddr));
-      ptlast = riscv_pgvaddr(paddr);
-      if (!ptlast)
-        {
-          return -EFAULT;
-        }
-
-      /* Then wipe the reference */
-
-      mmu_ln_clear(ptlevel + 1, ptlast, vaddr);
-      vaddr += MM_PGSIZE;
-    }
-
-  /* Flush the data cache, so the changes are committed to memory */
-
-  __DMB();
-
-  return OK;
+  return riscv_unmap_pages(addrenv, vaddr, npages);
 }
 
 #endif /* CONFIG_BUILD_KERNEL */
