@@ -159,22 +159,21 @@ size_t up_check_tcbstack(struct tcb_s *tcb)
   size_t size;
 
 #ifdef CONFIG_ARCH_ADDRENV
-  bool saved = false;
+  FAR struct addrenv_s *oldenv;
 
   if (tcb->addrenv_own != NULL)
     {
-      addrenv_select(tcb->addrenv_own);
-      saved = true;
+      addrenv_select(tcb->addrenv_own, &oldenv);
     }
 #endif
 
   size = riscv_stack_check((uintptr_t)tcb->stack_base_ptr,
-                           tcb->adj_stack_size);
+                                      tcb->adj_stack_size);
 
 #ifdef CONFIG_ARCH_ADDRENV
-  if (saved)
+  if (tcb->addrenv_own != NULL)
     {
-      addrenv_restore();
+      addrenv_restore(oldenv);
     }
 #endif
 
