@@ -1180,7 +1180,10 @@ static int mpfs_lock(struct sdio_dev_s *dev, bool lock)
 {
   /* The multiplex bus is part of board support package. */
 
-  mpfs_muxbus_sdio_lock(dev, lock);
+  /* FIXME: Implement the below function to support bus share:
+   *
+   * mpfs_muxbus_sdio_lock(dev, lock);
+   */
 
   return OK;
 }
