@@ -37,9 +37,10 @@
  * Name: up_getusrsp
  ****************************************************************************/
 
-uintptr_t up_getusrsp(void)
+uintptr_t up_getusrsp(void *regs)
 {
-  return g_current_regs[REG_ESP];
+  uint32_t *ptr = regs;
+  return ptr[REG_ESP];
 }
 
 /****************************************************************************
