@@ -42,10 +42,13 @@
 #ifdef CONFIG_BUILD_PROTECTED
 #  include "esp32s3_userspace.h"
 #endif
+#include "esp32s3_spi_timing.h"
 #include "hardware/esp32s3_cache_memory.h"
 #include "hardware/esp32s3_system.h"
 #include "hardware/esp32s3_extmem.h"
 #include "rom/esp32s3_libc_stubs.h"
+#include "rom/esp32s3_spiflash.h"
+#include "rom/esp32s3_opi_flash.h"
 
 /****************************************************************************
  * Pre-processor Definitions
@@ -336,6 +339,12 @@ void noreturn_function IRAM_ATTR __esp32s3_start(void)
 
   showprogress('A');
 
+#if defined(CONFIG_ESP32S3_FLASH_MODE_OCT) || \
+    defined(CONFIG_ESP32S3_SPIRAM_MODE_OCT)
+  esp_rom_opiflash_pin_config();
+  esp32s3_spi_timing_set_pin_drive_strength();
+#endif
+
 #if defined(CONFIG_ESP32S3_SPIRAM_BOOT_INIT)
   if (esp_spiram_init() != OK)
     {
