@@ -38,6 +38,7 @@
 
 #include <arch/board/board.h>
 #include <arch/board/cxd56_alt1250.h>
+#include <arch/chip/pm.h>
 
 #include "cxd56_spi.h"
 #include "cxd56_dmac.h"
@@ -79,6 +80,8 @@
 
 static struct spi_dev_s *alt1250_poweron(void);
 static void alt1250_poweroff(void);
+static bool alt1250_powerstatus(void);
+static int  alt1250_hibernation_mode(bool enable);
 static void alt1250_reset(void);
 static void alt1250_irqattach(xcpt_t handler);
 static void alt1250_irqenable(bool enable);
@@ -95,6 +98,8 @@ static const struct alt1250_lower_s g_alt1250_lower =
 {
   .poweron      = alt1250_poweron,
   .poweroff     = alt1250_poweroff,
+  .powerstatus  = alt1250_powerstatus,
+  .hiber_mode   = alt1250_hibernation_mode,
   .reset        = alt1250_reset,
   .irqattach    = alt1250_irqattach,
   .irqenable    = alt1250_irqenable,
@@ -292,6 +297,43 @@ static void alt1250_poweroff(void)
   board_alt1250_poweroff();
 }
 
+/****************************************************************************
+ * Name: alt1250_powerstatus
+ *
+ * Description:
+ *   Get the power status for the Altair modem device on the board.
+ *
+ ****************************************************************************/
+
+static bool alt1250_powerstatus(void)
+{
+  return board_alt1250_powerstatus();
+}
+
+/****************************************************************************
+ * Name: alt1250_hibernation_mode
+ *
+ * Description:
+ *   Set power manager for entering hibernation mode.
+ *
+ ****************************************************************************/
+
+static int alt1250_hibernation_mode(bool enable)
+{
+  uint32_t bootmask = 0;
+
+  if (enable)
+    {
+      /* Set GPIO interrupt for wake-up */
+
+      bootmask = up_pm_get_bootmask();
+      bootmask |= PM_BOOT_COLD_GPIO;
+      up_pm_set_bootmask(bootmask);
+    }
+
+  return board_alt1250_powerkeep(enable);
+}
+
 /****************************************************************************
  * Name: alt1250_reset
  *
