@@ -177,9 +177,6 @@ static const struct file_operations g_bmp280pressfops =
   bmp280_write,                 /* write */
   NULL,                         /* seek */
   bmp280_ioctl_press            /* ioctl */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL                        /* unlink */
-#endif
 };
 
 static const struct file_operations g_bmp280tempfops =
@@ -190,9 +187,6 @@ static const struct file_operations g_bmp280tempfops =
   bmp280_write,                 /* write */
   NULL,                         /* seek */
   bmp280_ioctl_temp             /* ioctl */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL                        /* unlink */
-#endif
 };
 
 /* SCU instructions for pick pressure sensing data. */
