@@ -45,6 +45,14 @@
 #  include <nuttx/usb/rndis.h>
 #endif
 
+#ifdef CONFIG_TIMER
+#  include "nrf52_timer.h"
+#endif
+
+#ifdef CONFIG_NRF52_PROGMEM
+#  include "nrf52_progmem.h"
+#endif
+
 #ifdef CONFIG_NRF52_SOFTDEVICE_CONTROLLER
 #  include "nrf52_sdc.h"
 #endif
@@ -261,15 +269,32 @@ int nrf52_bringup(void)
   usbdev_rndis_initialize(mac);
 #endif
 
+#ifdef CONFIG_NRF52_QSPI
+  /* Initialize the MX25 QSPU memory */
+
+  ret = nrf52_mx25_initialize();
+  if (ret < 0)
+    {
+      syslog(LOG_ERR, "ERROR: nrf52_mx25_initialize() failed: %d\n", ret);
+    }
+#endif
+
 #ifdef CONFIG_NRF52_SOFTDEVICE_CONTROLLER
   ret = nrf52_sdc_initialize();
-
   if (ret < 0)
     {
       syslog(LOG_ERR, "ERROR: nrf52_sdc_initialize() failed: %d\n", ret);
     }
 #endif
 
+#ifdef CONFIG_NRF52_PROGMEM
+  ret = nrf52_progmem_init();
+  if (ret < 0)
+    {
+      syslog(LOG_ERR, "ERROR: Failed to initialize MTD progmem: %d\n", ret);
+    }
+#endif /* CONFIG_MTD */
+
   UNUSED(ret);
   return OK;
 }
