@@ -28,6 +28,7 @@
 
 #include <nuttx/config.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -68,16 +69,6 @@
 #  error CONFIG_SAM34_RC32K be selected in the board configuration
 #endif
 
-/* The ever-present MIN/MAX macros ******************************************/
-
-#ifndef MIN
-#  define MIN(a,b) (a < b ? a : b)
-#endif
-
-#ifndef MAX
-#  define MAX(a,b) (a > b ? a : b)
-#endif
-
 /* LCD **********************************************************************/
 
 /* LCD characteristics.  The logic in this driver is not portable; it is
@@ -296,10 +287,9 @@ static const struct file_operations g_slcdops =
   slcd_write,    /* write */
   NULL,          /* seek */
   slcd_ioctl,    /* ioctl */
+  NULL,          /* mmap */
+  NULL,          /* truncate */
   slcd_poll      /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL         /* unlink */
-#endif
 };
 
 /* LCD state data */
