@@ -226,6 +226,8 @@ struct mcp2515_can_s
 
 /* MCP2515 Register access */
 
+static void mcp2515_config_spi(FAR struct mcp2515_can_s *priv);
+
 static void mcp2515_readregs(FAR struct mcp2515_can_s *priv, uint8_t regaddr,
               FAR uint8_t *buffer, uint8_t len);
 static void mcp2515_writeregs(FAR struct mcp2515_can_s *priv,
@@ -302,6 +304,16 @@ static const struct can_ops_s g_mcp2515ops =
  * Private Functions
  ****************************************************************************/
 
+static void mcp2515_config_spi(FAR struct mcp2515_can_s *priv)
+{
+  /* Setup SPI frequency and mode */
+
+  SPI_SETFREQUENCY(priv->config->spi, CONFIG_MCP2515_SPI_SCK_FREQUENCY);
+  SPI_SETMODE(priv->config->spi, MCP2515_SPI_MODE);
+  SPI_SETBITS(priv->config->spi, 8);
+  SPI_HWFEATURES(priv->config->spi, 0);
+}
+
 static void mcp2515_read_2regs(FAR struct mcp2515_can_s *priv, uint8_t reg,
                                FAR uint8_t *v1, FAR uint8_t *v2)
 {
@@ -309,9 +321,10 @@ static void mcp2515_read_2regs(FAR struct mcp2515_can_s *priv, uint8_t reg,
   priv->spi_txbuf[1] = reg;
 
   SPI_LOCK(priv->config->spi, true);
-  SPI_SELECT(priv->config->spi, SPIDEV_CANBUS(0), true);
+  mcp2515_config_spi(priv);
+  SPI_SELECT(priv->config->spi, SPIDEV_CANBUS(priv->config->devid), true);
   SPI_EXCHANGE(priv->config->spi, priv->spi_txbuf, priv->spi_rxbuf, 4);
-  SPI_SELECT(priv->config->spi, SPIDEV_CANBUS(0), false);
+  SPI_SELECT(priv->config->spi, SPIDEV_CANBUS(priv->config->devid), false);
   SPI_LOCK(priv->config->spi, false);
 
   *v1 = priv->spi_rxbuf[2];
@@ -342,9 +355,11 @@ static void mcp2515_readregs(FAR struct mcp2515_can_s *priv, uint8_t regaddr,
 
   SPI_LOCK(config->spi, true);
 
+  mcp2515_config_spi(priv);
+
   /* Select the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), true);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), true);
 
   /* Send the READ command */
 
@@ -357,7 +372,7 @@ static void mcp2515_readregs(FAR struct mcp2515_can_s *priv, uint8_t regaddr,
 
   /* Deselect the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), false);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), false);
 
   /* Unlock bus */
 
@@ -377,9 +392,11 @@ static void mcp2515_transfer(FAR struct mcp2515_can_s *priv, uint8_t len)
 
   SPI_LOCK(config->spi, true);
 
+  mcp2515_config_spi(priv);
+
   /* Select the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), true);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), true);
 
   /* Send the READ command */
 
@@ -387,7 +404,7 @@ static void mcp2515_transfer(FAR struct mcp2515_can_s *priv, uint8_t len)
 
   /* Deselect the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), false);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), false);
 
   /* Unlock bus */
 
@@ -426,9 +443,11 @@ static void mcp2515_writeregs(FAR struct mcp2515_can_s *priv,
 
   SPI_LOCK(config->spi, true);
 
+  mcp2515_config_spi(priv);
+
   /* Select the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), true);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), true);
 
   /* Send the READ command */
 
@@ -441,7 +460,7 @@ static void mcp2515_writeregs(FAR struct mcp2515_can_s *priv,
 
   /* Deselect the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), false);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), false);
 
   /* Unlock bus */
 
@@ -475,15 +494,17 @@ static void mcp2515_modifyreg(FAR struct mcp2515_can_s *priv,
 
   SPI_LOCK(config->spi, true);
 
+  mcp2515_config_spi(priv);
+
   /* Select the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), true);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), true);
 
   SPI_SNDBLOCK(config->spi, wr, 4);
 
   /* Deselect the MCP2515 */
 
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), false);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), false);
 
   /* Unlock bus */
 
@@ -1175,8 +1196,10 @@ static void mcp2515_reset_lowlevel(FAR struct mcp2515_can_s *priv)
   /* Send SPI reset command to MCP2515 */
 
   SPI_LOCK(config->spi, true);
-  SPI_SELECT(config->spi, SPIDEV_CANBUS(0), true);
+  mcp2515_config_spi(priv);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), true);
   SPI_SEND(config->spi, MCP2515_RESET);
+  SPI_SELECT(config->spi, SPIDEV_CANBUS(config->devid), false);
   SPI_LOCK(config->spi, false);
 
   /* Wait 1ms to let MCP2515 restart */
@@ -2498,13 +2521,6 @@ FAR struct mcp2515_can_s *
       return NULL;
     }
 
-  /* Setup SPI frequency and mode */
-
-  SPI_SETFREQUENCY(config->spi, CONFIG_MCP2515_SPI_SCK_FREQUENCY);
-  SPI_SETMODE(config->spi, MCP2515_SPI_MODE);
-  SPI_SETBITS(config->spi, 8);
-  SPI_HWFEATURES(config->spi, 0);
-
   /* Perform one time data initialization */
 
   memset(priv, 0, sizeof(struct mcp2515_can_s));
