@@ -26,6 +26,8 @@
 
 #include <string.h>
 
+#include <sys/param.h>
+
 #include <nuttx/clk/clk.h>
 #include <nuttx/clk/clk_provider.h>
 #include <nuttx/kmalloc.h>
@@ -48,10 +50,6 @@
 #define CLK_RPMSG_ROUNDRATE         6
 #define CLK_RPMSG_ISENABLED         7
 
-#ifndef ARRAY_SIZE
-#  define ARRAY_SIZE(x)             (sizeof(x) / sizeof((x)[0]))
-#endif
-
 /****************************************************************************
  * Private Types
  ****************************************************************************/
@@ -606,7 +604,7 @@ static int clk_rpmsg_ept_cb(FAR struct rpmsg_endpoint *ept, FAR void *data,
           ret = 0;
         }
     }
-  else if (cmd < ARRAY_SIZE(g_clk_rpmsg_handler)
+  else if (cmd < nitems(g_clk_rpmsg_handler)
            && g_clk_rpmsg_handler[cmd])
     {
       hdr->response = 1;
@@ -640,7 +638,7 @@ static int clk_rpmsg_enable(FAR struct clk_s *clk)
 
   DEBUGASSERT(len <= size);
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   return clk_rpmsg_sendrecv(ept, CLK_RPMSG_ENABLE,
                            (struct clk_rpmsg_header_s *)msg,
@@ -671,7 +669,7 @@ static void clk_rpmsg_disable(FAR struct clk_s *clk)
 
   DEBUGASSERT(len <= size);
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   clk_rpmsg_sendrecv(ept, CLK_RPMSG_DISABLE,
                     (struct clk_rpmsg_header_s *)msg, len);
@@ -701,7 +699,7 @@ static int clk_rpmsg_is_enabled(FAR struct clk_s *clk)
 
   DEBUGASSERT(len <= size);
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   return clk_rpmsg_sendrecv(ept, CLK_RPMSG_ISENABLED,
                            (struct clk_rpmsg_header_s *)msg, len);
@@ -734,7 +732,7 @@ static uint32_t clk_rpmsg_round_rate(FAR struct clk_s *clk, uint32_t rate,
   DEBUGASSERT(len <= size);
 
   msg->rate = rate;
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   ret = clk_rpmsg_sendrecv(ept, CLK_RPMSG_ROUNDRATE,
                           (struct clk_rpmsg_header_s *)msg, len);
@@ -772,7 +770,7 @@ static int clk_rpmsg_set_rate(FAR struct clk_s *clk, uint32_t rate,
   DEBUGASSERT(len <= size);
 
   msg->rate = rate;
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   return clk_rpmsg_sendrecv(ept, CLK_RPMSG_SETRATE,
                            (struct clk_rpmsg_header_s *)msg, len);
@@ -804,7 +802,7 @@ static uint32_t clk_rpmsg_recalc_rate(FAR struct clk_s *clk,
 
   DEBUGASSERT(len <= size);
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   ret = clk_rpmsg_sendrecv(ept, CLK_RPMSG_GETRATE,
                           (struct clk_rpmsg_header_s *)msg, len);
@@ -840,7 +838,7 @@ static int clk_rpmsg_get_phase(FAR struct clk_s *clk)
 
   DEBUGASSERT(len <= size);
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   return clk_rpmsg_sendrecv(ept, CLK_RPMSG_GETPHASE,
                            (struct clk_rpmsg_header_s *)msg, len);
@@ -871,7 +869,7 @@ static int clk_rpmsg_set_phase(FAR struct clk_s *clk, int degrees)
   DEBUGASSERT(len <= size);
 
   msg->degrees = degrees;
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, size - sizeof(*msg));
 
   return clk_rpmsg_sendrecv(ept, CLK_RPMSG_SETPHASE,
                            (struct clk_rpmsg_header_s *)msg, len);
