@@ -255,10 +255,9 @@ static const struct file_operations g_hidkbd_fops =
   spq10kbd_write,            /* write */
   NULL,                      /* seek */
   NULL,                      /* ioctl */
+  NULL,                      /* mmap */
+  NULL,                      /* truncate */
   spq10kbd_poll              /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL                     /* unlink */
-#endif
 };
 
 /****************************************************************************
