@@ -26,6 +26,7 @@
 
 #include <nuttx/config.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -44,10 +45,6 @@
  * Pre-processor Definitions
  ****************************************************************************/
 
-#ifndef MAX
-#  define MAX(a,b)    ((a) > (b) ? (a) : (b))
-#endif
-
 /* Configuration ************************************************************/
 
 /* APA102 Configuration Settings:
