@@ -73,10 +73,6 @@ static const struct file_operations g_ncp5623c_fileops =
   ncp5623c_write,              /* write */
   NULL,                        /* seek */
   ncp5623c_ioctl,              /* ioctl */
-  NULL                         /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL                      /* unlink */
-#endif
 };
 
 /****************************************************************************
