@@ -32,6 +32,32 @@
  * Pre-processor Definitions
  ****************************************************************************/
 
+/* CMD6 (MMC_SWITCH) argument
+ * MMC_SWITCH argument format:
+ *
+ *  [31:26] Always 0
+ *  [25:24] Access Mode
+ *  [23:16] Location of target Byte in EXT_CSD
+ *  [15:08] Value Byte
+ *  [07:03] Always 0
+ *  [02:00] Command Set
+ */
+#define MMCSD_CMD6_BUSWIDTH_RWSHIFT   (16)
+#  define MMCSD_CMD6_BUSWIDTH_RW      ((uint32_t)0xb7 << MMCSD_CMD6_BUSWIDTH_RWSHIFT)  /* R/W */
+
+#define MMCSD_CMD6_WRITE_BYTE_SHIFT   (24)
+#  define MMCSD_CMD6_MODE_CMD_SET     ((uint32_t)0x00 << MMCSD_CMD6_WRITE_BYTE_SHIFT)  /* Change the command set */
+#  define MMCSD_CMD6_MODE_SET_BITS    ((uint32_t)0x01 << MMCSD_CMD6_WRITE_BYTE_SHIFT)  /* Set bits which are 1 in value */
+#  define MMCSD_CMD6_MODE_CLEAR_BITS  ((uint32_t)0x02 << MMCSD_CMD6_WRITE_BYTE_SHIFT)  /* Clear bits which are 1 in value */
+#  define MMCSD_CMD6_MODE_WRITE_BYTE  ((uint32_t)0x03 << MMCSD_CMD6_WRITE_BYTE_SHIFT)  /* Set target to value */
+
+#define MMCSD_CMD6_BUS_WIDTH_SHIFT    (8)
+#  define MMCSD_CMD6_BUS_WIDTH_1      ((uint32_t)0x00 << MMCSD_CMD6_BUS_WIDTH_SHIFT)  /* Card is in 1 bit mode */
+#  define MMCSD_CMD6_BUS_WIDTH_4      ((uint32_t)0x01 << MMCSD_CMD6_BUS_WIDTH_SHIFT)  /* Card is in 4 bit mode */
+#  define MMCSD_CMD6_CSD_BUS_WIDTH_8  ((uint32_t)0x02 << MMCSD_CMD6_BUS_WIDTH_SHIFT)  /* Card is in 8 bit mode */
+#  define MMCSD_CMD6_DDR_BUS_WIDTH_4  ((uint32_t)0x05 << MMCSD_CMD6_BUS_WIDTH_SHIFT)  /* Card is in 4 bit DDR mode */
+#  define MMCSD_CMD6_DDR_BUS_WIDTH_8  ((uint32_t)0x06 << MMCSD_CMD6_BUS_WIDTH_SHIFT)  /* Card is in 8 bit DDR mode */
+
 /* CMD8 Argument:
  *    [31:12]: Reserved (shall be set to '0')
  *    [11:8]: Supply Voltage (VHS) 0x1 (Range: 2.7-3.6 V)
