@@ -30,43 +30,23 @@
 
 #include <stdbool.h>
 #include <stdint.h>
+#include <nuttx/wireless/lte/lte.h>
 
 /****************************************************************************
  * Pre-processor Definitions
  ****************************************************************************/
 
-#define APICMD_ACTIVATEPDN_APNNAME_MAXLEN       (101)
-#define APICMD_ACTIVATEPDN_USERNAME_MAXLEN      (64)
-#define APICMD_ACTIVATEPDN_PASSWORD_MAXLEN      (32)
-
-#define APICMD_CELLINFO_CELLID_MIN              (0)
-#define APICMD_CELLINFO_CELLID_MAX              (503)
-#define APICMD_CELLINFO_EARFCN_MIN              (0)
-#define APICMD_CELLINFO_EARFCN_MAX              (262143)
-#define APICMD_CELLINFO_DIGIT_NUM_MIN           (0)
-#define APICMD_CELLINFO_DIGIT_NUM_MAX           (9)
-#define APICMD_CELLINFO_MCC_DIGIT               (3)
-#define APICMD_CELLINFO_MNC_DIGIT_MIN           (2)
-#define APICMD_CELLINFO_MNC_DIGIT_MAX           (3)
-#define APICMD_CELLINFO_GCID_MAX                (16)
-#define APICMD_CELLINFO_TIMEDIFF_INDEX_MAX      (4095)
-#define APICMD_CELLINFO_TA_MAX                  (1282)
-#define APICMD_CELLINFO_SFN_MAX                 (0x03FF)
-#define APICMD_CELLINFO_NEIGHBOR_CELL_NUM_MAX   (32)
-#define APICMD_CELLINFO_VALID_TIMEDIFFIDX       (1 << 1)
-#define APICMD_CELLINFO_VALID_TA                (1 << 2)
-#define APICMD_CELLINFO_VALID_SFN               (1 << 3)
-#define APICMD_CELLINFO_VALID_RSRP              (1 << 4)
-#define APICMD_CELLINFO_VALID_RSRQ              (1 << 5)
-
-#define APICMD_DATAALLOW_DATAALLOW_ALLOW        (0)
-#define APICMD_DATAALLOW_DATAALLOW_DISALLOW     (1)
-
-#define APICMD_DATAALLOW_DATAROAMALLOW_ALLOW    (0)
-#define APICMD_DATAALLOW_DATAROAMALLOW_DISALLOW (1)
-
-#define APICMD_DEACTIVATEPDN_SESSIONID_MIN      (0)
-#define APICMD_DEACTIVATEPDN_SESSIONID_MAX      (255)
+#define APICMD_CELLINFO_CELLID_MAX    (503)
+#define APICMD_CELLINFO_EARFCN_MAX    (262143)
+#define APICMD_CELLINFO_DIGIT_NUM_MIN (0)
+#define APICMD_CELLINFO_DIGIT_NUM_MAX (9)
+#define APICMD_CELLINFO_MNC_DIGIT_MIN         (2)
+#define APICMD_CELLINFO_GCID_MAX              (16)
+#define APICMD_CELLINFO_VALID_TIMEDIFFIDX     (1 << 1)
+#define APICMD_CELLINFO_VALID_TA              (1 << 2)
+#define APICMD_CELLINFO_VALID_SFN             (1 << 3)
+#define APICMD_CELLINFO_VALID_RSRP            (1 << 4)
+#define APICMD_CELLINFO_VALID_RSRQ            (1 << 5)
 
 #define APICMD_EDRX_ACTTYPE_NOTUSE   (0) /* eDRX is not running */
 #define APICMD_EDRX_ACTTYPE_ECGSMIOT (1) /* EC-GSM-IoT (A/Gb mode) */
@@ -81,157 +61,63 @@
 
 #define APICMD_ERRINFO_ERRSTR_MAX_LEN            (64)
 
-#define APICMD_IMSCAP_ENABLE         (0)
-#define APICMD_IMSCAP_DISABLE        (1)
-
-#define APICMD_GETSIMINFO_MCC_DIXIT_MAX          (3)
-#define APICMD_GETSIMINFO_MNC_DIXIT_MAX          (3)
-#define APICMD_GETSIMINFO_SPN_MAX_LEN            (16)
-#define APICMD_GETSIMINFO_ICCID_MAX_LEN          (10)
-#define APICMD_GETSIMINFO_IMSI_LEN               (15)
-#define APICMD_GETSIMINFO_GID_LEN                (128)
-
-#define APICMD_IMEI_LEN                           16
-#define APICMD_IMSI_LEN                           16
+#define APICMD_IMSCAP_ENABLE                     (0)
+#define APICMD_IMSCAP_DISABLE                    (1)
 
 #define APICMD_NETINFO_PDNCOUNT_MAX              (5)
 
-#define APICMD_OPERATOR_LEN                       17
-#define APICMD_OPERATOR_LEN_V4                    33
+#define APICMD_OPERATOR_LEN_V4  33
 
-#define APICMD_PDN_IMS_REG                       (0)
-#define APICMD_PDN_IMS_UNREG                     (1)
+#define APICMD_PDN_IMS_REG                (0)
+#define APICMD_PDN_IMS_UNREG              (1)
 
-#define APICMD_PDN_DATAALLOW_ALLOW               (0)
-#define APICMD_PDN_DATAALLOW_DISALLOW            (1)
+#define APICMD_PDN_DATAALLOW_ALLOW        (0)
+#define APICMD_PDN_DATAALLOW_DISALLOW     (1)
 
-#define APICMD_PDN_DATAROAMALLOW_ALLOW           (0)
-#define APICMD_PDN_DATAROAMALLOW_DISALLOW        (1)
+#define APICMD_PDN_DATAROAMALLOW_ALLOW    (0)
+#define APICMD_PDN_DATAROAMALLOW_DISALLOW (1)
 
-#define APICMD_PDN_IPCOUNT_MAX                   (2)
-#define APICMD_PDN_IPADDR_MAXLEN                 (40)
+#define APICMD_PDN_DNSCOUNT_MAX           (4)
 
-#define APICMD_PDN_DNSCOUNT_MAX                  (4)
+#define APICMD_QUALITY_DISABLE  (0)
+#define APICMD_QUALITY_ENABLE   (1)
+#define APICMD_QUALITY_RSRP_MIN (-140)
+#define APICMD_QUALITY_RSRP_MAX (0)
+#define APICMD_QUALITY_RSRQ_MIN (-60)
+#define APICMD_QUALITY_RSRQ_MAX (0)
+#define APICMD_QUALITY_SINR_MIN (-128)
+#define APICMD_QUALITY_SINR_MAX (40)
 
-#define APICMD_PHONENO_LEN                        41
+#define APICMD_SET_REP_EVT_DISABLE                          (0)
+#define APICMD_SET_REP_EVT_ENABLE                           (1)
 
-#define APICMD_PSM_TIMER_MIN                     (1)
-#define APICMD_PSM_TIMER_MAX                     (31)
+#define APICMD_SET_REP_EVT_LTIME                            (1 << 0)
+#define APICMD_SET_REP_EVT_SIMD                             (1 << 1)
+#define APICMD_SET_REP_EVT_SIMSTATE                         (1 << 2)
 
-#define APICMD_QUALITY_DISABLE                   (0)
-#define APICMD_QUALITY_ENABLE                    (1)
-#define APICMD_QUALITY_RSRP_MIN                  (-140)
-#define APICMD_QUALITY_RSRP_MAX                  (0)
-#define APICMD_QUALITY_RSRQ_MIN                  (-60)
-#define APICMD_QUALITY_RSRQ_MAX                  (0)
-#define APICMD_QUALITY_SINR_MIN                  (-128)
-#define APICMD_QUALITY_SINR_MAX                  (40)
+#define APICMD_SET_REP_EVT_RES_OK                           (0)
+#define APICMD_SET_REP_EVT_RES_ERR                          (1)
 
-#define APICMD_RAT_RES_OK                        (0)
-#define APICMD_RAT_RES_ERR                       (1)
-#define APICMD_RAT_RAT_DEFAULT                   (1)
-#define APICMD_RAT_RAT_CATM                      (2)
-#define APICMD_RAT_RAT_NBIOT                     (3)
-#define APICMD_RAT_RAT_GSM                       (4)
-#define APICMD_RAT_RAT_C2D                       (5)
-#define APICMD_RAT_RAT_N2D                       (6)
-#define APICMD_RAT_RAT_G2D                       (7)
-#define APICMD_RAT_NOT_PERSIS                    (0)
-#define APICMD_RAT_PERSIS                        (1)
-#define APICMD_RAT_RAT_MODE_SINGLE               (0)
-#define APICMD_RAT_RAT_MODE_MULTIPLE             (1)
-#define APICMD_RAT_SOURCE_NONE                   (0)
-#define APICMD_RAT_SOURCE_HOST                   (1)
-#define APICMD_RAT_SOURCE_LWM2M                  (2)
+#define APICMD_REPORT_EVT_TYPE_LTIME                        (0)
+#define APICMD_REPORT_EVT_TYPE_SIMD                         (1)
+#define APICMD_REPORT_EVT_TYPE_SIMSTATE                     (2)
 
-#define APICMD_SET_REPCELLINFO_INTERVAL_MIN      (1)
-#define APICMD_SET_REPCELLINFO_INTERVAL_MAX      (4233600)
-
-#define APICMD_SET_REP_EVT_DISABLE               (0)
-#define APICMD_SET_REP_EVT_ENABLE                (1)
-
-#define APICMD_SET_REP_EVT_LTIME                 (1 << 0)
-#define APICMD_SET_REP_EVT_SIMD                  (1 << 1)
-#define APICMD_SET_REP_EVT_SIMSTATE              (1 << 2)
-#define APICMD_SET_REP_EVT_REGSTATE              (1 << 3)
-#define APICMD_SET_REP_EVT_PSMSTATE              (1 << 4)
-#define APICMD_SET_REP_EVT_DYNPSM                (1 << 5)
-#define APICMD_SET_REP_EVT_DYNEDRX               (1 << 6)
-#define APICMD_SET_REP_EVT_CONNPHASE             (1 << 7)
-#define APICMD_SET_REP_EVT_ANTITAMPER            (1 << 8)
-#define APICMD_SET_REP_EVT_MAX                   (1 << 15)
-
-#define APICMD_SET_REP_EVT_RES_OK                (0)
-#define APICMD_SET_REP_EVT_RES_ERR               (1)
-
-#define APICMD_REPORT_EVT_TYPE_LTIME             (0)
-#define APICMD_REPORT_EVT_TYPE_SIMD              (1)
-#define APICMD_REPORT_EVT_TYPE_SIMSTATE          (2)
-#define APICMD_REPORT_EVT_TYPE_REGSTATE          (3)
-#define APICMD_REPORT_EVT_TYPE_PSMSTATE          (4)
-#define APICMD_REPORT_EVT_TYPE_DYNPSM            (5)
-#define APICMD_REPORT_EVT_TYPE_DYNEDRX           (6)
-#define APICMD_REPORT_EVT_TYPE_CONNPHASE         (7)
-#define APICMD_REPORT_EVT_TYPE_ANTITAMPER        (8)
-
-#define APICMD_REPORT_EVT_SIMD_REMOVAL           (0)
-#define APICMD_REPORT_EVT_SIMD_INSERTION         (1)
+#define APICMD_REPORT_EVT_SIMD_REMOVAL                      (0)
+#define APICMD_REPORT_EVT_SIMD_INSERTION                    (1)
 
 #define APICMD_REPORT_EVT_SIMSTATE_DEACTIVATED              (0)
 #define APICMD_REPORT_EVT_SIMSTATE_SIM_INIT_WAIT_PIN_UNLOCK (1)
 #define APICMD_REPORT_EVT_SIMSTATE_PERSONALIZATION_FAILED   (2)
 #define APICMD_REPORT_EVT_SIMSTATE_ACTIVATION_COMPLETED     (3)
 
-#define APICMD_REPORT_EVT_ALERTU_TEMP                       (0)
-#define APICMD_REPORT_EVT_ALERTU_ANTIAMPER                  (1)
-
-#define APICMD_REP_PSMST_NOT_ACTIVE                         (0)
-#define APICMD_REP_PSMST_ACTIVE                             (1)
-#define APICMD_REP_PSMST_ACTIVE_AND_CAMPED                  (2)
-#define APICMD_REP_PSMST_CAMP_INTERRUPTED                   (3)
-
-#define APICMD_REP_CONNPHASE_START_SCAN                     (0)
-#define APICMD_REP_CONNPHASE_FAIL_SCAN                      (1)
-#define APICMD_REP_CONNPHASE_ENTER_CAMPED                   (2)
-#define APICMD_REP_CONNPHASE_CONNECTION_ESTABLISHMENT       (3)
-#define APICMD_REP_CONNPHASE_START_RESCAN                   (4)
-#define APICMD_REP_CONNPHASE_ENTER_CONNECTED                (5)
-#define APICMD_REP_CONNPHASE_NO_SUITABLE_CELL               (6)
-#define APICMD_REP_CONNPHASE_REG_ATTEMPT_FAILED             (7)
-#define APICMD_REP_CONNPHASE_NOT_AVAIL                      (99)
-
-#define APICMD_REP_CONPHASE_RAT_CATM                        (0)
-#define APICMD_REP_CONPHASE_RAT_NBIOT                       (1)
-#define APICMD_REP_CONPHASE_RAT_GSM                         (3)
-#define APICMD_REP_CONPHASE_RAT_NOT_AVAIL                   (99)
-
-#define APICMD_REP_CONPHASE_SCAN_MRU_ONLY                   (0)
-#define APICMD_REP_CONPHASE_SCAN_MRU_AND_FULL_SCAN          (1)
-#define APICMD_REP_CONPHASE_SCAN_MRU_AND_COUNTRY_SCAN       (2)
-#define APICMD_REP_CONPHASE_SCAN_MRU_AND_LS                 (3)
-#define APICMD_REP_CONPHASE_SCAN_MRU_NOT_AVAIL              (99)
-
-#define APICMD_REP_CONPHASE_SCAN_REASON_INIT_SCAN           (0)
-#define APICMD_REP_CONPHASE_SCAN_REASON_OUT_OF_COVERAGE     (1)
-#define APICMD_REP_CONPHASE_SCAN_REASON_HIGH_PRIORITY       (2)
-#define APICMD_REP_CONPHASE_SCAN_REASON_LIMITED_SERVICE     (3)
-#define APICMD_REP_CONPHASE_SCAN_REASON_COPS                (4)
-#define APICMD_REP_CONPHASE_SCAN_REASON_OTHER               (5)
-#define APICMD_REP_CONPHASE_SCAN_REASON_NOT_AVAIL           (99)
-
 #define APICMD_REPNETINFO_RES_OK                            (0)
 #define APICMD_REPNETINFO_RES_ERR                           (1)
 
 #define APICMD_REPNETINFO_REPORT_ENABLE                     (0)
 #define APICMD_REPNETINFO_REPORT_DISABLE                    (1)
 
-#define APICMD_REPNETINFO_NWSTAT_ATCH                       (0)
-#define APICMD_REPNETINFO_NWSTAT_DTCH                       (1)
-
 #define APICMD_SET_REPQUALITY_DISABLE                       (0)
 #define APICMD_SET_REPQUALITY_ENABLE                        (1)
-#define APICMD_SET_REPQUALITY_INTERVAL_MIN                  (1)
-#define APICMD_SET_REPQUALITY_INTERVAL_MAX                  (4233600)
 
 #define APICMD_SETPINCODE_PINCODE_LEN                        9
 
@@ -240,9 +126,6 @@
 
 #define APICMD_SETPINLOCK_PINCODE_LEN                        9
 
-#define APICMD_VERSION_RES_BB_PRODUCT_LEN                    5
-#define APICMD_VERSION_RES_NP_PACKAGE_LEN                    32
-
 #define APICMD_FW_INJECTDATA_MAXLEN                         (4096)
 #define APICMD_FW_INJECTDATA_MAXLEN_V4                      (3000)
 
@@ -258,7 +141,7 @@
 begin_packed_struct struct apicmd_ipaddr_s
 {
   uint8_t iptype;
-  uint8_t address[APICMD_PDN_IPADDR_MAXLEN];
+  uint8_t address[LTE_IPADDR_MAX_LEN];
 } end_packed_struct;
 
 begin_packed_struct struct apicmd_pdnset_s
@@ -268,7 +151,7 @@ begin_packed_struct struct apicmd_pdnset_s
   uint32_t apntype;
   uint8_t ipaddr_num;
   struct apicmd_ipaddr_s
-    ip_address[APICMD_PDN_IPCOUNT_MAX];
+    ip_address[LTE_PDN_IPADDR_MAX_COUNT];
   uint8_t imsregister;
   uint8_t dataallow;
   uint8_t dararoamingallow;
@@ -281,7 +164,7 @@ begin_packed_struct struct apicmd_pdnset_v4_s
   uint32_t apntype;
   uint8_t ipaddr_num;
   struct apicmd_ipaddr_s
-    ip_address[APICMD_PDN_IPCOUNT_MAX];
+    ip_address[LTE_PDN_IPADDR_MAX_COUNT];
   uint8_t imsregister;
   uint8_t dataallow;
   uint8_t dararoamingallow;
@@ -375,10 +258,10 @@ begin_packed_struct struct apicmd_cmddat_activatepdn_s
 {
   uint32_t apntype;
   uint8_t iptype;
-  uint8_t apnname[APICMD_ACTIVATEPDN_APNNAME_MAXLEN];
+  uint8_t apnname[LTE_APN_LEN];
   uint8_t authtype;
-  uint8_t username[APICMD_ACTIVATEPDN_USERNAME_MAXLEN];
-  uint8_t password[APICMD_ACTIVATEPDN_PASSWORD_MAXLEN];
+  uint8_t username[LTE_APN_USER_NAME_LEN];
+  uint8_t password[LTE_APN_PASSWD_LEN];
 } end_packed_struct;
 
 begin_packed_struct struct apicmd_cmddat_activatepdnres_s
@@ -407,9 +290,9 @@ begin_packed_struct struct apicmd_cmddat_cellinfo_s
   uint8_t  valid;
   uint32_t cell_id;
   uint32_t earfcn;
-  uint8_t  mcc[APICMD_CELLINFO_MCC_DIGIT];
+  uint8_t  mcc[LTE_MCC_DIGIT];
   uint8_t  mnc_digit;
-  uint8_t  mnc[APICMD_CELLINFO_MNC_DIGIT_MAX];
+  uint8_t  mnc[LTE_MNC_DIGIT_MAX];
 } end_packed_struct;
 
 begin_packed_struct struct apicmd_cmddat_cellinfo_v4_s
@@ -417,9 +300,9 @@ begin_packed_struct struct apicmd_cmddat_cellinfo_v4_s
   uint8_t enability;
   uint32_t cell_id;
   uint32_t earfcn;
-  uint8_t mcc[APICMD_CELLINFO_MCC_DIGIT];
+  uint8_t mcc[LTE_MCC_DIGIT];
   uint8_t mnc_digit;
-  uint8_t mnc[APICMD_CELLINFO_MNC_DIGIT_MAX];
+  uint8_t mnc[LTE_MNC_DIGIT_MAX];
   uint8_t cgid[APICMD_CELLINFO_GCID_MAX + 1];
   uint16_t tac;
   uint16_t time_diffidx;
@@ -429,7 +312,7 @@ begin_packed_struct struct apicmd_cmddat_cellinfo_v4_s
   int16_t rsrq;
   uint8_t neighbor_num;
   struct apicmd_cmddat_neighbor_cell_s
-           neighbor_cell[APICMD_CELLINFO_NEIGHBOR_CELL_NUM_MAX];
+           neighbor_cell[LTE_NEIGHBOR_CELL_MAX];
 } end_packed_struct;
 
 /* structure for APICMDID_DATA_ALLOW */
@@ -602,19 +485,19 @@ begin_packed_struct struct apicmd_cmddat_getsiminfo_res_s
 {
   uint8_t  result;
   uint32_t option;
-  uint8_t  mcc[APICMD_GETSIMINFO_MCC_DIXIT_MAX];
+  uint8_t  mcc[LTE_MCC_DIGIT];
   uint8_t  mnc_digit;
-  uint8_t  mnc[APICMD_GETSIMINFO_MNC_DIXIT_MAX];
+  uint8_t  mnc[LTE_MNC_DIGIT_MAX];
   uint8_t  spn_len;
-  uint8_t  spn[APICMD_GETSIMINFO_SPN_MAX_LEN];
+  uint8_t  spn[LTE_SIMINFO_SPN_LEN];
   uint8_t  iccid_len;
-  uint8_t  iccid[APICMD_GETSIMINFO_ICCID_MAX_LEN];
+  uint8_t  iccid[LTE_SIMINFO_ICCID_LEN];
   uint8_t  imsi_len;
-  uint8_t  imsi[APICMD_GETSIMINFO_IMSI_LEN];
+  uint8_t  imsi[LTE_SIMINFO_IMSI_LEN];
   uint8_t  gid1_len;
-  uint8_t  gid1[APICMD_GETSIMINFO_GID_LEN];
+  uint8_t  gid1[LTE_SIMINFO_GID_LEN];
   uint8_t  gid2_len;
-  uint8_t  gid2[APICMD_GETSIMINFO_GID_LEN];
+  uint8_t  gid2[LTE_SIMINFO_GID_LEN];
 } end_packed_struct;
 
 /* structure for APICMDID_GET_IMEI */
@@ -622,7 +505,7 @@ begin_packed_struct struct apicmd_cmddat_getsiminfo_res_s
 begin_packed_struct struct apicmd_cmddat_getimeires_s
 {
   uint8_t result;
-  uint8_t imei[APICMD_IMEI_LEN];
+  uint8_t imei[LTE_IMEI_LEN];
 } end_packed_struct;
 
 /* structure for APICMDID_GET_IMSI */
@@ -631,7 +514,7 @@ begin_packed_struct struct apicmd_cmddat_getimsires_s
 {
   uint8_t result;
   uint8_t errcause;
-  uint8_t imsi[APICMD_IMSI_LEN];
+  uint8_t imsi[LTE_IMSI_LEN];
 } end_packed_struct;
 
 /* structure for APICMDID_GET_LTIME */
@@ -647,7 +530,7 @@ begin_packed_struct struct apicmd_cmddat_getltimeres_s
 begin_packed_struct struct apicmd_cmddat_getoperatorres_s
 {
   uint8_t result;
-  uint8_t oper[APICMD_OPERATOR_LEN];
+  uint8_t oper[LTE_OPERATOR_LEN];
 } end_packed_struct;
 
 begin_packed_struct struct apicmd_cmddat_getoperatorres_v4_s
@@ -662,7 +545,7 @@ begin_packed_struct struct apicmd_cmddat_phonenores_s
 {
   uint8_t result;
   uint8_t errcause;
-  uint8_t phoneno[APICMD_PHONENO_LEN];
+  uint8_t phoneno[LTE_PHONENO_LEN];
 } end_packed_struct;
 
 /* structure for APICMDID_RADIO_OFF */
@@ -928,8 +811,8 @@ begin_packed_struct struct apicmd_cmddat_setpsmres_s
 begin_packed_struct struct apicmd_cmddat_getverres_s
 {
   uint8_t result;
-  uint8_t bb_product[APICMD_VERSION_RES_BB_PRODUCT_LEN];
-  uint8_t np_package[APICMD_VERSION_RES_NP_PACKAGE_LEN];
+  uint8_t bb_product[LTE_VER_BB_PRODUCT_LEN];
+  uint8_t np_package[LTE_VER_NP_PACKAGE_LEN];
 } end_packed_struct;
 
 /* structure for APICMDID_FW_INJECTDELTAIMG */
