@@ -1554,8 +1554,9 @@ static int dm9x_addmac(FAR struct net_driver_s *dev, FAR const uint8_t *mac)
 
   /* Add the MAC address to the hardware multicast routing table */
 
-#warning "Multicast MAC support not implemented"
-  return OK;
+  /* #warning "Multicast MAC support not implemented" */
+
+  return -ENOSYS;
 }
 #endif
 
@@ -1585,8 +1586,9 @@ static int dm9x_rmmac(FAR struct net_driver_s *dev, FAR const uint8_t *mac)
 
   /* Add the MAC address to the hardware multicast routing table */
 
-#warning "Multicast MAC support not implemented"
-  return OK;
+  /* #warning "Multicast MAC support not implemented" */
+
+  return -ENOSYS;
 }
 #endif
 
