@@ -29,6 +29,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <sys/param.h>
+
 #include <nuttx/kmalloc.h>
 #include <nuttx/list.h>
 #include <nuttx/power/consumer.h>
@@ -38,10 +40,6 @@
  * Pre-processor Definitions
  ****************************************************************************/
 
-#ifndef ARRAY_SIZE
-#  define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
-#endif
-
 #define REGULATOR_RPMSG_EPT_NAME    "rpmsg-regulator"
 
 #define REGULATOR_RPMSG_ENABLE      0
@@ -400,7 +398,7 @@ static int regulator_rpmsg_ept_cb(FAR struct rpmsg_endpoint *ept,
       nxsem_post(&cookie->sem);
       ret = 0;
     }
-  else if (cmd < ARRAY_SIZE(g_regulator_rpmsg_handler)
+  else if (cmd < nitems(g_regulator_rpmsg_handler)
            && g_regulator_rpmsg_handler[cmd])
     {
       header->response = 1;
@@ -528,7 +526,7 @@ static int regulator_rpmsg_enable(FAR struct regulator_dev_s *rdev)
       return -ENOMEM;
     }
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, len - sizeof(*msg));
   return regulator_rpmsg_sendrecv(ept, REGULATOR_RPMSG_ENABLE,
                                  (struct regulator_rpmsg_header_s *)msg,
                                   len);
@@ -554,7 +552,7 @@ static int regulator_rpmsg_disable(FAR struct regulator_dev_s *rdev)
       return -ENOMEM;
     }
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, len - sizeof(*msg));
   return regulator_rpmsg_sendrecv(ept, REGULATOR_RPMSG_DISABLE,
                                  (struct regulator_rpmsg_header_s *)msg,
                                  len);
@@ -582,7 +580,7 @@ static int regulator_rpmsg_set_voltage(FAR struct regulator_dev_s *rdev,
       return -ENOMEM;
     }
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, len - sizeof(*msg));
   msg->min_uv = min_uv;
   msg->max_uv = max_uv;
 
@@ -611,7 +609,7 @@ static int regulator_rpmsg_get_voltage(FAR struct regulator_dev_s *rdev)
       return -ENOMEM;
     }
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, len - sizeof(*msg));
   return regulator_rpmsg_sendrecv(ept, REGULATOR_RPMSG_GET_VOLTAGE,
                                  (struct regulator_rpmsg_header_s *)msg,
                                   len);
@@ -637,7 +635,7 @@ static int regulator_rpmsg_is_enabled(FAR struct regulator_dev_s *rdev)
       return -ENOMEM;
     }
 
-  strcpy(msg->name, name);
+  strlcpy(msg->name, name, len - sizeof(*msg));
   return regulator_rpmsg_sendrecv(ept, REGULATOR_RPMSG_IS_ENABLED,
                                  (struct regulator_rpmsg_header_s *)msg,
                                   len);
