@@ -26,8 +26,10 @@
 
 #include <ctype.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <stdint.h>
 #include <stdbool.h>
+#include <time.h>
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
@@ -223,10 +225,6 @@ static int uart_putxmitchar(FAR uart_dev_t *dev, int ch, bool oktoblock)
 #endif
           else
             {
-              /* Inform the interrupt level logic that we are waiting. */
-
-              dev->xmitwaiting = true;
-
               /* Wait for some characters to be sent from the buffer with
                * the TX interrupt enabled.  When the TX interrupt is enabled,
                * uart_xmitchars() should execute and remove some of the data
@@ -413,10 +411,6 @@ static int uart_tcdrain(FAR uart_dev_t *dev,
           ret = OK;
           while (ret >= 0 && dev->xmit.head != dev->xmit.tail)
             {
-              /* Inform the interrupt level logic that we are waiting. */
-
-              dev->xmitwaiting = true;
-
               /* Wait for some characters to be sent from the buffer with
                * the TX interrupt enabled.  When the TX interrupt is
                * enabled, uart_xmitchars() should execute and remove some
@@ -1046,8 +1040,18 @@ static ssize_t uart_read(FAR struct file *filep,
                    * thread goes to sleep.
                    */
 
-                  dev->recvwaiting = true;
-                  ret = nxsem_wait(&dev->recvsem);
+#ifdef CONFIG_SERIAL_TERMIOS
+                  dev->minrecv = MIN(buflen - recvd, dev->minread - recvd);
+                  if (dev->timeout)
+                    {
+                      ret = nxsem_tickwait(&dev->recvsem,
+                                           DSEC2TICK(dev->timeout));
+                    }
+                  else
+#endif
+                    {
+                      ret = nxsem_wait(&dev->recvsem);
+                    }
                 }
 
               leave_critical_section(flags);
@@ -1078,9 +1082,9 @@ static ssize_t uart_read(FAR struct file *filep,
                        */
 
 #ifdef CONFIG_SERIAL_REMOVABLE
-                      recvd = dev->disconnected ? -ENOTCONN : -EINTR;
+                      recvd = dev->disconnected ? -ENOTCONN : ret;
 #else
-                      recvd = -EINTR;
+                      recvd = ret;
 #endif
                     }
 
@@ -1539,6 +1543,10 @@ static int uart_ioctl(FAR struct file *filep, int cmd, unsigned long arg)
               termiosp->c_iflag = dev->tc_iflag;
               termiosp->c_oflag = dev->tc_oflag;
               termiosp->c_lflag = dev->tc_lflag;
+#ifdef CONFIG_SERIAL_TERMIOS
+              termiosp->c_cc[VTIME] = dev->timeout;
+              termiosp->c_cc[VMIN] = dev->minread;
+#endif
 
               ret = 0;
             }
@@ -1560,6 +1568,10 @@ static int uart_ioctl(FAR struct file *filep, int cmd, unsigned long arg)
               dev->tc_iflag = termiosp->c_iflag;
               dev->tc_oflag = termiosp->c_oflag;
               dev->tc_lflag = termiosp->c_lflag;
+#ifdef CONFIG_SERIAL_TERMIOS
+              dev->timeout = termiosp->c_cc[VTIME];
+              dev->minread = termiosp->c_cc[VMIN];
+#endif
               ret = 0;
             }
             break;
@@ -1750,7 +1762,8 @@ static void uart_launch_worker(void *arg)
                  CONFIG_TTY_LAUNCH_ENTRYPOINT,
                  NULL, &attr, argv, NULL);
 #else
-      exec_spawn(CONFIG_TTY_LAUNCH_FILEPATH, argv, NULL, NULL, 0, &attr);
+      exec_spawn(CONFIG_TTY_LAUNCH_FILEPATH,
+                 argv, NULL, NULL, 0, NULL, &attr);
 #endif
       posix_spawnattr_destroy(&attr);
     }
@@ -1762,6 +1775,23 @@ static void uart_launch(void)
 }
 #endif
 
+static void uart_wakeup(FAR sem_t *sem)
+{
+  int sval;
+
+  if (nxsem_get_value(sem, &sval) != OK)
+    {
+      return;
+    }
+
+  /* Yes... wake up all waiting threads */
+
+  while (sval++ < 1)
+    {
+      nxsem_post(sem);
+    }
+}
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
@@ -1812,6 +1842,11 @@ int uart_register(FAR const char *path, FAR uart_dev_t *dev)
   nxsem_init(&dev->recvsem, 0, 0);
   nxmutex_init(&dev->polllock);
 
+#ifdef CONFIG_SERIAL_TERMIOS
+  dev->timeout = 0;
+  dev->minread = 1;
+#endif
+
   /* Register the serial driver */
 
   sinfo("Registering %s\n", path);
@@ -1836,13 +1871,7 @@ void uart_datareceived(FAR uart_dev_t *dev)
 
   /* Is there a thread waiting for read data?  */
 
-  if (dev->recvwaiting)
-    {
-      /* Yes... wake it up */
-
-      dev->recvwaiting = false;
-      nxsem_post(&dev->recvsem);
-    }
+  uart_wakeup(&dev->recvsem);
 
 #if defined(CONFIG_PM) && defined(CONFIG_SERIAL_CONSOLE)
   /* Call pm_activity when characters are received on the console device */
@@ -1874,13 +1903,7 @@ void uart_datasent(FAR uart_dev_t *dev)
 
   /* Is there a thread waiting for space in xmit.buffer?  */
 
-  if (dev->xmitwaiting)
-    {
-      /* Yes... wake it up */
-
-      dev->xmitwaiting = false;
-      nxsem_post(&dev->xmitsem);
-    }
+  uart_wakeup(&dev->xmitsem);
 }
 
 /****************************************************************************
@@ -1928,23 +1951,11 @@ void uart_connected(FAR uart_dev_t *dev, bool connected)
 
       /* Is there a thread waiting for space in xmit.buffer?  */
 
-      if (dev->xmitwaiting)
-        {
-          /* Yes... wake it up */
-
-          dev->xmitwaiting = false;
-          nxsem_post(&dev->xmitsem);
-        }
+      uart_wakeup(&dev->xmitsem);
 
       /* Is there a thread waiting for read data?  */
 
-      if (dev->recvwaiting)
-        {
-          /* Yes... wake it up */
-
-          dev->recvwaiting = false;
-          nxsem_post(&dev->recvsem);
-        }
+      uart_wakeup(&dev->recvsem);
     }
 
   leave_critical_section(flags);
