@@ -36,6 +36,7 @@
 #include <nuttx/fs/fs.h>
 #include <nuttx/irq.h>
 #include <nuttx/kmalloc.h>
+#include <nuttx/panic_notifier.h>
 #include <nuttx/power/pm.h>
 #include <nuttx/mutex.h>
 #include <nuttx/wdog.h>
@@ -55,7 +56,7 @@
       !defined(CONFIG_WATCHDOG_AUTOMONITOR_BY_IDLE)
 #    if CONFIG_WATCHDOG_AUTOMONITOR_PING_INTERVAL == 0
 #      define WATCHDOG_AUTOMONITOR_PING_INTERVAL \
-         (CONFIG_WATCHDOG_AUTOMONITOR_PING_INTERVAL / 2)
+         (CONFIG_WATCHDOG_AUTOMONITOR_TIMEOUT / 2)
 #    else
 #      define WATCHDOG_AUTOMONITOR_PING_INTERVAL \
          CONFIG_WATCHDOG_AUTOMONITOR_PING_INTERVAL
@@ -75,6 +76,9 @@
 
 struct watchdog_upperhalf_s
 {
+  /* When a crash occurs, stop the watchdog */
+
+  struct notifier_block nb;
 #ifdef CONFIG_WATCHDOG_AUTOMONITOR
 #  if defined(CONFIG_WATCHDOG_AUTOMONITOR_BY_ONESHOT)
   FAR struct oneshot_lowerhalf_s *oneshot;
@@ -124,10 +128,6 @@ static const struct file_operations g_wdogops =
   wdog_write, /* write */
   NULL,       /* seek */
   wdog_ioctl, /* ioctl */
-  NULL        /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL      /* unlink */
-#endif
 };
 
 /****************************************************************************
@@ -300,6 +300,24 @@ static void watchdog_automonitor_stop(FAR struct watchdog_upperhalf_s *upper)
 }
 #endif
 
+static int wdog_notifier(FAR struct notifier_block *nb, unsigned long action,
+                         FAR void *data)
+{
+  FAR struct watchdog_upperhalf_s *upper =
+                                       (FAR struct watchdog_upperhalf_s *)nb;
+
+  if (action == PANIC_KERNEL)
+    {
+#ifdef CONFIG_WATCHDOG_AUTOMONITOR
+      watchdog_automonitor_stop(upper);
+#else
+      return upper->lower->ops->stop(upper->lower);
+#endif
+    }
+
+  return 0;
+}
+
 /****************************************************************************
  * Name: wdog_open
  *
@@ -418,7 +436,48 @@ static ssize_t wdog_read(FAR struct file *filep, FAR char *buffer,
 static ssize_t wdog_write(FAR struct file *filep, FAR const char *buffer,
                           size_t buflen)
 {
-  return 0;
+#ifdef CONFIG_WATCHDOG_MAGIC_V
+  FAR struct inode                *inode = filep->f_inode;
+  FAR struct watchdog_upperhalf_s *upper;
+  FAR struct watchdog_lowerhalf_s *lower;
+  int err = 0;
+  int i;
+
+  upper = inode->i_private;
+  DEBUGASSERT(upper != NULL);
+  lower = upper->lower;
+  DEBUGASSERT(lower != NULL);
+
+  nxmutex_lock(&upper->lock);
+
+  for (i = 0; i < buflen; i++)
+    {
+      if (buffer[i] == 'V')
+        {
+#ifdef CONFIG_WATCHDOG_AUTOMONITOR
+          watchdog_automonitor_stop(upper);
+#else
+          err = lower->ops->stop(lower);
+#endif
+          break;
+        }
+    }
+
+  if (i == buflen)
+    {
+      err = lower->ops->keepalive(lower);
+    }
+
+  nxmutex_unlock(&upper->lock);
+
+  if (err < 0)
+    {
+      return err;
+    }
+
+#endif
+
+  return buflen;
 }
 
 /****************************************************************************
@@ -719,6 +778,9 @@ FAR void *watchdog_register(FAR const char *path,
   watchdog_automonitor_start(upper);
 #endif
 
+  upper->nb.notifier_call = wdog_notifier;
+  panic_notifier_chain_register(&upper->nb);
+
   return (FAR void *)upper;
 
 errout_with_path:
@@ -773,6 +835,7 @@ void watchdog_unregister(FAR void *handle)
   /* Unregister the watchdog timer device */
 
   unregister_driver(upper->path);
+  panic_notifier_chain_unregister(&upper->nb);
 
   /* Then free all of the driver resources */
 
