@@ -391,10 +391,9 @@ static const struct file_operations g_hidmouse_fops =
   usbhost_write,           /* write */
   NULL,                    /* seek */
   NULL,                    /* ioctl */
+  NULL,                    /* mmap */
+  NULL,                    /* truncate */
   usbhost_poll             /* poll */
-#ifndef CONFIG_DISABLE_PSEUDOFS_OPERATIONS
-  , NULL                   /* unlink */
-#endif
 };
 
 /* This is a bitmap that is used to allocate device names /dev/mouse0-31. */
