@@ -48,6 +48,7 @@
 struct usrsock_rpmsg_s
 {
   rmutex_t                  mutex;
+  ssize_t                   remain;
   struct iovec              iov[CONFIG_NET_USRSOCK_RPMSG_SERVER_NIOVEC];
   struct socket             socks[CONFIG_NET_USRSOCK_RPMSG_SERVER_NSOCKS];
   FAR struct rpmsg_endpoint *epts[CONFIG_NET_USRSOCK_RPMSG_SERVER_NSOCKS];
@@ -382,21 +383,28 @@ static int usrsock_rpmsg_sendto_handler(FAR struct rpmsg_endpoint *ept,
   FAR struct usrsock_rpmsg_s *priv = priv_;
   uint16_t events = 0;
   ssize_t ret = -EBADF;
-  size_t total;
   int retr;
   int i;
 
-  if (priv->iov[0].iov_base)
+  if (priv->remain > 0)
     {
       size_t hlen;
       struct msghdr msg =
       {
       };
 
+      priv->remain -= len;
+
+      if (!priv->iov[0].iov_base)
+        {
+          /* Maybe error occurred previously, skip processing. */
+
+          return 0;
+        }
+
       req = priv->iov[0].iov_base;
       hlen = sizeof(*req) + req->addrlen;
 
-      total = len;
       for (i = 0; i < CONFIG_NET_USRSOCK_RPMSG_SERVER_NIOVEC; i++)
         {
           if (!priv->iov[i].iov_base)
@@ -406,23 +414,24 @@ static int usrsock_rpmsg_sendto_handler(FAR struct rpmsg_endpoint *ept,
               rpmsg_hold_rx_buffer(ept, data);
               break;
             }
-
-          total += priv->iov[i].iov_len;
-        }
-
-      if (i == CONFIG_NET_USRSOCK_RPMSG_SERVER_NIOVEC)
-        {
-          ret = -ENOMEM;
-          goto out;
         }
 
       /* Partial packet ? continue to fetch */
 
-      if (req->buflen > total - hlen)
+      if (priv->remain > 0)
         {
+          /* We've used the last I/O vector, cannot continue. */
+
+          if (i == CONFIG_NET_USRSOCK_RPMSG_SERVER_NIOVEC - 1)
+            {
+              nerr("ERROR: Request %d too large!\n", req->usockid);
+              ret = -ENOMEM;
+              goto out;
+            }
+
           return 0;
         }
-      else if (req->buflen < total - hlen)
+      else if (priv->remain < 0)
         {
           ret = -EINVAL;
           goto out;
@@ -452,14 +461,18 @@ static int usrsock_rpmsg_sendto_handler(FAR struct rpmsg_endpoint *ept,
       if (req->usockid >= 0 &&
           req->usockid < CONFIG_NET_USRSOCK_RPMSG_SERVER_NSOCKS)
         {
-          total = sizeof(*req) + req->addrlen + req->buflen;
-          if (total > len)
+          priv->remain = sizeof(*req) + req->addrlen + req->buflen - len;
+          if (priv->remain > 0)
             {
+#if CONFIG_NET_USRSOCK_RPMSG_SERVER_NIOVEC >= 2
               priv->iov[0].iov_base = data;
               priv->iov[0].iov_len = len;
 
               rpmsg_hold_rx_buffer(ept, data);
               return 0;
+#else
+              ret = -ENOMEM;
+#endif
             }
           else
             {
@@ -483,7 +496,8 @@ out:
   retr = usrsock_rpmsg_send_ack(ept, events, req->head.xid, ret);
   if (retr >= 0 && events == 0)
     {
-      usrsock_rpmsg_poll_setup(&priv->pfds[req->usockid], POLLOUT);
+      usrsock_rpmsg_poll_setup(&priv->pfds[req->usockid],
+                               priv->pfds[req->usockid].events | POLLOUT);
     }
 
   if (priv->iov[0].iov_base)
@@ -495,9 +509,9 @@ out:
               break;
             }
 
-            rpmsg_release_rx_buffer(ept, priv->iov[i].iov_base);
-            priv->iov[i].iov_base = NULL;
-            priv->iov[i].iov_len = 0;
+          rpmsg_release_rx_buffer(ept, priv->iov[i].iov_base);
+          priv->iov[i].iov_base = NULL;
+          priv->iov[i].iov_len = 0;
         }
     }
 
@@ -564,7 +578,6 @@ static int usrsock_rpmsg_recvfrom_handler(FAR struct rpmsg_endpoint *ept,
                                                             false);
               if (!iov[i].iov_base)
                 {
-                  events |= USRSOCK_EVENT_RECVFROM_AVAIL;
                   break;
                 }
 
@@ -593,12 +606,16 @@ static int usrsock_rpmsg_recvfrom_handler(FAR struct rpmsg_endpoint *ept,
               else
                 {
                   iov[i].iov_len = 0;
-                  events |= USRSOCK_EVENT_RECVFROM_AVAIL;
                   break;
                 }
 
               i++;
             }
+
+          if (usrsock_rpmsg_available(&priv->socks[req->usockid], FIONREAD))
+            {
+              events |= USRSOCK_EVENT_RECVFROM_AVAIL;
+            }
         }
     }
 
@@ -627,7 +644,8 @@ static int usrsock_rpmsg_recvfrom_handler(FAR struct rpmsg_endpoint *ept,
 
   if (retr >= 0 && events == 0)
     {
-      usrsock_rpmsg_poll_setup(&priv->pfds[req->usockid], POLLIN);
+      usrsock_rpmsg_poll_setup(&priv->pfds[req->usockid],
+                               priv->pfds[req->usockid].events | POLLIN);
     }
 
   return retr;
@@ -926,6 +944,10 @@ static int usrsock_rpmsg_send_dns_event(FAR void *arg,
   uint32_t len;
 
   dns = rpmsg_get_tx_payload_buffer(ept, &len, true);
+  if (dns == NULL)
+    {
+      return -ENOMEM;
+    }
 
   dns->head.msgid = USRSOCK_RPMSG_DNS_EVENT;
   dns->head.flags = USRSOCK_MESSAGE_FLAG_EVENT;
@@ -1010,7 +1032,7 @@ static int usrsock_rpmsg_ept_cb(FAR struct rpmsg_endpoint *ept,
   FAR struct usrsock_request_common_s *common = data;
   FAR struct usrsock_rpmsg_s *priv = priv_;
 
-  if (priv->iov[0].iov_base)
+  if (priv->remain > 0)
     {
       return usrsock_rpmsg_sendto_handler(ept, data, len, src, priv);
     }
@@ -1041,16 +1063,21 @@ static void usrsock_rpmsg_poll_setup(FAR struct pollfd *pfds,
 
   if (events)
     {
-      if (!pfds->events)
+      if (pfds->events)
         {
+          ret = psock_poll(psock, pfds, false);
+        }
+
+      if (ret >= 0)
+        {
+          /* The protocol stack monitor flag is different when the events is
+           * POLLIN or POLLOUT, so we have to call poll_setup again.
+           */
+
           pfds->revents = 0;
           pfds->events = events;
           ret = psock_poll(psock, pfds, true);
         }
-      else
-        {
-          pfds->events = events;
-        }
     }
   else
     {
@@ -1072,6 +1099,7 @@ static void usrsock_rpmsg_poll_setup(FAR struct pollfd *pfds,
 static void usrsock_rpmsg_poll_cb(FAR struct pollfd *pfds)
 {
   FAR struct usrsock_rpmsg_s *priv = (FAR struct usrsock_rpmsg_s *)pfds->arg;
+  int oldevents;
   int events = 0;
 
   nxrmutex_lock(&priv->mutex);
@@ -1082,6 +1110,7 @@ static void usrsock_rpmsg_poll_cb(FAR struct pollfd *pfds)
       return;
     }
 
+  oldevents = pfds->events;
   if (pfds->revents & POLLIN)
     {
       events |= USRSOCK_EVENT_RECVFROM_AVAIL;
@@ -1114,9 +1143,9 @@ static void usrsock_rpmsg_poll_cb(FAR struct pollfd *pfds)
         }
     }
 
-  if (!(pfds->events & (POLLIN | POLLOUT)))
+  if (oldevents != pfds->events)
     {
-      usrsock_rpmsg_poll_setup(pfds, 0);
+      usrsock_rpmsg_poll_setup(pfds, pfds->events);
     }
 
   if (events != 0)
