@@ -39,6 +39,14 @@
 #include <nuttx/fs/fs.h>
 #include <nuttx/fs/ioctl.h>
 #include <nuttx/video/fb.h>
+#include <nuttx/clock.h>
+#include <nuttx/wdog.h>
+
+/****************************************************************************
+ * Pre-processor definitions
+ ****************************************************************************/
+
+#define FB_NO_OVERLAY -1
 
 /****************************************************************************
  * Private Types
@@ -53,18 +61,31 @@
 struct fb_chardev_s
 {
   FAR struct fb_vtable_s *vtable; /* Framebuffer interface */
-  FAR void *fbmem;                /* Start of frame buffer memory */
   FAR struct pollfd *fds;         /* Polling structure of waiting thread */
-  size_t fblen;                   /* Size of the framebuffer */
   uint8_t plane;                  /* Video plan number */
-  uint8_t bpp;                    /* Bits per pixel */
   volatile bool pollready;        /* Poll ready flag */
+  clock_t vsyncoffset;            /* VSync offset ticks */
+  struct wdog_s wdog;             /* VSync offset timer */
+#ifdef CONFIG_FB_OVERLAY
+  int overlay;                    /* Overlay number */
+#endif
+  mutex_t lock;                  /* Mutual exclusion */
+  int16_t crefs;                 /* Number of open references */
+};
+
+struct fb_panelinfo_s
+{
+  FAR void *fbmem;                /* Start of frame buffer memory */
+  size_t fblen;                   /* Size of the framebuffer */
+  uint8_t bpp;                    /* Bits per pixel */
 };
 
 /****************************************************************************
  * Private Function Prototypes
  ****************************************************************************/
 
+static int     fb_open(FAR struct file *filep);
+static int     fb_close(FAR struct file *filep);
 static ssize_t fb_read(FAR struct file *filep, FAR char *buffer,
                        size_t buflen);
 static ssize_t fb_write(FAR struct file *filep, FAR const char *buffer,
@@ -75,15 +96,17 @@ static int     fb_mmap(FAR struct file *filep,
                        FAR struct mm_map_entry_s *map);
 static int     fb_poll(FAR struct file *filep, FAR struct pollfd *fds,
                        bool setup);
+static int     fb_get_panelinfo(FAR struct fb_chardev_s *fb,
+                                FAR struct fb_panelinfo_s *panelinfo);
 
 /****************************************************************************
  * Private Data
  ****************************************************************************/
 
-static const struct file_operations fb_fops =
+static const struct file_operations g_fb_fops =
 {
-  NULL,          /* open */
-  NULL,          /* close */
+  fb_open,       /* open */
+  fb_close,      /* close */
   fb_read,       /* read */
   fb_write,      /* write */
   fb_seek,       /* seek */
@@ -97,6 +120,86 @@ static const struct file_operations fb_fops =
  * Private Functions
  ****************************************************************************/
 
+/****************************************************************************
+ * Name: fb_open
+ ****************************************************************************/
+
+static int fb_open(FAR struct file *filep)
+{
+  FAR struct inode *inode;
+  FAR struct fb_chardev_s *fb;
+  int ret;
+
+  DEBUGASSERT(filep != NULL && filep->f_inode != NULL);
+  inode = filep->f_inode;
+  fb    = inode->i_private;
+
+  DEBUGASSERT(fb->vtable != NULL);
+
+  ret = nxmutex_lock(&fb->lock);
+  if (ret < 0)
+    {
+      return ret;
+    }
+
+  if (fb->crefs == 0)
+    {
+      if (fb->vtable->open != NULL)
+        {
+          ret = fb->vtable->open(fb->vtable);
+        }
+    }
+
+  if (ret >= 0)
+    {
+      fb->crefs++;
+      DEBUGASSERT(fb->crefs > 0);
+    }
+
+  nxmutex_unlock(&fb->lock);
+  return ret;
+}
+
+/****************************************************************************
+ * Name: fb_close
+ ****************************************************************************/
+
+static int fb_close(FAR struct file *filep)
+{
+  FAR struct inode *inode;
+  FAR struct fb_chardev_s *fb;
+  int ret;
+
+  DEBUGASSERT(filep != NULL && filep->f_inode != NULL);
+  inode = filep->f_inode;
+  fb    = inode->i_private;
+
+  DEBUGASSERT(fb->vtable != NULL);
+
+  ret = nxmutex_lock(&fb->lock);
+  if (ret < 0)
+    {
+      return ret;
+    }
+
+  if (fb->crefs == 1)
+    {
+      if (fb->vtable->close != NULL)
+        {
+          ret = fb->vtable->close(fb->vtable);
+        }
+    }
+
+  if (ret >= 0)
+    {
+      DEBUGASSERT(fb->crefs > 0);
+      fb->crefs--;
+    }
+
+  nxmutex_unlock(&fb->lock);
+  return ret;
+}
+
 /****************************************************************************
  * Name: fb_read
  ****************************************************************************/
@@ -105,9 +208,11 @@ static ssize_t fb_read(FAR struct file *filep, FAR char *buffer, size_t len)
 {
   FAR struct inode *inode;
   FAR struct fb_chardev_s *fb;
+  struct fb_panelinfo_s panelinfo;
   size_t start;
   size_t end;
   size_t size;
+  int ret;
 
   ginfo("len: %u\n", (unsigned int)len);
 
@@ -117,25 +222,34 @@ static ssize_t fb_read(FAR struct file *filep, FAR char *buffer, size_t len)
   inode = filep->f_inode;
   fb    = (FAR struct fb_chardev_s *)inode->i_private;
 
+  /* Get panel info */
+
+  ret = fb_get_panelinfo(fb, &panelinfo);
+
+  if (ret < 0)
+    {
+      return ret;
+    }
+
   /* Get the start and size of the transfer */
 
   start = filep->f_pos;
-  if (start >= fb->fblen)
+  if (start >= panelinfo.fblen)
     {
       return 0;  /* Return end-of-file */
     }
 
   end = start + len;
-  if (end >= fb->fblen)
+  if (end >= panelinfo.fblen)
     {
-      end = fb->fblen;
+      end = panelinfo.fblen;
     }
 
   size = end - start;
 
   /* And transfer the data from the frame buffer */
 
-  memcpy(buffer, fb->fbmem + start, size);
+  memcpy(buffer, panelinfo.fbmem + start, size);
   filep->f_pos += size;
   return size;
 }
@@ -149,9 +263,11 @@ static ssize_t fb_write(FAR struct file *filep, FAR const char *buffer,
 {
   FAR struct inode *inode;
   FAR struct fb_chardev_s *fb;
+  struct fb_panelinfo_s panelinfo;
   size_t start;
   size_t end;
   size_t size;
+  int ret;
 
   ginfo("len: %u\n", (unsigned int)len);
 
@@ -161,27 +277,36 @@ static ssize_t fb_write(FAR struct file *filep, FAR const char *buffer,
   inode = filep->f_inode;
   fb    = (FAR struct fb_chardev_s *)inode->i_private;
 
+  /* Get panel info */
+
+  ret = fb_get_panelinfo(fb, &panelinfo);
+
+  if (ret < 0)
+    {
+      return ret;
+    }
+
   fb->pollready = false;
 
   /* Get the start and size of the transfer */
 
   start = filep->f_pos;
-  if (start >= fb->fblen)
+  if (start >= panelinfo.fblen)
     {
       return -EFBIG;  /* Cannot extend the framebuffer */
     }
 
   end = start + len;
-  if (end >= fb->fblen)
+  if (end >= panelinfo.fblen)
     {
-      end = fb->fblen;
+      end = panelinfo.fblen;
     }
 
   size = end - start;
 
   /* And transfer the data into the frame buffer */
 
-  memcpy(fb->fbmem + start, buffer, size);
+  memcpy(panelinfo.fbmem + start, buffer, size);
   filep->f_pos += size;
   return size;
 }
@@ -200,6 +325,7 @@ static off_t fb_seek(FAR struct file *filep, off_t offset, int whence)
 {
   FAR struct inode *inode;
   FAR struct fb_chardev_s *fb;
+  struct fb_panelinfo_s panelinfo;
   off_t newpos;
   int ret;
 
@@ -224,7 +350,17 @@ static off_t fb_seek(FAR struct file *filep, off_t offset, int whence)
       break;
 
     case SEEK_END:
-      newpos = fb->fblen + offset;
+
+      /* Get panel info */
+
+      ret = fb_get_panelinfo(fb, &panelinfo);
+
+      if (ret < 0)
+        {
+          return ret;
+        }
+
+      newpos = panelinfo.fblen + offset;
       break;
 
     default:
@@ -384,9 +520,7 @@ static int fb_ioctl(FAR struct file *filep, int cmd, unsigned long arg)
           ret = fb->vtable->getoverlayinfo(fb->vtable, arg, &oinfo);
           if (ret == OK)
             {
-              fb->fbmem = oinfo.fbmem;
-              fb->fblen = oinfo.fblen;
-              fb->bpp   = oinfo.bpp;
+              fb->overlay = arg;
             }
         }
         break;
@@ -540,6 +674,13 @@ static int fb_ioctl(FAR struct file *filep, int cmd, unsigned long arg)
         }
         break;
 
+      case FBIOSET_VSYNCOFFSET:
+        {
+          fb->vsyncoffset = USEC2TICK(arg);
+          ret = OK;
+        }
+        break;
+
       case FBIOGET_VSCREENINFO:
         {
           struct fb_videoinfo_s vinfo;
@@ -674,8 +815,15 @@ static int fb_ioctl(FAR struct file *filep, int cmd, unsigned long arg)
         break;
 
       default:
-        gerr("ERROR: Unsupported IOCTL command: %d\n", cmd);
-        ret = -ENOTTY;
+        if (fb->vtable->ioctl != NULL)
+          {
+            ret = fb->vtable->ioctl(fb->vtable, cmd, arg);
+          }
+        else
+          {
+            gerr("ERROR: Unsupported IOCTL command: %d\n", cmd);
+            ret = -ENOTTY;
+          }
         break;
     }
 
@@ -686,7 +834,8 @@ static int fb_mmap(FAR struct file *filep, FAR struct mm_map_entry_s *map)
 {
   FAR struct inode *inode;
   FAR struct fb_chardev_s *fb;
-  int ret = -EINVAL;
+  struct fb_panelinfo_s panelinfo;
+  int ret;
 
   /* Get the framebuffer instance */
 
@@ -694,16 +843,25 @@ static int fb_mmap(FAR struct file *filep, FAR struct mm_map_entry_s *map)
   inode = filep->f_inode;
   fb    = (FAR struct fb_chardev_s *)inode->i_private;
 
-  /* Return the address corresponding to the start of frame buffer. */
+  /* Get panel info */
 
-  if (map->offset >= 0 && map->offset < fb->fblen &&
-      map->length && map->offset + map->length <= fb->fblen)
+  ret = fb_get_panelinfo(fb, &panelinfo);
+
+  if (ret < 0)
     {
-      map->vaddr = (FAR char *)fb->fbmem + map->offset;
-      ret = OK;
+      return ret;
     }
 
-  return ret;
+  /* Return the address corresponding to the start of frame buffer. */
+
+  if (map->offset >= 0 && map->offset < panelinfo.fblen &&
+      map->length && map->offset + map->length <= panelinfo.fblen)
+    {
+      map->vaddr = (FAR char *)panelinfo.fbmem + map->offset;
+      return OK;
+    }
+
+  return -EINVAL;
 }
 
 /****************************************************************************
@@ -751,6 +909,75 @@ static int fb_poll(FAR struct file *filep, struct pollfd *fds, bool setup)
   return OK;
 }
 
+/****************************************************************************
+ * Name: fb_get_panelinfo
+ ****************************************************************************/
+
+static int fb_get_panelinfo(FAR struct fb_chardev_s *fb,
+                            FAR struct fb_panelinfo_s *panelinfo)
+{
+  struct fb_planeinfo_s pinfo;
+  int ret;
+
+#ifdef CONFIG_FB_OVERLAY
+  if (fb->overlay != FB_NO_OVERLAY)
+    {
+      struct fb_overlayinfo_s oinfo;
+      DEBUGASSERT(fb->vtable->getoverlayinfo != NULL);
+      memset(&oinfo, 0, sizeof(oinfo));
+      ret = fb->vtable->getoverlayinfo(fb->vtable, fb->overlay, &oinfo);
+
+      if (ret < 0)
+        {
+          gerr("ERROR: getoverlayinfo() failed: %d\n", ret);
+          return ret;
+        }
+
+      panelinfo->fbmem  = oinfo.fbmem;
+      panelinfo->fblen  = oinfo.fblen;
+      panelinfo->bpp    = oinfo.bpp;
+      return OK;
+    }
+#endif
+
+  DEBUGASSERT(fb->vtable != NULL);
+  DEBUGASSERT(fb->vtable->getplaneinfo != NULL);
+  memset(&pinfo, 0, sizeof(pinfo));
+
+  ret = fb->vtable->getplaneinfo(fb->vtable, fb->plane, &pinfo);
+
+  if (ret < 0)
+    {
+      gerr("ERROR: getplaneinfo() failed: %d\n", ret);
+      return ret;
+    }
+
+  panelinfo->fbmem  = pinfo.fbmem;
+  panelinfo->fblen  = pinfo.fblen;
+  panelinfo->bpp    = pinfo.bpp;
+
+  return OK;
+}
+
+/****************************************************************************
+ * Name: fb_do_pollnotify
+ ****************************************************************************/
+
+static void fb_do_pollnotify(wdparm_t arg)
+{
+  FAR struct fb_chardev_s *fb = (FAR struct fb_chardev_s *)arg;
+
+  fb->pollready = true;
+
+  /* Notify framebuffer is writable. */
+
+  poll_notify(&fb->fds, 1, POLLOUT);
+}
+
+/****************************************************************************
+ * Public Functions
+ ****************************************************************************/
+
 /****************************************************************************
  * Name: fb_pollnotify
  *
@@ -770,17 +997,23 @@ void fb_pollnotify(FAR struct fb_vtable_s *vtable)
 
   fb = vtable->priv;
 
-  fb->pollready = true;
+  /* Prevent calling before getting the vtable. */
 
-  /* Notify framebuffer is writable. */
+  if (fb == NULL)
+    {
+      return;
+    }
 
-  poll_notify(&fb->fds, 1, POLLOUT);
+  if (fb->vsyncoffset > 0)
+    {
+      wd_start(&fb->wdog, fb->vsyncoffset, fb_do_pollnotify, (wdparm_t)fb);
+    }
+  else
+    {
+      fb_do_pollnotify((wdparm_t)fb);
+    }
 }
 
-/****************************************************************************
- * Public Functions
- ****************************************************************************/
-
 /****************************************************************************
  * Name: fb_register
  *
@@ -807,8 +1040,8 @@ void fb_pollnotify(FAR struct fb_vtable_s *vtable)
 int fb_register(int display, int plane)
 {
   FAR struct fb_chardev_s *fb;
+  struct fb_panelinfo_s panelinfo;
   struct fb_videoinfo_s vinfo;
-  struct fb_planeinfo_s pinfo;
 #ifdef CONFIG_FB_OVERLAY
   struct fb_overlayinfo_s oinfo;
 #endif
@@ -824,6 +1057,13 @@ int fb_register(int display, int plane)
       return -ENOMEM;
     }
 
+#ifdef CONFIG_FB_OVERLAY
+
+  /* Set the default overlay number */
+
+  fb->overlay = FB_NO_OVERLAY;
+#endif
+
   /* Initialize the frame buffer device. */
 
   ret = up_fbinitialize(display);
@@ -859,22 +1099,18 @@ int fb_register(int display, int plane)
   nplanes = vinfo.nplanes;
   DEBUGASSERT(vinfo.nplanes > 0 && (unsigned)plane < vinfo.nplanes);
 
-  DEBUGASSERT(fb->vtable->getplaneinfo != NULL);
-  memset(&pinfo, 0, sizeof(pinfo));
-  ret = fb->vtable->getplaneinfo(fb->vtable, plane, &pinfo);
+  /* Get panel info */
+
+  ret = fb_get_panelinfo(fb, &panelinfo);
+
   if (ret < 0)
     {
-      gerr("ERROR: getplaneinfo() failed: %d\n", ret);
       goto errout_with_fb;
     }
 
-  fb->fbmem  = pinfo.fbmem;
-  fb->fblen  = pinfo.fblen;
-  fb->bpp    = pinfo.bpp;
-
   /* Clear the framebuffer memory */
 
-  memset(pinfo.fbmem, 0, pinfo.fblen);
+  memset(panelinfo.fbmem, 0, panelinfo.fblen);
 
 #ifdef CONFIG_FB_OVERLAY
   /* Initialize first overlay but do not select */
@@ -906,7 +1142,9 @@ int fb_register(int display, int plane)
       snprintf(devname, 16, "/dev/fb%d.%d", display, plane);
     }
 
-  ret = register_driver(devname, &fb_fops, 0666, (FAR void *)fb);
+  nxmutex_init(&fb->lock);
+
+  ret = register_driver(devname, &g_fb_fops, 0666, (FAR void *)fb);
   if (ret < 0)
     {
       gerr("ERROR: register_driver() failed: %d\n", ret);
@@ -916,6 +1154,7 @@ int fb_register(int display, int plane)
   return OK;
 
 errout_with_fb:
+  nxmutex_destroy(&fb->lock);
   kmm_free(fb);
   return ret;
 }
