@@ -37,6 +37,14 @@
 #include <nuttx/mutex.h>
 #include <nuttx/sched.h>
 
+#ifdef CONFIG_FDSAN
+#  include <android/fdsan.h>
+#endif
+
+#ifdef CONFIG_FDCHECK
+#  include <nuttx/fdcheck.h>
+#endif
+
 #include "inode/inode.h"
 
 /****************************************************************************
@@ -185,7 +193,7 @@ void files_releaselist(FAR struct filelist *list)
 }
 
 /****************************************************************************
- * Name: file_allocate
+ * Name: file_allocate_from_tcb
  *
  * Description:
  *   Allocate a struct files instance and associate it with an inode
@@ -197,8 +205,9 @@ void files_releaselist(FAR struct filelist *list)
  *
  ****************************************************************************/
 
-int file_allocate(FAR struct inode *inode, int oflags, off_t pos,
-                  FAR void *priv, int minfd, bool addref)
+int file_allocate_from_tcb(FAR struct tcb_s *tcb, FAR struct inode *inode,
+                           int oflags, off_t pos, FAR void *priv, int minfd,
+                           bool addref)
 {
   FAR struct filelist *list;
   int ret;
@@ -207,7 +216,7 @@ int file_allocate(FAR struct inode *inode, int oflags, off_t pos,
 
   /* Get the file descriptor list.  It should not be NULL in this context. */
 
-  list = nxsched_get_files();
+  list = nxsched_get_files_from_tcb(tcb);
   DEBUGASSERT(list != NULL);
 
   ret = nxmutex_lock(&list->fl_lock);
@@ -253,7 +262,12 @@ int file_allocate(FAR struct inode *inode, int oflags, off_t pos,
                   inode_addref(inode);
                 }
 
+#ifdef CONFIG_FDCHECK
+              return
+                fdcheck_protect(i * CONFIG_NFILE_DESCRIPTORS_PER_BLOCK + j);
+#else
               return i * CONFIG_NFILE_DESCRIPTORS_PER_BLOCK + j;
+#endif
             }
         }
       while (++j < CONFIG_NFILE_DESCRIPTORS_PER_BLOCK);
@@ -282,7 +296,31 @@ int file_allocate(FAR struct inode *inode, int oflags, off_t pos,
       inode_addref(inode);
     }
 
+#ifdef CONFIG_FDCHECK
+  return fdcheck_protect(i * CONFIG_NFILE_DESCRIPTORS_PER_BLOCK);
+#else
   return i * CONFIG_NFILE_DESCRIPTORS_PER_BLOCK;
+#endif
+}
+
+/****************************************************************************
+ * Name: file_allocate
+ *
+ * Description:
+ *   Allocate a struct files instance and associate it with an inode
+ *   instance.
+ *
+ * Returned Value:
+ *     Returns the file descriptor == index into the files array on success;
+ *     a negated errno value is returned on any failure.
+ *
+ ****************************************************************************/
+
+int file_allocate(FAR struct inode *inode, int oflags, off_t pos,
+                  FAR void *priv, int minfd, bool addref)
+{
+  return file_allocate_from_tcb(nxsched_self(), inode, oflags,
+                                pos, priv, minfd, addref);
 }
 
 /****************************************************************************
@@ -382,6 +420,10 @@ int fs_getfilep(int fd, FAR struct file **filep)
   FAR struct filelist *list;
   int ret;
 
+#ifdef CONFIG_FDCHECK
+  fd = fdcheck_restore(fd);
+#endif
+
   DEBUGASSERT(filep != NULL);
   *filep = NULL;
 
@@ -432,14 +474,15 @@ int fs_getfilep(int fd, FAR struct file **filep)
 }
 
 /****************************************************************************
- * Name: nx_dup2
+ * Name: nx_dup2_from_tcb
  *
  * Description:
- *   nx_dup2() is similar to the standard 'dup2' interface except that is
- *   not a cancellation point and it does not modify the errno variable.
+ *   nx_dup2_from_tcb() is similar to the standard 'dup2' interface
+ *   except that is not a cancellation point and it does not modify the
+ *   errno variable.
  *
- *   nx_dup2() is an internal NuttX interface and should not be called from
- *   applications.
+ *   nx_dup2_from_tcb() is an internal NuttX interface and should not be
+ *   called from applications.
  *
  *   Clone a file descriptor to a specific descriptor number.
  *
@@ -449,11 +492,11 @@ int fs_getfilep(int fd, FAR struct file **filep)
  *
  ****************************************************************************/
 
-int nx_dup2(int fd1, int fd2)
+int nx_dup2_from_tcb(FAR struct tcb_s *tcb, int fd1, int fd2)
 {
   FAR struct filelist *list;
-  FAR struct file     *filep;
-  FAR struct file      file;
+  FAR struct file *filep;
+  FAR struct file  file;
   int ret;
 
   if (fd1 == fd2)
@@ -461,10 +504,14 @@ int nx_dup2(int fd1, int fd2)
       return fd1;
     }
 
-  /* Get the file descriptor list.  It should not be NULL in this context. */
+#ifdef CONFIG_FDCHECK
+  fd1 = fdcheck_restore(fd1);
+  fd2 = fdcheck_restore(fd2);
+#endif
 
-  list = nxsched_get_files();
-  DEBUGASSERT(list != NULL);
+  list = nxsched_get_files_from_tcb(tcb);
+
+  /* Get the file descriptor list.  It should not be NULL in this context. */
 
   if (fd1 < 0 || fd1 >= CONFIG_NFILE_DESCRIPTORS_PER_BLOCK * list->fl_rows ||
       fd2 < 0)
@@ -500,10 +547,43 @@ int nx_dup2(int fd1, int fd2)
   ret = file_dup2(&list->fl_files[fd1 / CONFIG_NFILE_DESCRIPTORS_PER_BLOCK]
                                  [fd1 % CONFIG_NFILE_DESCRIPTORS_PER_BLOCK],
                   filep);
+
+#ifdef CONFIG_FDSAN
+  filep->f_tag = file.f_tag;
+#endif
+
   nxmutex_unlock(&list->fl_lock);
 
   file_close(&file);
+
+#ifdef CONFIG_FDCHECK
+  return ret < 0 ? ret : fdcheck_protect(fd2);
+#else
   return ret < 0 ? ret : fd2;
+#endif
+}
+
+/****************************************************************************
+ * Name: nx_dup2
+ *
+ * Description:
+ *   nx_dup2() is similar to the standard 'dup2' interface except that is
+ *   not a cancellation point and it does not modify the errno variable.
+ *
+ *   nx_dup2() is an internal NuttX interface and should not be called from
+ *   applications.
+ *
+ *   Clone a file descriptor to a specific descriptor number.
+ *
+ * Returned Value:
+ *   fd2 is returned on success; a negated errno value is return on
+ *   any failure.
+ *
+ ****************************************************************************/
+
+int nx_dup2(int fd1, int fd2)
+{
+  return nx_dup2_from_tcb(nxsched_self(), fd1, fd2);
 }
 
 /****************************************************************************
@@ -530,14 +610,15 @@ int dup2(int fd1, int fd2)
 }
 
 /****************************************************************************
- * Name: nx_close
+ * Name: nx_close_from_tcb
  *
  * Description:
- *   nx_close() is similar to the standard 'close' interface except that is
- *   not a cancellation point and it does not modify the errno variable.
+ *   nx_close_from_tcb() is similar to the standard 'close' interface
+ *   except that is not a cancellation point and it does not modify the
+ *   errno variable.
  *
- *   nx_close() is an internal NuttX interface and should not be called from
- *   applications.
+ *   nx_close_from_tcb() is an internal NuttX interface and should not
+ *   be called from applications.
  *
  *   Close an inode (if open)
  *
@@ -551,19 +632,18 @@ int dup2(int fd1, int fd2)
  *
  ****************************************************************************/
 
-int nx_close(int fd)
+int nx_close_from_tcb(FAR struct tcb_s *tcb, int fd)
 {
-  FAR struct filelist *list;
   FAR struct file     *filep;
   FAR struct file      file;
+  FAR struct filelist *list;
   int                  ret;
 
-  /* Get the thread-specific file list.  It should never be NULL in this
-   * context.
-   */
+#ifdef CONFIG_FDCHECK
+  fd = fdcheck_restore(fd);
+#endif
 
-  list = nxsched_get_files();
-  DEBUGASSERT(list != NULL);
+  list = nxsched_get_files_from_tcb(tcb);
 
   /* Perform the protected close operation */
 
@@ -592,6 +672,33 @@ int nx_close(int fd)
   return file_close(&file);
 }
 
+/****************************************************************************
+ * Name: nx_close
+ *
+ * Description:
+ *   nx_close() is similar to the standard 'close' interface except that is
+ *   not a cancellation point and it does not modify the errno variable.
+ *
+ *   nx_close() is an internal NuttX interface and should not be called from
+ *   applications.
+ *
+ *   Close an inode (if open)
+ *
+ * Returned Value:
+ *   Zero (OK) is returned on success; A negated errno value is returned on
+ *   on any failure.
+ *
+ * Assumptions:
+ *   Caller holds the list mutex because the file descriptor will be
+ *   freed.
+ *
+ ****************************************************************************/
+
+int nx_close(int fd)
+{
+  return nx_close_from_tcb(nxsched_self(), fd);
+}
+
 /****************************************************************************
  * Name: close
  *
@@ -619,6 +726,10 @@ int close(int fd)
 {
   int ret;
 
+#ifdef CONFIG_FDSAN
+  android_fdsan_exchange_owner_tag(fd, 0, 0);
+#endif
+
   /* close() is a cancellation point */
 
   enter_cancellation_point();
