@@ -537,7 +537,7 @@ ssize_t nxffs_write(FAR struct file *filep, FAR const char *buffer,
   ret = nxmutex_lock(&volume->lock);
   if (ret < 0)
     {
-      ferr("ERROR: nxsem_wait failed: %d\n", ret);
+      ferr("ERROR: nxmutex_lock failed: %d\n", ret);
       goto errout;
     }
 
@@ -607,8 +607,8 @@ ssize_t nxffs_write(FAR struct file *filep, FAR const char *buffer,
 
   /* Success.. return the number of bytes written */
 
-  ret           = total;
-  filep->f_pos  = wrfile->datlen;
+  ret          = total;
+  filep->f_pos = wrfile->datlen;
 
 errout_with_lock:
   nxmutex_unlock(&volume->lock);
