@@ -62,7 +62,7 @@ static int shmfs_munmap(FAR struct task_group_s *group,
  * Public Data
  ****************************************************************************/
 
-const struct file_operations shmfs_operations =
+const struct file_operations g_shmfs_operations =
 {
   NULL,             /* open */
   shmfs_close,      /* close */
@@ -302,7 +302,7 @@ static int shmfs_mmap(FAR struct file *filep,
         {
           entry->munmap = shmfs_munmap;
           entry->priv.p = (FAR void *)filep->f_inode;
-          mm_map_add(entry);
+          mm_map_add(get_current_mm(), entry);
         }
     }
 
