@@ -447,6 +447,14 @@ struct pmsm_model_f32_s
   float                         iq_int; /* Iq integral part */
 };
 
+/* Average filter */
+
+struct avg_filter_data_s
+{
+  float prev_avg;      /* Previous average */
+  float k;             /* k counter */
+};
+
 /****************************************************************************
  * Public Functions Prototypes
  ****************************************************************************/
@@ -601,6 +609,12 @@ int pmsm_model_elec(FAR struct pmsm_model_f32_s *model,
                     FAR ab_frame_f32_t *vab);
 int pmsm_model_mech(FAR struct pmsm_model_f32_s *model, float load);
 
+/* Average filter */
+
+void avg_filter_data_init(FAR struct avg_filter_data_s *data,
+                          float prev_avg, float k);
+float avg_filter(FAR struct avg_filter_data_s *data, float x);
+
 #undef EXTERN
 #if defined(__cplusplus)
 }
