@@ -126,12 +126,14 @@
 #define IPV6_UNICAST_HOPS     (__SO_PROTOCOL + 6)  /* Unicast hop limit */
 #define IPV6_V6ONLY           (__SO_PROTOCOL + 7)  /* Restrict AF_INET6 socket
                                                     * to IPv6 communications only */
-#define IPV6_PKTINFO          (__SO_PROTOCOL + 8)  /* Get some information about
+#define IPV6_PKTINFO          (__SO_PROTOCOL + 8)  /* Information about the
+                                                    * incoming packet */
+#define IPV6_RECVPKTINFO      (__SO_PROTOCOL + 9)  /* Receive the information about
                                                     * the incoming packet */
-#define IPV6_RECVPKTINFO      (__SO_PROTOCOL + 9)  /* It functions just same as
-                                                    * IPV6_PKTINFO for now */
 #define IPV6_TCLASS           (__SO_PROTOCOL + 10) /* Access the Traffic Class
                                                     * field */
+#define IPV6_RECVHOPLIMIT     (__SO_PROTOCOL + 11) /* Access the hop limit field */
+#define IPV6_HOPLIMIT         (__SO_PROTOCOL + 12) /* Hop limit */
 
 /* Values used with SIOCSIFMCFILTER and SIOCGIFMCFILTER ioctl's */
 
