@@ -76,16 +76,6 @@
 #  define CONFIG_IOB_ALIGNMENT      1
 #endif
 
-#if !defined(CONFIG_IOB_HEADSIZE)
-#  define CONFIG_IOB_HEADSIZE       0
-#endif
-
-/* For backward compatibility when not using iob header padding */
-
-#if CONFIG_IOB_HEADSIZE == 0
-#  define  io_head  io_data
-#endif
-
 /* IOB helpers */
 
 #define IOB_DATA(p)      (&(p)->io_data[(p)->io_offset])
@@ -124,10 +114,7 @@ struct iob_s
 #endif
   unsigned int io_pktlen; /* Total length of the packet */
 
-#if CONFIG_IOB_HEADSIZE > 0
-  uint8_t  io_head[CONFIG_IOB_HEADSIZE];
-#endif
-  uint8_t  io_data[CONFIG_IOB_BUFSIZE] aligned_data(CONFIG_IOB_ALIGNMENT);
+  uint8_t  io_data[CONFIG_IOB_BUFSIZE];
 };
 
 #if CONFIG_IOB_NCHAINS > 0
@@ -418,7 +405,7 @@ unsigned int iob_get_queue_size(FAR struct iob_queue_s *queue);
  ****************************************************************************/
 
 int iob_copyin(FAR struct iob_s *iob, FAR const uint8_t *src,
-               unsigned int len, unsigned int offset, bool throttled);
+               unsigned int len, int offset, bool throttled);
 
 /****************************************************************************
  * Name: iob_trycopyin
@@ -431,7 +418,7 @@ int iob_copyin(FAR struct iob_s *iob, FAR const uint8_t *src,
  ****************************************************************************/
 
 int iob_trycopyin(FAR struct iob_s *iob, FAR const uint8_t *src,
-                  unsigned int len, unsigned int offset, bool throttled);
+                  unsigned int len, int offset, bool throttled);
 
 /****************************************************************************
  * Name: iob_copyout
@@ -443,7 +430,7 @@ int iob_trycopyin(FAR struct iob_s *iob, FAR const uint8_t *src,
  ****************************************************************************/
 
 int iob_copyout(FAR uint8_t *dest, FAR const struct iob_s *iob,
-                unsigned int len, unsigned int offset);
+                unsigned int len, int offset);
 
 /****************************************************************************
  * Name: iob_tailroom
@@ -464,8 +451,33 @@ unsigned int iob_tailroom(FAR struct iob_s *iob);
  *
  ****************************************************************************/
 
-int iob_clone(FAR struct iob_s *iob1,
-              FAR struct iob_s *iob2, bool throttled);
+int iob_clone(FAR struct iob_s *iob1, FAR struct iob_s *iob2,
+              bool throttled, bool block);
+
+/****************************************************************************
+ * Name: iob_clone_partial
+ *
+ * Description:
+ *   Duplicate the data from partial bytes of iob1 to iob2
+ *
+ * Input Parameters:
+ *   iob1      - Pointer to source iob_s
+ *   len       - Number of bytes to copy
+ *   offset1   - Offset of source iobs_s
+ *   iob2      - Pointer to destination iob_s
+ *   offset2   - Offset of destination iobs_s
+ *   throttled - An indication of the IOB allocation is "throttled"
+ *   block     - Flag of Enable/Disable nonblocking operation
+ *
+ * Returned Value:
+ *   == 0  - Partial clone successfully.
+ *   < 0   - No available to clone to destination iob.
+ *
+ ****************************************************************************/
+
+int iob_clone_partial(FAR struct iob_s *iob1, unsigned int len,
+                      int offset1, FAR struct iob_s *iob2,
+                      int offset2, bool throttled, bool block);
 
 /****************************************************************************
  * Name: iob_concat
@@ -568,6 +580,16 @@ void iob_reserve(FAR struct iob_s *iob, unsigned int reserved);
 
 void iob_update_pktlen(FAR struct iob_s *iob, unsigned int pktlen);
 
+/****************************************************************************
+ * Name: iob_count
+ *
+ * Description:
+ *   Get iob entries count in chain.
+ *
+ ****************************************************************************/
+
+int iob_count(FAR struct iob_s *iob);
+
 /****************************************************************************
  * Name: iob_dump
  *
