@@ -272,8 +272,6 @@ struct uart_dev_s
 
   uint8_t              open_count;   /* Number of times the device has been opened */
   uint8_t              escape;       /* Number of the character to be escaped */
-  volatile bool        xmitwaiting;  /* true: User waiting for space in xmit.buffer */
-  volatile bool        recvwaiting;  /* true: User waiting for data in recv.buffer */
 #ifdef CONFIG_SERIAL_REMOVABLE
   volatile bool        disconnected; /* true: Removable device is not connected */
 #endif
@@ -321,6 +319,12 @@ struct uart_dev_s
    * retained in the f_priv field of the 'struct file'.
    */
 
+#ifdef CONFIG_SERIAL_TERMIOS
+  uint8_t minrecv;                   /* Minimum received bytes */
+  uint8_t minread;                   /* c_cc[VMIN] */
+  uint8_t timeout;                   /* c_cc[VTIME] */
+#endif
+
   struct pollfd *fds[CONFIG_SERIAL_NPOLLWAITERS];
 };
 
