@@ -288,12 +288,15 @@
 
 #define FBIO_CLEARNOTIFY      _FBIOC(0x0017)  /* Clear notify signal */
 
+#define FBIOSET_VSYNCOFFSET   _FBIOC(0x0018)  /* Set VSync offset in usec
+                                               * Argument:             int */
+
 /* Linux Support ************************************************************/
 
-#define FBIOGET_VSCREENINFO   _FBIOC(0x0018)  /* Get video variable info */
+#define FBIOGET_VSCREENINFO   _FBIOC(0x0019)  /* Get video variable info */
                                               /* Argument: writable struct
                                                *           fb_var_screeninfo */
-#define FBIOGET_FSCREENINFO   _FBIOC(0x0019)  /* Get video fix info */
+#define FBIOGET_FSCREENINFO   _FBIOC(0x001a)  /* Get video fix info */
                                               /* Argument: writable struct
                                                *           fb_fix_screeninfo */
 
@@ -670,6 +673,11 @@ struct fb_vtable_s
   int (*getplaneinfo)(FAR struct fb_vtable_s *vtable, int planeno,
                       FAR struct fb_planeinfo_s *pinfo);
 
+  /* open/close window. */
+
+  int (*open)(FAR struct fb_vtable_s *vtable);
+  int (*close)(FAR struct fb_vtable_s *vtable);
+
 #ifdef CONFIG_FB_CMAP
   /* The following are provided only if the video hardware supports RGB
    * color mapping
@@ -750,7 +758,7 @@ struct fb_vtable_s
   int (*setarea)(FAR struct fb_vtable_s *vtable,
                  FAR const struct fb_overlayinfo_s *oinfo);
 
-# ifdef CONFIG_FB_OVERLAY_BLIT
+#  ifdef CONFIG_FB_OVERLAY_BLIT
   /* The following are provided only if the video hardware supports
    * blit operation between overlays.
    */
@@ -764,7 +772,7 @@ struct fb_vtable_s
 
   int (*blend)(FAR struct fb_vtable_s *vtable,
                FAR const struct fb_overlayblend_s *blend);
-# endif
+#  endif
 #endif
 
   /* Pan display for multiple buffers. */
@@ -790,6 +798,10 @@ struct fb_vtable_s
 
   int (*setpower)(FAR struct fb_vtable_s *vtable, int power);
 
+  /* Passthrough the unknown ioctl commands. */
+
+  int (*ioctl)(FAR struct fb_vtable_s *vtable, int cmd, unsigned long arg);
+
   /* Pointer to framebuffer device private data. */
 
   FAR void *priv;
