@@ -291,6 +291,29 @@ FAR const char *getprogname(void);
 int __cxa_atexit(CODE void (*func)(FAR void *), FAR void *arg,
                  FAR void *dso_handle);
 
+#if CONFIG_FORTIFY_SOURCE > 0
+fortify_function(realpath) FAR char *realpath(FAR const char *path,
+                                              FAR char *resolved)
+{
+  FAR char *ret = __real_realpath(path, resolved);
+  if (ret != NULL && resolved != NULL)
+    {
+      size_t len = 1;
+      FAR char *p;
+
+      p = ret;
+      while (*p++ != '\0')
+        {
+          len++;
+        }
+
+      fortify_assert(len <= fortify_size(resolved, 0));
+    }
+
+  return ret;
+}
+#endif
+
 #undef EXTERN
 #if defined(__cplusplus)
 }
