@@ -23,6 +23,7 @@
  ****************************************************************************/
 
 #include <sys/mman.h>
+#include <sys/stat.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <stdio.h>
@@ -32,8 +33,13 @@
  * Pre-processor Definitions
  ****************************************************************************/
 
-#define LIBC_MEM_FD_VFS_PATH \
-          CONFIG_LIBC_TMPDIR "/" CONFIG_LIBC_MEM_FD_VFS_PATH "/%s"
+#ifdef CONFIG_LIBC_MEMFD_TMPFS
+#  define LIBC_MEM_FD_VFS_PATH CONFIG_LIBC_TMPDIR "/" CONFIG_LIBC_MEM_FD_VFS_PATH
+#else
+#  define LIBC_MEM_FD_VFS_PATH CONFIG_LIBC_MEM_FD_VFS_PATH
+#endif
+
+#define LIBC_MEM_FD_VFS_PATH_FMT LIBC_MEM_FD_VFS_PATH "/%s"
 
 /****************************************************************************
  * Public Functions
@@ -41,13 +47,18 @@
 
 int memfd_create(FAR const char *name, unsigned int flags)
 {
-#ifdef CONFIG_FS_TMPFS
-  char path[PATH_MAX];
-
-  snprintf(path, sizeof(path), LIBC_MEM_FD_VFS_PATH, name);
-  return open(path, O_RDWR | flags);
-#else
+#ifdef CONFIG_LIBC_MEMFD_ERROR
   set_errno(ENOSYS);
   return -1;
+#else
+  char path[PATH_MAX];
+
+  snprintf(path, sizeof(path), LIBC_MEM_FD_VFS_PATH_FMT, name);
+#  ifdef CONFIG_LIBC_MEMFD_SHMFS
+  return shm_open(path, O_RDWR | flags, 0660);
+#  else
+  mkdir(LIBC_MEM_FD_VFS_PATH, 0666);
+  return open(path, O_RDWR | flags, 0660);
+#  endif
 #endif
 }
