@@ -89,7 +89,11 @@ int nxmutex_init(FAR mutex_t *mutex)
     }
 
   mutex->holder = NXMUTEX_NO_HOLDER;
+#ifdef CONFIG_PRIORITY_INHERITANCE
   _SEM_SETPROTOCOL(&mutex->sem, SEM_TYPE_MUTEX | SEM_PRIO_INHERIT);
+#else
+  _SEM_SETPROTOCOL(&mutex->sem, SEM_TYPE_MUTEX);
+#endif
   return ret;
 }
 
