@@ -24,6 +24,7 @@
 
 #include <nuttx/config.h>
 
+#include <sys/param.h>
 #include <sys/types.h>
 
 #include <stdint.h>
@@ -47,14 +48,6 @@
 #define ELF_ALIGNUP(a)   (((unsigned long)(a) + ELF_ALIGN_MASK) & ~ELF_ALIGN_MASK)
 #define ELF_ALIGNDOWN(a) ((unsigned long)(a) & ~ELF_ALIGN_MASK)
 
-#ifndef MAX
-#  define MAX(x,y) ((x) > (y) ? (x) : (y))
-#endif
-
-#ifndef MIN
-#  define MIN(x,y) ((x) < (y) ? (x) : (y))
-#endif
-
 /* _ALIGN_UP: 'a' is assumed to be a power of two */
 
 #define _ALIGN_UP(v, a) (((v) + ((a) - 1)) & ~((a) - 1))
