@@ -23,12 +23,15 @@
  ****************************************************************************/
 
 #include <strings.h>
+#include <syslog.h>
 #include <sys/param.h>
 
 #include <nuttx/mutex.h>
 #include <nuttx/kmalloc.h>
 #include <nuttx/mm/mempool.h>
 
+#include <assert.h>
+
 /****************************************************************************
  * Pre-processor Definitions
  ****************************************************************************/
@@ -49,19 +52,32 @@ struct mpool_dict_s
   size_t                size; /* Record expand memary size */
 };
 
+struct mpool_chunk_s
+{
+  sq_entry_t entry;
+  FAR void *start;
+  FAR void *next;
+  FAR void *end;
+  size_t used;
+};
+
 struct mempool_multiple_s
 {
-  FAR struct mempool_s    *pools;       /* The memory pool array */
-  size_t                   npools;      /* The number of memory pool array elements */
-  size_t                   expandsize;  /* The number not will use it to init erery
-                                         * pool expandsize
-                                         */
-  size_t                   minpoolsize; /* The number is align for each memory pool */
-  FAR void                *arg;         /* This pointer is used to store the user's
-                                         * private data
-                                         */
-  mempool_multiple_alloc_t alloc;       /* The alloc function for mempool */
-  mempool_multiple_free_t  free;        /* The free function for mempool */
+  FAR struct mempool_s         *pools;       /* The memory pool array */
+  size_t                        npools;      /* The number of memory pool array elements */
+  size_t                        expandsize;  /* The number not will use it to init erery
+                                              * pool expandsize
+                                              */
+  size_t                        minpoolsize; /* The number is align for each memory pool */
+  FAR void                     *arg;         /* This pointer is used to store the user's
+                                              * private data
+                                              */
+  mempool_multiple_alloc_t      alloc;       /* The alloc function for mempool */
+  mempool_multiple_alloc_size_t alloc_size;  /* Get the address size of the
+                                              * alloc function
+                                              */
+  mempool_multiple_free_t       free;        /* The free function for mempool */
+  size_t                        alloced;     /* Total size of alloc */
 
   /* This delta describes the relationship between the block size of each
    * mempool in multiple mempool by user initialized. It is automatically
@@ -70,17 +86,19 @@ struct mempool_multiple_s
    * arithmetic progressions, otherwise it is an increasing progressions.
    */
 
-  size_t                   delta;
+  size_t                        delta;
 
   /* It is used to record the information recorded by the mempool during
    * expansion, and find the mempool by adding an index
    */
 
-  mutex_t                   dict_lock;
-  size_t                    dict_used;
-  size_t                    dict_col_num_log2;
-  size_t                    dict_row_num;
-  FAR struct mpool_dict_s **dict;
+  mutex_t                       lock;
+  sq_queue_t                    chunk_queue;
+  size_t                        chunk_size;
+  size_t                        dict_used;
+  size_t                        dict_col_num_log2;
+  size_t                        dict_row_num;
+  FAR struct mpool_dict_s     **dict;
 };
 
 /****************************************************************************
@@ -133,6 +151,90 @@ mempool_multiple_find(FAR struct mempool_multiple_s *mpool, size_t size)
   return &mpool->pools[left];
 }
 
+static FAR void *
+mempool_multiple_alloc_chunk(FAR struct mempool_multiple_s *mpool,
+                             size_t align, size_t size)
+{
+  FAR struct mpool_chunk_s *chunk;
+  FAR char *tmp;
+  FAR void *ret;
+
+  if (mpool->chunk_size < mpool->expandsize)
+    {
+      ret = mpool->alloc(mpool->arg, align, size);
+      if (ret)
+        {
+          mpool->alloced += mpool->alloc_size(mpool->arg, ret);
+        }
+
+      return ret;
+    }
+
+  chunk = (FAR struct mpool_chunk_s *)sq_peek(&mpool->chunk_queue);
+  if (chunk == NULL)
+    {
+retry:
+      tmp = mpool->alloc(mpool->arg, mpool->expandsize,
+                         mpool->chunk_size +
+                         sizeof(struct mpool_chunk_s));
+
+      if (tmp == NULL)
+        {
+          return NULL;
+        }
+
+      mpool->alloced += mpool->alloc_size(mpool->arg, tmp);
+      chunk = (FAR struct mpool_chunk_s *)(tmp + mpool->chunk_size);
+      chunk->end = tmp + mpool->chunk_size;
+      chunk->start = tmp;
+      chunk->next = tmp;
+      chunk->used = 0;
+      sq_addfirst(&chunk->entry, &mpool->chunk_queue);
+    }
+
+  ret = (FAR void *)ALIGN_UP(chunk->next, align);
+  if (chunk->end - ret < size)
+    {
+      goto retry;
+    }
+
+  chunk->used++;
+  chunk->next = ret + size;
+  return ret;
+}
+
+static void
+mempool_multiple_free_chunk(FAR struct mempool_multiple_s *mpool,
+                            FAR void *ptr)
+{
+  FAR struct mpool_chunk_s *chunk;
+  FAR sq_entry_t *entry;
+
+  if (mpool->chunk_size < mpool->expandsize)
+    {
+      mpool->free(mpool->arg, ptr);
+      return;
+    }
+
+  nxmutex_lock(&mpool->lock);
+  sq_for_every(&mpool->chunk_queue, entry)
+    {
+      chunk = (FAR struct mpool_chunk_s *)entry;
+      if (ptr >= chunk->start && ptr < chunk->next)
+        {
+          if (--chunk->used == 0)
+            {
+              sq_rem(&chunk->entry, &mpool->chunk_queue);
+              mpool->free(mpool->arg, chunk->start);
+            }
+
+          break;
+        }
+    }
+
+  nxmutex_unlock(&mpool->lock);
+}
+
 static FAR void *mempool_multiple_alloc_callback(FAR struct mempool_s *pool,
                                                  size_t size)
 {
@@ -141,14 +243,15 @@ static FAR void *mempool_multiple_alloc_callback(FAR struct mempool_s *pool,
   size_t row;
   size_t col;
 
-  ret = mpool->alloc(mpool->arg, mpool->expandsize,
-                     mpool->minpoolsize + size);
+  nxmutex_lock(&mpool->lock);
+  ret = mempool_multiple_alloc_chunk(mpool, mpool->expandsize,
+                                     mpool->minpoolsize + size);
   if (ret == NULL)
     {
+      nxmutex_unlock(&mpool->lock);
       return NULL;
     }
 
-  nxmutex_lock(&mpool->dict_lock);
   row = mpool->dict_used >> mpool->dict_col_num_log2;
 
   /* There is no new pointer address to store the dictionarys */
@@ -159,16 +262,17 @@ static FAR void *mempool_multiple_alloc_callback(FAR struct mempool_s *pool,
 
   if (mpool->dict[row] == NULL)
     {
-      mpool->dict[row] = mpool->alloc(mpool->arg, sizeof(uintptr_t),
-                                      (1 << mpool->dict_col_num_log2) *
-                                      sizeof(struct mpool_dict_s));
+      mpool->dict[row] =
+        mempool_multiple_alloc_chunk(mpool, sizeof(uintptr_t),
+                                     (1 << mpool->dict_col_num_log2)
+                                     * sizeof(struct mpool_dict_s));
     }
 
   mpool->dict[row][col].pool = pool;
   mpool->dict[row][col].addr = ret;
   mpool->dict[row][col].size = mpool->minpoolsize + size;
   *(FAR size_t *)ret = mpool->dict_used++;
-  nxmutex_unlock(&mpool->dict_lock);
+  nxmutex_unlock(&mpool->lock);
   return (FAR char *)ret + mpool->minpoolsize;
 }
 
@@ -177,7 +281,8 @@ static void mempool_multiple_free_callback(FAR struct mempool_s *pool,
 {
   FAR struct mempool_multiple_s *mpool = pool->priv;
 
-  mpool->free(mpool->arg, (FAR char *)addr - mpool->minpoolsize);
+  mempool_multiple_free_chunk(mpool,
+                              (FAR char *)addr - mpool->minpoolsize);
 }
 
 /****************************************************************************
@@ -251,11 +356,12 @@ mempool_multiple_get_dict(FAR struct mempool_multiple_s *mpool,
  *   poolsize        - The block size array for pools in multiples pool.
  *   npools          - How many pools in multiples pool.
  *   alloc           - The alloc memory function for multiples pool.
+ *   alloc_size      - Get the address size of the alloc function.
  *   free            - The free memory function for multiples pool.
  *   arg             - The alloc & free memory fuctions used arg.
+ *   chunksize       - The multiples pool chunk size.
  *   expandsize      - The expend mempry for all pools in multiples pool.
  *   dict_expendsize - The expend size for multiple dictnoary.
- *   calibrate       - Whether to calibrate when counting memory usage.
  * Returned Value:
  *   Return an initialized multiple pool pointer on success,
  *   otherwise NULL is returned.
@@ -266,9 +372,10 @@ FAR struct mempool_multiple_s *
 mempool_multiple_init(FAR const char *name,
                       FAR size_t *poolsize, size_t npools,
                       mempool_multiple_alloc_t alloc,
-                      mempool_multiple_free_t free,
-                      FAR void *arg, size_t expandsize,
-                      size_t dict_expendsize, bool calibrate)
+                      mempool_multiple_alloc_size_t alloc_size,
+                      mempool_multiple_free_t free, FAR void *arg,
+                      size_t chunksize, size_t expandsize,
+                      size_t dict_expendsize)
 {
   FAR struct mempool_multiple_s *mpool;
   FAR struct mempool_s *pools;
@@ -303,8 +410,16 @@ mempool_multiple_init(FAR const char *name,
       return NULL;
     }
 
-  pools = alloc(arg, sizeof(uintptr_t),
-                npools * sizeof(FAR struct mempool_s));
+  mpool->alloc_size = alloc_size;
+  mpool->expandsize = expandsize;
+  mpool->chunk_size = chunksize;
+  mpool->alloc = alloc;
+  mpool->free = free;
+  mpool->arg = arg;
+  mpool->alloced = alloc_size(arg, mpool);
+  sq_init(&mpool->chunk_queue);
+  pools = mempool_multiple_alloc_chunk(mpool, sizeof(uintptr_t),
+                                       npools * sizeof(struct mempool_s));
   if (pools == NULL)
     {
       goto err_with_mpool;
@@ -312,11 +427,8 @@ mempool_multiple_init(FAR const char *name,
 
   mpool->pools = pools;
   mpool->npools = npools;
-  mpool->expandsize = expandsize;
   mpool->minpoolsize = minpoolsize;
-  mpool->alloc = alloc;
-  mpool->free = free;
-  mpool->arg = arg;
+  mpool->delta = 0;
 
   for (i = 0; i < npools; i++)
     {
@@ -327,7 +439,6 @@ mempool_multiple_init(FAR const char *name,
       pools[i].priv = mpool;
       pools[i].alloc = mempool_multiple_alloc_callback;
       pools[i].free = mempool_multiple_free_callback;
-      pools[i].calibrate = calibrate;
 #if CONFIG_MM_BACKTRACE >= 0
       pools[i].blockalign = mpool->minpoolsize;
 #endif
@@ -339,7 +450,7 @@ mempool_multiple_init(FAR const char *name,
 
       if (i + 1 != npools)
         {
-          size_t delta = pools[i + 1].blocksize - pools[i].blocksize;
+          size_t delta = poolsize[i + 1] - poolsize[i];
 
           if (i == 0)
             {
@@ -356,17 +467,18 @@ mempool_multiple_init(FAR const char *name,
   mpool->dict_col_num_log2 = fls(dict_expendsize /
                                  sizeof(struct mpool_dict_s));
 
-  mpool->dict_row_num = dict_expendsize / sizeof(struct mpool_dict_s *);
-  mpool->dict = alloc(arg, sizeof(struct mpool_dict_s *),
-                      sizeof(struct mpool_dict_s *) * mpool->dict_row_num);
+  mpool->dict_row_num = dict_expendsize / sizeof(FAR struct mpool_dict_s *);
+  mpool->dict = mempool_multiple_alloc_chunk(
+                mpool, sizeof(FAR struct mpool_dict_s *),
+                sizeof(FAR struct mpool_dict_s *) * mpool->dict_row_num);
   if (mpool->dict == NULL)
     {
       goto err_with_pools;
     }
 
   memset(mpool->dict, 0,
-         mpool->dict_row_num * sizeof(struct mpool_dict_s *));
-  nxmutex_init(&mpool->dict_lock);
+         mpool->dict_row_num * sizeof(FAR struct mpool_dict_s *));
+  nxmutex_init(&mpool->lock);
 
   return mpool;
 
@@ -376,7 +488,7 @@ err_with_pools:
       mempool_deinit(pools + i);
     }
 
-  free(arg, pools);
+  mempool_multiple_free_chunk(mpool, pools);
 err_with_mpool:
   free(arg, mpool);
   return NULL;
@@ -406,11 +518,6 @@ FAR void *mempool_multiple_alloc(FAR struct mempool_multiple_s *mpool,
   FAR struct mempool_s *end;
   FAR struct mempool_s *pool;
 
-  if (size < 1)
-    {
-      return NULL;
-    }
-
   pool = mempool_multiple_find(mpool, size);
   if (pool == NULL)
     {
@@ -459,12 +566,6 @@ FAR void *mempool_multiple_realloc(FAR struct mempool_multiple_s *mpool,
       return mempool_multiple_alloc(mpool, size);
     }
 
-  if (size < 1)
-    {
-      mempool_multiple_free(mpool, oldblk);
-      return NULL;
-    }
-
   dict = mempool_multiple_get_dict(mpool, oldblk);
   if (dict == NULL)
     {
@@ -603,18 +704,87 @@ FAR void *mempool_multiple_memalign(FAR struct mempool_multiple_s *mpool,
 }
 
 /****************************************************************************
- * Name: mempool_multiple_info_task
+ * Name: mempool_multiple_foreach
  ****************************************************************************/
 
-void mempool_multiple_info_task(FAR struct mempool_multiple_s *mpool,
-                                FAR struct mempoolinfo_task *info)
+void mempool_multiple_foreach(FAR struct mempool_multiple_s *mpool,
+                              mempool_multiple_foreach_t handle,
+                              FAR void *arg)
 {
   size_t i;
+  for (i = 0; i < mpool->npools; i++)
+    {
+      handle(mpool->pools + i, arg);
+    }
+}
+
+/****************************************************************************
+ * Name: mempool_multiple_mallinfo
+ ****************************************************************************/
+
+struct mallinfo
+mempool_multiple_mallinfo(FAR struct mempool_multiple_s *mpool)
+{
+  struct mallinfo info;
+  size_t i;
+
+  memset(&info, 0, sizeof(struct mallinfo));
+
+  nxmutex_lock(&mpool->lock);
+  info.arena = mpool->alloced;
+
+  if (mpool->chunk_size >= mpool->expandsize)
+    {
+      FAR struct mpool_chunk_s *chunk;
+
+      chunk = (FAR struct mpool_chunk_s *)sq_peek(&mpool->chunk_queue);
+      info.fordblks += chunk->end - chunk->next;
+    }
+
+  nxmutex_unlock(&mpool->lock);
 
   for (i = 0; i < mpool->npools; i++)
     {
-      mempool_info_task(mpool->pools + i, info);
+      struct mempoolinfo_s poolinfo;
+
+      mempool_info(mpool->pools + i, &poolinfo);
+      info.fordblks += (poolinfo.ordblks + poolinfo.iordblks)
+                       * poolinfo.sizeblks;
+      info.ordblks += poolinfo.ordblks + poolinfo.iordblks;
+      info.aordblks += poolinfo.aordblks;
+      if (info.mxordblk < poolinfo.sizeblks)
+        {
+          info.mxordblk = poolinfo.sizeblks;
+        }
     }
+
+  info.uordblks += mpool->alloced - info.fordblks;
+  return info;
+}
+
+/****************************************************************************
+ * Name: mempool_multiple_info_task
+ ****************************************************************************/
+
+struct mallinfo_task
+mempool_multiple_info_task(FAR struct mempool_multiple_s *mpool,
+                           FAR const struct malltask *task)
+{
+  int i;
+  struct mallinfo_task info;
+  struct mallinfo_task ret =
+    {
+      0, 0
+    };
+
+  for (i = 0; i < mpool->npools; i++)
+    {
+      info = mempool_info_task(mpool->pools + i, task);
+      ret.aordblks += info.aordblks;
+      ret.uordblks += info.uordblks;
+    }
+
+  return ret;
 }
 
 /****************************************************************************
@@ -630,18 +800,18 @@ void mempool_multiple_info_task(FAR struct mempool_multiple_s *mpool,
  *
  * Input Parameters:
  *   mpool - The handle of multiple memory pool to be used.
- *   pid   - The pid of task.
+ *   dump  - The information of what need dump.
  *
  ****************************************************************************/
 
 void mempool_multiple_memdump(FAR struct mempool_multiple_s *mpool,
-                              pid_t pid)
+                              FAR const struct mm_memdump_s *dump)
 {
   size_t i;
 
   for (i = 0; i < mpool->npools; i++)
     {
-      mempool_memdump(mpool->pools + i, pid);
+      mempool_memdump(mpool->pools + i, dump);
     }
 }
 
@@ -671,7 +841,7 @@ void mempool_multiple_deinit(FAR struct mempool_multiple_s *mpool)
     {
       if (mpool->dict[i] != NULL)
         {
-          mpool->free(mpool->arg, mpool->dict[i]);
+          mempool_multiple_free_chunk(mpool, mpool->dict[i]);
         }
       else
         {
@@ -679,8 +849,8 @@ void mempool_multiple_deinit(FAR struct mempool_multiple_s *mpool)
         }
     }
 
-  mpool->free(mpool->arg, mpool->dict);
-  mpool->free(mpool->arg, mpool->pools);
-  mpool->free(mpool->arg, mpool);
-  nxmutex_destroy(&mpool->dict_lock);
+  mempool_multiple_free_chunk(mpool, mpool->dict);
+  mempool_multiple_free_chunk(mpool, mpool->pools);
+  nxmutex_destroy(&mpool->lock);
+  mpool->free(mpool, mpool);
 }
