@@ -41,6 +41,40 @@
 #  define MEMPOOL_NPOOLS (CONFIG_MM_HEAP_MEMPOOL_THRESHOLD / MM_MIN_CHUNK)
 #endif
 
+/****************************************************************************
+ * Private Functions
+ ****************************************************************************/
+
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0 && CONFIG_MM_BACKTRACE >= 0
+
+/****************************************************************************
+ * Name: mempool_memalign
+ *
+ * Description:
+ *   This function call mm_memalign and set mm_backtrace pid to free pid
+ *   avoid repeated calculation.
+ ****************************************************************************/
+
+static FAR void *mempool_memalign(FAR void *arg, size_t alignment,
+                                  size_t size)
+{
+  FAR struct mm_allocnode_s *node;
+  FAR void *ret;
+
+  ret = mm_memalign(arg, alignment, size);
+  if (ret)
+    {
+      node = (FAR struct mm_allocnode_s *)
+      ((FAR char *)ret - SIZEOF_MM_ALLOCNODE);
+      node->pid = PID_MM_MEMPOOL;
+    }
+
+  return ret;
+}
+#else
+#  define mempool_memalign mm_memalign
+#endif
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
@@ -83,7 +117,7 @@ void mm_addregion(FAR struct mm_heap_s *heap, FAR void *heapstart,
     }
 
 #else
-# define IDX 0
+#  define IDX 0
 #endif
 
 #if defined(CONFIG_MM_SMALL) && !defined(CONFIG_SMALL_MEMORY)
@@ -253,11 +287,12 @@ FAR struct mm_heap_s *mm_initialize(FAR const char *name,
     }
 
   heap->mm_mpool = mempool_multiple_init(name, poolsize, MEMPOOL_NPOOLS,
-                                  (mempool_multiple_alloc_t)mm_memalign,
-                                  (mempool_multiple_free_t)mm_free, heap,
-                                  CONFIG_MM_HEAP_MEMPOOL_EXPAND,
-                                  CONFIG_MM_HEAP_MEMPOOL_DICTIONARY_EXPAND,
-                                  true);
+                              (mempool_multiple_alloc_t)mempool_memalign,
+                              (mempool_multiple_alloc_size_t)mm_malloc_size,
+                              (mempool_multiple_free_t)mm_free, heap,
+                              CONFIG_MM_HEAP_MEMPOOL_CHUNK_SIZE,
+                              CONFIG_MM_HEAP_MEMPOOL_EXPAND_SIZE,
+                              CONFIG_MM_HEAP_MEMPOOL_DICTIONARY_EXPAND_SIZE);
 #endif
 
   return heap;
