@@ -24,7 +24,6 @@
 
 #include <nuttx/config.h>
 
-#include <malloc.h>
 #include <assert.h>
 #include <debug.h>
 
@@ -32,6 +31,16 @@
 
 #include "mm_heap/mm.h"
 
+/****************************************************************************
+ * Private Types
+ ****************************************************************************/
+
+struct mm_mallinfo_handler_s
+{
+  FAR const struct malltask *task;
+  FAR struct mallinfo_task *info;
+};
+
 /****************************************************************************
  * Private Functions
  ****************************************************************************/
@@ -78,7 +87,9 @@ static void mallinfo_handler(FAR struct mm_allocnode_s *node, FAR void *arg)
 static void mallinfo_task_handler(FAR struct mm_allocnode_s *node,
                                   FAR void *arg)
 {
-  FAR struct mallinfo_task *info = arg;
+  FAR struct mm_mallinfo_handler_s *handler = arg;
+  FAR const struct malltask *task = handler->task;
+  FAR struct mallinfo_task *info = handler->info;
   size_t nodesize = SIZEOF_MM_NODE(node);
 
   /* Check if the node corresponds to an allocated memory chunk */
@@ -87,16 +98,23 @@ static void mallinfo_task_handler(FAR struct mm_allocnode_s *node,
     {
       DEBUGASSERT(nodesize >= SIZEOF_MM_ALLOCNODE);
 #if CONFIG_MM_BACKTRACE < 0
-      if (info->pid == -1)
-#else
-      if (info->pid == -1 || node->pid == info->pid)
-#endif
+      if (task->pid == PID_MM_ALLOC)
         {
           info->aordblks++;
           info->uordblks += nodesize;
         }
+#else
+      if ((task->pid == node->pid ||
+           (task->pid == PID_MM_ALLOC && node->pid != PID_MM_MEMPOOL) ||
+           (task->pid == PID_MM_LEAK && !!nxsched_get_tcb(node->pid))) &&
+          node->seqno >= task->seqmin && node->seqno <= task->seqmax)
+        {
+          info->aordblks++;
+          info->uordblks += nodesize;
+        }
+#endif
     }
-  else if (info->pid == -2)
+  else if (task->pid == PID_MM_FREE)
     {
       info->aordblks++;
       info->uordblks += nodesize;
@@ -115,33 +133,29 @@ static void mallinfo_task_handler(FAR struct mm_allocnode_s *node,
  *
  ****************************************************************************/
 
-int mm_mallinfo(FAR struct mm_heap_s *heap, FAR struct mallinfo *info)
+struct mallinfo mm_mallinfo(FAR struct mm_heap_s *heap)
 {
-#if CONFIG_MM_REGIONS > 1
-  int region = heap->mm_nregions;
-#else
-# define region 1
+  struct mallinfo info;
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
+  struct mallinfo poolinfo;
 #endif
 
-  DEBUGASSERT(info);
+  memset(&info, 0, sizeof(info));
+  mm_foreach(heap, mallinfo_handler, &info);
+  info.arena = heap->mm_heapsize;
+  info.arena += sizeof(struct mm_heap_s);
+  info.uordblks += sizeof(struct mm_heap_s);
 
-  memset(info, 0, sizeof(*info));
-  mm_foreach(heap, mallinfo_handler, info);
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
+  poolinfo = mempool_multiple_mallinfo(heap->mm_mpool);
 
-  info->arena = heap->mm_heapsize;
+  info.uordblks -= poolinfo.fordblks;
+  info.fordblks += poolinfo.fordblks;
+#endif
 
-  /* Account for the heap->mm_heapend[region] node overhead and the
-   * heap->mm_heapstart[region]->preceding:
-   * heap->mm_heapend[region] overhead size     = OVERHEAD_MM_ALLOCNODE
-   * heap->mm_heapstart[region]->preceding size = sizeof(mmsize_t)
-   * and SIZEOF_MM_ALLOCNODE = OVERHEAD_MM_ALLOCNODE + sizeof(mmsize_t).
-   */
+  DEBUGASSERT(info.uordblks + info.fordblks == info.arena);
 
-  info->uordblks += region * SIZEOF_MM_ALLOCNODE;
-
-  DEBUGASSERT((size_t)info->uordblks + info->fordblks == heap->mm_heapsize);
-
-  return OK;
+  return info;
 }
 
 /****************************************************************************
@@ -153,18 +167,22 @@ int mm_mallinfo(FAR struct mm_heap_s *heap, FAR struct mallinfo *info)
  *
  ****************************************************************************/
 
-int mm_mallinfo_task(FAR struct mm_heap_s *heap,
-                     FAR struct mallinfo_task *info)
+struct mallinfo_task mm_mallinfo_task(FAR struct mm_heap_s *heap,
+                                      FAR const struct malltask *task)
 {
-  DEBUGASSERT(info);
-
-  info->uordblks = 0;
-  info->aordblks = 0;
+  struct mm_mallinfo_handler_s handle;
+  struct mallinfo_task info =
+    {
+      0, 0
+    };
 
 #if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
-  mempool_multiple_info_task(heap->mm_mpool, info);
+  info = mempool_multiple_info_task(heap->mm_mpool, task);
 #endif
 
-  mm_foreach(heap, mallinfo_task_handler, info);
-  return OK;
+  handle.task = task;
+  handle.info = &info;
+  mm_foreach(heap, mallinfo_task_handler, &handle);
+
+  return info;
 }
