@@ -43,7 +43,7 @@
  *   within that chunk that meets the alignment request and then frees any
  *   leading or trailing space.
  *
- *   The alignment argument must be a power of two.  8-byte alignment is
+ *   The alignment argument must be a power of two. 16-byte alignment is
  *   guaranteed by normal malloc calls.
  *
  ****************************************************************************/
@@ -52,9 +52,9 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
                       size_t size)
 {
   FAR struct mm_allocnode_s *node;
-  size_t rawchunk;
-  size_t alignedchunk;
-  size_t mask = (size_t)(alignment - 1);
+  uintptr_t rawchunk;
+  uintptr_t alignedchunk;
+  size_t mask;
   size_t allocsize;
   size_t newsize;
 
@@ -72,20 +72,34 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
       return NULL;
     }
 
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
+  node = mempool_multiple_memalign(heap->mm_mpool, alignment, size);
+  if (node != NULL)
+    {
+      return node;
+    }
+#endif
+
   /* If this requested alinement's less than or equal to the natural
    * alignment of malloc, then just let malloc do the work.
    */
 
-  if (alignment <= MM_MIN_CHUNK)
+  if (alignment <= MM_ALIGN)
     {
       FAR void *ptr = mm_malloc(heap, size);
       DEBUGASSERT(ptr == NULL || ((uintptr_t)ptr) % alignment == 0);
       return ptr;
     }
+  else if (alignment < MM_MIN_CHUNK)
+    {
+      alignment = MM_MIN_CHUNK;
+    }
 
-  /* Adjust the size to account for (1) the size of the allocated node, (2)
-   * to make sure that it is an even multiple of our granule size, and to
-   * include the alignment amount.
+  mask = alignment - 1;
+
+  /* Adjust the size to account for (1) the size of the allocated node and
+   * (2) to make sure that it is aligned with MM_ALIGN and its size is at
+   * least MM_MIN_CHUNK.
    *
    * Notice that we increase the allocation size by twice the requested
    * alignment.  We do this so that there will be at least two valid
@@ -95,11 +109,15 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
    * not include SIZEOF_MM_ALLOCNODE.
    */
 
-  newsize  = MM_ALIGN_UP(size);   /* Make multiples of our granule size */
+  if (size < MM_MIN_CHUNK - OVERHEAD_MM_ALLOCNODE)
+    {
+      size = MM_MIN_CHUNK - OVERHEAD_MM_ALLOCNODE;
+    }
 
+  newsize = MM_ALIGN_UP(size);         /* Make multiples of our granule size */
   allocsize = newsize + 2 * alignment; /* Add double full alignment size */
 
-  if ((newsize < size) || (allocsize < newsize))
+  if (newsize < size || allocsize < newsize)
     {
       /* Integer overflow */
 
@@ -108,13 +126,14 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
 
   /* Then malloc that size */
 
-  rawchunk = (size_t)mm_malloc(heap, allocsize);
+  rawchunk = (uintptr_t)mm_malloc(heap, allocsize);
   if (rawchunk == 0)
     {
       return NULL;
     }
 
-  kasan_poison((FAR void *)rawchunk, mm_malloc_size((FAR void *)rawchunk));
+  kasan_poison((FAR void *)rawchunk,
+               mm_malloc_size(heap, (FAR void *)rawchunk));
 
   /* We need to hold the MM mutex while we muck with the chunks and
    * nodelist.
@@ -139,17 +158,12 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
       FAR struct mm_allocnode_s *newnode;
       FAR struct mm_allocnode_s *next;
       size_t precedingsize;
+      size_t newnodesize;
 
       /* Get the node the next node after the allocation. */
 
-      next = (FAR struct mm_allocnode_s *)((FAR char *)node + node->size);
-
-      /* Make sure that there is space to convert the preceding
-       * mm_allocnode_s into an mm_freenode_s.  I think that this should
-       * always be true
-       */
-
-      DEBUGASSERT(alignedchunk >= rawchunk + 8);
+      next = (FAR struct mm_allocnode_s *)
+        ((FAR char *)node + SIZEOF_MM_NODE(node));
 
       newnode = (FAR struct mm_allocnode_s *)
         (alignedchunk - SIZEOF_MM_ALLOCNODE);
@@ -158,7 +172,7 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
        * SIZEOF_MM_ALLOCNODE
        */
 
-      precedingsize = (size_t)newnode - (size_t)node;
+      precedingsize = (uintptr_t)newnode - (uintptr_t)node;
 
       /* If we were unlucky, then the alignedchunk can lie in such a position
        * that precedingsize < SIZEOF_NODE_FREENODE.  We can't let that happen
@@ -168,33 +182,55 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
        * alignment point.
        */
 
-      if (precedingsize < SIZEOF_MM_FREENODE)
+      if (precedingsize < MM_MIN_CHUNK)
         {
           alignedchunk += alignment;
           newnode       = (FAR struct mm_allocnode_s *)
                           (alignedchunk - SIZEOF_MM_ALLOCNODE);
-          precedingsize = (size_t)newnode - (size_t)node;
+          precedingsize = (uintptr_t)newnode - (uintptr_t)node;
         }
 
+      /* If the previous node is free, merge node and previous node, then
+       * set up the node size.
+       */
+
+      if ((node->size & MM_PREVFREE_BIT) != 0)
+        {
+          FAR struct mm_freenode_s *prev =
+            (FAR struct mm_freenode_s *)((FAR char *)node - node->preceding);
+
+          /* Remove the node.  There must be a predecessor, but there may
+           * not be a successor node.
+           */
+
+          DEBUGASSERT(prev->blink);
+          prev->blink->flink = prev->flink;
+          if (prev->flink)
+            {
+              prev->flink->blink = prev->blink;
+            }
+
+          precedingsize += SIZEOF_MM_NODE(prev);
+          node = (FAR struct mm_allocnode_s *)prev;
+        }
+
+      node->size = precedingsize;
+
       /* Set up the size of the new node */
 
-      newnode->size = (size_t)next - (size_t)newnode;
-      newnode->preceding = precedingsize | MM_ALLOC_BIT;
+      newnodesize = (uintptr_t)next - (uintptr_t)newnode;
+      newnode->size = newnodesize | MM_ALLOC_BIT | MM_PREVFREE_BIT;
+      newnode->preceding = precedingsize;
 
-      /* Reduce the size of the original chunk and mark it not allocated, */
+      /* Clear the previous free bit of the next node */
 
-      node->size = precedingsize;
-      node->preceding &= ~MM_MASK_BIT;
-
-      /* Fix the preceding size of the next node */
-
-      next->preceding = newnode->size | (next->preceding & MM_ALLOC_BIT);
+      next->size &= ~MM_PREVFREE_BIT;
 
       /* Convert the newnode chunk size back into malloc-compatible size by
-       * subtracting the header size SIZEOF_MM_ALLOCNODE.
+       * subtracting the header size OVERHEAD_MM_ALLOCNODE.
        */
 
-      allocsize = newnode->size - SIZEOF_MM_ALLOCNODE;
+      allocsize = newnodesize - OVERHEAD_MM_ALLOCNODE;
 
       /* Add the original, newly freed node to the free nodelist */
 
@@ -208,16 +244,16 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
     }
 
   /* Check if there is free space at the end of the aligned chunk. Convert
-   * malloc-compatible chunk size to include SIZEOF_MM_ALLOCNODE as needed
+   * malloc-compatible chunk size to include OVERHEAD_MM_ALLOCNODE as needed
    * for mm_shrinkchunk.
    */
 
-  size = MM_ALIGN_UP(size + SIZEOF_MM_ALLOCNODE);
+  size = MM_ALIGN_UP(size + OVERHEAD_MM_ALLOCNODE);
 
   if (allocsize > size)
     {
       /* Shrink the chunk by that much -- remember, mm_shrinkchunk wants
-       * internal chunk sizes that include SIZEOF_MM_ALLOCNODE.
+       * internal chunk sizes that include OVERHEAD_MM_ALLOCNODE.
        */
 
       mm_shrinkchunk(heap, node, size);
@@ -228,7 +264,7 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
   MM_ADD_BACKTRACE(heap, node);
 
   kasan_unpoison((FAR void *)alignedchunk,
-                 mm_malloc_size((FAR void *)alignedchunk));
+                 mm_malloc_size(heap, (FAR void *)alignedchunk));
 
   DEBUGASSERT(alignedchunk % alignment == 0);
   return (FAR void *)alignedchunk;
