@@ -29,7 +29,6 @@
 #include <assert.h>
 #include <debug.h>
 #include <execinfo.h>
-#include <malloc.h>
 #include <sched.h>
 #include <stdio.h>
 #include <string.h>
@@ -110,23 +109,23 @@ struct mm_heap_s
 #endif
 };
 
-struct memdump_info_s
-{
-  pid_t pid;
-  int   blks;
-  int   size;
-};
-
 #if CONFIG_MM_BACKTRACE >= 0
 struct memdump_backtrace_s
 {
   pid_t pid;                                /* The pid for caller */
+  unsigned long seqno;                      /* The sequence of memory malloc */
 #if CONFIG_MM_BACKTRACE > 0
   FAR void *backtrace[CONFIG_MM_BACKTRACE]; /* The backtrace buffer for caller */
 #endif
 };
 #endif
 
+struct mm_mallinfo_handler_s
+{
+  FAR const struct malltask *task;
+  FAR struct mallinfo_task *info;
+};
+
 /****************************************************************************
  * Private Functions
  ****************************************************************************/
@@ -138,22 +137,23 @@ struct memdump_backtrace_s
  ****************************************************************************/
 
 static void memdump_backtrace(FAR struct mm_heap_s *heap,
-                              FAR struct memdump_backtrace_s *dump)
+                              FAR struct memdump_backtrace_s *buf)
 {
 #  if CONFIG_MM_BACKTRACE > 0
   FAR struct tcb_s *tcb;
 #  endif
 
-  dump->pid = _SCHED_GETPID();
+  buf->pid = _SCHED_GETTID();
+  buf->seqno = g_mm_seqno++;
 #  if CONFIG_MM_BACKTRACE > 0
-  tcb = nxsched_get_tcb(dump->pid);
+  tcb = nxsched_get_tcb(buf->pid);
   if (heap->mm_procfs.backtrace ||
-      (tcb && tcb->flags & TCB_FLAG_HEAPDUMP))
+      (tcb && tcb->flags & TCB_FLAG_HEAP_DUMP))
     {
-      int ret = backtrace(dump->backtrace, CONFIG_MM_BACKTRACE);
+      int ret = backtrace(buf->backtrace, CONFIG_MM_BACKTRACE);
       if (ret < CONFIG_MM_BACKTRACE)
         {
-          dump->backtrace[ret] = NULL;
+          buf->backtrace[ret] = NULL;
         }
     }
 #  endif
@@ -220,6 +220,35 @@ static void free_delaylist(FAR struct mm_heap_s *heap)
 #endif
 }
 
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0 && CONFIG_MM_BACKTRACE >= 0
+
+/****************************************************************************
+ * Name: mempool_memalign
+ *
+ * Description:
+ *   This function call mm_memalign and set mm_backtrace pid to free pid
+ *   avoid repeated calculation.
+ ****************************************************************************/
+
+static FAR void *mempool_memalign(FAR void *arg, size_t alignment,
+                                  size_t size)
+{
+  FAR struct memdump_backtrace_s *buf;
+  FAR void *ret;
+
+  ret = mm_memalign(arg, alignment, size);
+  if (ret)
+    {
+      buf = ret + mm_malloc_size(arg, ret);
+      buf->pid = PID_MM_MEMPOOL;
+    }
+
+  return ret;
+}
+#else
+#  define mempool_memalign mm_memalign
+#endif
+
 /****************************************************************************
  * Name: mallinfo_handler
  ****************************************************************************/
@@ -251,33 +280,37 @@ static void mallinfo_handler(FAR void *ptr, size_t size, int used,
 static void mallinfo_task_handler(FAR void *ptr, size_t size, int used,
                                   FAR void *user)
 {
-#if CONFIG_MM_BACKTRACE >= 0
-  FAR struct memdump_backtrace_s *dump;
-#endif
-  FAR struct mallinfo_task *info = user;
-
-#if CONFIG_MM_BACKTRACE >= 0
-  size -= sizeof(struct memdump_backtrace_s);
-  dump = ptr + size;
+  FAR struct mm_mallinfo_handler_s *handler = user;
+  FAR const struct malltask *task = handler->task;
+  FAR struct mallinfo_task *info = handler->info;
 
   if (used)
     {
 #if CONFIG_MM_BACKTRACE < 0
-      if (info->pid = -1)
-#else
-      if (info->pid == -1 || info->pid == dump->pid)
-#endif
+      if (task->pid == PID_MM_ALLOC)
         {
           info->aordblks++;
           info->uordblks += size;
         }
+#else
+      FAR struct memdump_backtrace_s *buf =
+        ptr + size - sizeof(struct memdump_backtrace_s);
+
+      if ((task->pid == buf->pid ||
+           (task->pid == PID_MM_ALLOC && buf->pid != PID_MM_MEMPOOL) ||
+           (task->pid == PID_MM_LEAK && !!nxsched_get_tcb(buf->pid))) &&
+          buf->seqno >= task->seqmin && buf->seqno <= task->seqmax)
+        {
+          info->aordblks++;
+          info->uordblks += size;
+        }
+#endif
     }
-  else if (info->pid == -2)
+  else if (task->pid == PID_MM_FREE)
     {
       info->aordblks++;
       info->uordblks += size;
     }
-#endif
 }
 
 /****************************************************************************
@@ -366,47 +399,46 @@ static void mm_unlock(FAR struct mm_heap_s *heap)
 static void memdump_handler(FAR void *ptr, size_t size, int used,
                             FAR void *user)
 {
-  pid_t pid = *(FAR pid_t *)user;
-#if CONFIG_MM_BACKTRACE >= 0
-  FAR struct memdump_backtrace_s *dump;
-
-  size -= sizeof(struct memdump_backtrace_s);
-  dump = ptr + size;
-#endif
+  FAR const struct mm_memdump_s *dump = user;
 
   if (used)
     {
 #if CONFIG_MM_BACKTRACE < 0
-      if (pid == -1)
+      if (dump->pid == PID_MM_ALLOC)
 #else
-      if (pid == -1 || dump->pid == pid)
+      FAR struct memdump_backtrace_s *buf =
+        ptr + size - sizeof(struct memdump_backtrace_s);
+
+      if ((dump->pid == buf->pid ||
+           (dump->pid == PID_MM_ALLOC && buf->pid != PID_MM_MEMPOOL) ||
+           (dump->pid == PID_MM_LEAK && !!nxsched_get_tcb(buf->pid))) &&
+          buf->seqno >= dump->seqmin && buf->seqno <= dump->seqmax)
 #endif
         {
 #if CONFIG_MM_BACKTRACE < 0
           syslog(LOG_INFO, "%12zu%*p\n", size, MM_PTR_FMT_WIDTH, ptr);
 #else
-#  if CONFIG_MM_BACKTRACE > 0
-          int i;
-          FAR const char *format = " %0*p";
-#  endif
-          char buf[CONFIG_MM_BACKTRACE * MM_PTR_FMT_WIDTH + 1];
+          char tmp[CONFIG_MM_BACKTRACE * MM_PTR_FMT_WIDTH + 1] = "";
 
-          buf[0] = '\0';
 #  if CONFIG_MM_BACKTRACE > 0
-          for (i = 0; i < CONFIG_MM_BACKTRACE && dump->backtrace[i]; i++)
+          FAR const char *format = " %0*p";
+          int i;
+
+          for (i = 0; i < CONFIG_MM_BACKTRACE && buf->backtrace[i]; i++)
             {
-              sprintf(buf + i * MM_PTR_FMT_WIDTH, format,
-                      MM_PTR_FMT_WIDTH - 1, dump->backtrace[i]);
+              snprintf(tmp + i * MM_PTR_FMT_WIDTH,
+                       sizeof(tmp) - i * MM_PTR_FMT_WIDTH,
+                       format, MM_PTR_FMT_WIDTH - 1, buf->backtrace[i]);
             }
 #  endif
 
-         syslog(LOG_INFO, "%6d%12zu%*p%s\n",
-                (int)dump->pid, size, MM_PTR_FMT_WIDTH,
-                ptr, buf);
+         syslog(LOG_INFO, "%6d%12zu%12lu%*p%s\n",
+                buf->pid, size, buf->seqno, MM_PTR_FMT_WIDTH,
+                ptr, tmp);
 #endif
         }
     }
-  else if (pid <= -2)
+  else if (dump->pid == PID_MM_FREE)
     {
       syslog(LOG_INFO, "%12zu%*p\n", size, MM_PTR_FMT_WIDTH, ptr);
     }
@@ -451,7 +483,7 @@ void mm_addregion(FAR struct mm_heap_s *heap, FAR void *heapstart,
     }
 
 #else
-# define idx 0
+#  define idx 0
 #endif
 
   /* Register to KASan for access check */
@@ -513,25 +545,24 @@ FAR void *mm_brkaddr(FAR struct mm_heap_s *heap, int region)
 
 FAR void *mm_calloc(FAR struct mm_heap_s *heap, size_t n, size_t elem_size)
 {
-  FAR void *ret = NULL;
+  FAR void *mem = NULL;
 
-  /* Verify input parameters */
+  /* Verify input parameters
+   *
+   * elem_size or n is zero treats as valid input.
+   *
+   * Assure that the following multiplication cannot overflow the size_t
+   * type, i.e., that:  SIZE_MAX >= n * elem_size
+   *
+   * Refer to SEI CERT C Coding Standard.
+   */
 
-  if (n > 0 && elem_size > 0)
+  if (elem_size == 0 || n <= (SIZE_MAX / elem_size))
     {
-      /* Assure that the following multiplication cannot overflow the size_t
-       * type, i.e., that:  SIZE_MAX >= n * elem_size
-       *
-       * Refer to SEI CERT C Coding Standard.
-       */
-
-      if (n <= (SIZE_MAX / elem_size))
-        {
-          ret = mm_zalloc(heap, n * elem_size);
-        }
+      mem = mm_zalloc(heap, n * elem_size);
     }
 
-  return ret;
+  return mem;
 }
 
 #ifdef CONFIG_DEBUG_MM
@@ -548,7 +579,7 @@ void mm_checkcorruption(FAR struct mm_heap_s *heap)
 #if CONFIG_MM_REGIONS > 1
   int region;
 #else
-# define region 0
+#  define region 0
 #endif
 
   /* Visit each region */
@@ -800,11 +831,12 @@ FAR struct mm_heap_s *mm_initialize(FAR const char *name,
     }
 
   heap->mm_mpool = mempool_multiple_init(name, poolsize, MEMPOOL_NPOOLS,
-                                  (mempool_multiple_alloc_t)mm_memalign,
-                                  (mempool_multiple_free_t)mm_free, heap,
-                                  CONFIG_MM_HEAP_MEMPOOL_EXPAND,
-                                  CONFIG_MM_HEAP_MEMPOOL_DICTIONARY_EXPAND,
-                                  true);
+                              (mempool_multiple_alloc_t)mempool_memalign,
+                              (mempool_multiple_alloc_size_t)mm_malloc_size,
+                              (mempool_multiple_free_t)mm_free, heap,
+                              CONFIG_MM_HEAP_MEMPOOL_CHUNK_SIZE,
+                              CONFIG_MM_HEAP_MEMPOOL_EXPAND_SIZE,
+                              CONFIG_MM_HEAP_MEMPOOL_DICTIONARY_EXPAND_SIZE);
 #endif
 
   return heap;
@@ -818,17 +850,19 @@ FAR struct mm_heap_s *mm_initialize(FAR const char *name,
  *
  ****************************************************************************/
 
-int mm_mallinfo(FAR struct mm_heap_s *heap, FAR struct mallinfo *info)
+struct mallinfo mm_mallinfo(FAR struct mm_heap_s *heap)
 {
+  struct mallinfo info;
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
+  struct mallinfo poolinfo;
+#endif
 #if CONFIG_MM_REGIONS > 1
   int region;
 #else
-# define region 0
+#  define region 0
 #endif
 
-  DEBUGASSERT(info);
-
-  memset(info, 0, sizeof(struct mallinfo));
+  memset(&info, 0, sizeof(struct mallinfo));
 
   /* Visit each region */
 
@@ -840,34 +874,45 @@ int mm_mallinfo(FAR struct mm_heap_s *heap, FAR struct mallinfo *info)
 
       DEBUGVERIFY(mm_lock(heap));
       tlsf_walk_pool(heap->mm_heapstart[region],
-                     mallinfo_handler, info);
+                     mallinfo_handler, &info);
       mm_unlock(heap);
     }
 #undef region
 
-  info->arena    = heap->mm_heapsize;
-  info->uordblks = info->arena - info->fordblks;
+  info.arena    = heap->mm_heapsize;
+  info.uordblks = info.arena - info.fordblks;
 
-  return OK;
+#if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
+  poolinfo = mempool_multiple_mallinfo(heap->mm_mpool);
+
+  info.uordblks -= poolinfo.fordblks;
+  info.fordblks += poolinfo.fordblks;
+#endif
+
+  return info;
 }
 
-int mm_mallinfo_task(FAR struct mm_heap_s *heap,
-                     FAR struct mallinfo_task *info)
+struct mallinfo_task mm_mallinfo_task(FAR struct mm_heap_s *heap,
+                                      FAR const struct malltask *task)
 {
+  struct mm_mallinfo_handler_s handle;
+  struct mallinfo_task info =
+    {
+      0, 0
+    };
+
 #if CONFIG_MM_REGIONS > 1
   int region;
 #else
 #define region 0
 #endif
 
-  DEBUGASSERT(info);
-  info->uordblks = 0;
-  info->aordblks = 0;
-
 #if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
-  mempool_multiple_info_task(heap->mm_mpool, info);
+  info = mempool_multiple_info_task(heap->mm_mpool, task);
 #endif
 
+  handle.task = task;
+  handle.info = &info;
 #if CONFIG_MM_REGIONS > 1
   for (region = 0; region < heap->mm_nregions; region++)
 #endif
@@ -876,12 +921,12 @@ int mm_mallinfo_task(FAR struct mm_heap_s *heap,
 
       DEBUGVERIFY(mm_lock(heap));
       tlsf_walk_pool(heap->mm_heapstart[region],
-                     mallinfo_task_handler, info);
+                     mallinfo_task_handler, &handle);
       mm_unlock(heap);
     }
 #undef region
 
-  return OK;
+  return info;
 }
 
 /****************************************************************************
@@ -895,23 +940,24 @@ int mm_mallinfo_task(FAR struct mm_heap_s *heap,
  *   than or equal to 0, will dump pid allocated node and output backtrace.
  ****************************************************************************/
 
-void mm_memdump(FAR struct mm_heap_s *heap, pid_t pid)
+void mm_memdump(FAR struct mm_heap_s *heap,
+                FAR const struct mm_memdump_s *dump)
 {
 #if CONFIG_MM_REGIONS > 1
   int region;
 #else
-# define region 0
+#  define region 0
 #endif
-  struct memdump_info_s info;
+  struct mallinfo_task info;
 
-  if (pid >= -1)
+  if (dump->pid >= PID_MM_ALLOC)
     {
       syslog(LOG_INFO, "Dump all used memory node info:\n");
 #if CONFIG_MM_BACKTRACE < 0
       syslog(LOG_INFO, "%12s%*s\n", "Size", MM_PTR_FMT_WIDTH, "Address");
 #else
-      syslog(LOG_INFO, "%6s%12s%*s %s\n", "PID", "Size", MM_PTR_FMT_WIDTH,
-            "Address", "Backtrace");
+      syslog(LOG_INFO, "%6s%12s%12s%*s %s\n", "PID", "Size", "Sequence",
+                        MM_PTR_FMT_WIDTH, "Address", "Backtrace");
 #endif
     }
   else
@@ -921,7 +967,7 @@ void mm_memdump(FAR struct mm_heap_s *heap, pid_t pid)
     }
 
 #if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
-  mempool_multiple_memdump(heap->mm_mpool, pid);
+  mempool_multiple_memdump(heap->mm_mpool, dump);
 #endif
 
 #if CONFIG_MM_REGIONS > 1
@@ -930,15 +976,14 @@ void mm_memdump(FAR struct mm_heap_s *heap, pid_t pid)
     {
       DEBUGVERIFY(mm_lock(heap));
       tlsf_walk_pool(heap->mm_heapstart[region],
-                     memdump_handler, &pid);
+                     memdump_handler, (FAR void *)dump);
       mm_unlock(heap);
     }
 #undef region
 
-  info.pid = pid;
-  mm_mallinfo_task(heap, &info);
+  info = mm_mallinfo_task(heap, dump);
   syslog(LOG_INFO, "%12s%12s\n", "Total Blks", "Total Size");
-  syslog(LOG_INFO, "%12d%12d\n", info.blks, info.size);
+  syslog(LOG_INFO, "%12d%12d\n", info.aordblks, info.uordblks);
 }
 
 /****************************************************************************
@@ -977,6 +1022,13 @@ FAR void *mm_malloc(FAR struct mm_heap_s *heap, size_t size)
 {
   FAR void *ret;
 
+  /* In case of zero-length allocations allocate the minimum size object */
+
+  if (size < 1)
+    {
+      size = 1;
+    }
+
 #if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
   ret = mempool_multiple_alloc(heap->mm_mpool, size);
   if (ret != NULL)
@@ -1004,9 +1056,9 @@ FAR void *mm_malloc(FAR struct mm_heap_s *heap, size_t size)
   if (ret)
     {
 #if CONFIG_MM_BACKTRACE >= 0
-      FAR struct memdump_backtrace_s *dump = ret + mm_malloc_size(heap, ret);
+      FAR struct memdump_backtrace_s *buf = ret + mm_malloc_size(heap, ret);
 
-      memdump_backtrace(heap, dump);
+      memdump_backtrace(heap, buf);
 #endif
       kasan_unpoison(ret, mm_malloc_size(heap, ret));
     }
@@ -1058,9 +1110,9 @@ FAR void *mm_memalign(FAR struct mm_heap_s *heap, size_t alignment,
   if (ret)
     {
 #if CONFIG_MM_BACKTRACE >= 0
-      FAR struct memdump_backtrace_s *dump = ret + mm_malloc_size(heap, ret);
+      FAR struct memdump_backtrace_s *buf = ret + mm_malloc_size(heap, ret);
 
-      memdump_backtrace(heap, dump);
+      memdump_backtrace(heap, buf);
 #endif
       kasan_unpoison(ret, mm_malloc_size(heap, ret));
     }
@@ -1096,15 +1148,20 @@ FAR void *mm_realloc(FAR struct mm_heap_s *heap, FAR void *oldmem,
 {
   FAR void *newmem;
 
+  /* If oldmem is NULL, then realloc is equivalent to malloc */
+
   if (oldmem == NULL)
     {
       return mm_malloc(heap, size);
     }
 
-  if (size == 0)
+  /* If size is zero, reallocate to the minim size object, so
+   * the memory pointed by oldmem is freed
+   */
+
+  if (size < 1)
     {
-      mm_free(heap, oldmem);
-      return NULL;
+      size = 1;
     }
 
 #if CONFIG_MM_HEAP_MEMPOOL_THRESHOLD != 0
@@ -1127,7 +1184,6 @@ FAR void *mm_realloc(FAR struct mm_heap_s *heap, FAR void *oldmem,
 #endif
 
 #ifdef CONFIG_MM_KASAN
-
   newmem = mm_malloc(heap, size);
   if (newmem)
     {
@@ -1158,10 +1214,10 @@ FAR void *mm_realloc(FAR struct mm_heap_s *heap, FAR void *oldmem,
 #if CONFIG_MM_BACKTRACE >= 0
   if (newmem)
     {
-      FAR struct memdump_backtrace_s *dump = newmem +
-                                             mm_malloc_size(heap, newmem);
+      FAR struct memdump_backtrace_s *buf =
+        newmem + mm_malloc_size(heap, newmem);
 
-      memdump_backtrace(heap, dump);
+      memdump_backtrace(heap, buf);
     }
 #endif
 
