@@ -43,8 +43,9 @@
  * Private Data
  ****************************************************************************/
 
-#ifndef CONFIG_NET_ALLOC_CONNS
-static struct devif_callback_s g_cbprealloc[CONFIG_NET_NACTIVESOCKETS];
+#if CONFIG_NET_PREALLOC_DEVIF_CALLBACKS > 0
+static struct devif_callback_s
+  g_cbprealloc[CONFIG_NET_PREALLOC_DEVIF_CALLBACKS];
 #endif
 static FAR struct devif_callback_s *g_cbfreelist = NULL;
 
@@ -164,11 +165,24 @@ static void devif_callback_free(FAR struct net_driver_s *dev,
             }
         }
 
-      /* Put the structure into the free list */
+      /* If this is a preallocated or a batch allocated callback store it in
+       * the free callbacks list. Else free it.
+       */
+
+#if CONFIG_NET_ALLOC_DEVIF_CALLBACKS == 1
+      if (cb < g_cbprealloc || cb >= (g_cbprealloc +
+          CONFIG_NET_PREALLOC_DEVIF_CALLBACKS))
+        {
+          kmm_free(cb);
+        }
+      else
+#endif
+        {
+          cb->nxtconn  = g_cbfreelist;
+          cb->nxtdev   = NULL;
+          g_cbfreelist = cb;
+        }
 
-      cb->nxtconn  = g_cbfreelist;
-      cb->nxtdev   = NULL;
-      g_cbfreelist = cb;
       net_unlock();
     }
 }
@@ -230,10 +244,10 @@ static bool devif_event_trigger(uint16_t events, uint16_t triggers)
 
 void devif_callback_init(void)
 {
-#ifndef CONFIG_NET_ALLOC_CONNS
+#if CONFIG_NET_PREALLOC_DEVIF_CALLBACKS > 0
   int i;
 
-  for (i = 0; i < CONFIG_NET_NACTIVESOCKETS; i++)
+  for (i = 0; i < CONFIG_NET_PREALLOC_DEVIF_CALLBACKS; i++)
     {
       g_cbprealloc[i].nxtconn = g_cbfreelist;
       g_cbfreelist = &g_cbprealloc[i];
@@ -263,7 +277,7 @@ FAR struct devif_callback_s *
                        FAR struct devif_callback_s **list_tail)
 {
   FAR struct devif_callback_s *ret;
-#ifdef CONFIG_NET_ALLOC_CONNS
+#if CONFIG_NET_ALLOC_DEVIF_CALLBACKS > 0
   int i;
 #endif
 
@@ -290,14 +304,14 @@ FAR struct devif_callback_s *
 
   /* Allocate the callback entry from heap */
 
-#ifdef CONFIG_NET_ALLOC_CONNS
+#if CONFIG_NET_ALLOC_DEVIF_CALLBACKS > 0
   if (g_cbfreelist == NULL)
     {
       ret = kmm_zalloc(sizeof(struct devif_callback_s) *
-                       CONFIG_NET_NACTIVESOCKETS);
+                       CONFIG_NET_ALLOC_DEVIF_CALLBACKS);
       if (ret != NULL)
         {
-          for (i = 0; i < CONFIG_NET_NACTIVESOCKETS; i++)
+          for (i = 0; i < CONFIG_NET_ALLOC_DEVIF_CALLBACKS; i++)
             {
               ret[i].nxtconn = g_cbfreelist;
               g_cbfreelist = &ret[i];
