@@ -202,7 +202,7 @@ static void ieee802154_addref(FAR struct socket *psock)
   DEBUGASSERT(psock != NULL && psock->s_conn != NULL &&
               (psock->s_type == SOCK_DGRAM || psock->s_type == SOCK_CTRL));
 
-  conn = (FAR struct ieee802154_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
   DEBUGASSERT(conn->crefs > 0 && conn->crefs < 255);
   conn->crefs++;
 }
@@ -248,7 +248,7 @@ static int ieee802154_connect(FAR struct socket *psock,
   int ret = OK;
 
   DEBUGASSERT(psock != NULL || addr != NULL);
-  conn = (FAR struct ieee802154_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
   DEBUGASSERT(conn != NULL);
 
   /* Verify the address family */
@@ -315,7 +315,7 @@ static int ieee802154_bind(FAR struct socket *psock,
       return -EBADF;
     }
 
-  conn = (FAR struct ieee802154_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
 
   /* Bind a PF_IEEE802154 socket to an network device. */
 
@@ -402,7 +402,7 @@ static int ieee802154_getsockname(FAR struct socket *psock,
 
   DEBUGASSERT(psock != NULL && addr != NULL && addrlen != NULL);
 
-  conn = (FAR struct ieee802154_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
   DEBUGASSERT(conn != NULL);
 
   /* Create a copy of the full address on the stack */
@@ -464,7 +464,7 @@ static int ieee802154_getpeername(FAR struct socket *psock,
 
   DEBUGASSERT(psock != NULL && addr != NULL && addrlen != NULL);
 
-  conn = (FAR struct ieee802154_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
   DEBUGASSERT(conn != NULL);
 
   /* Create a copy of the full address on the stack */
