@@ -584,6 +584,9 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
             }
           else if (ackno == TCP_WBSEQNO(wrb))
             {
+#ifdef CONFIG_NET_TCP_CC_NEWRENO
+              if (conn->dupacks >= TCP_FAST_RETRANSMISSION_THRESH)
+#else
               /* Reset the duplicate ack counter */
 
               if ((flags & TCP_NEWDATA) != 0)
@@ -594,6 +597,7 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
               /* Duplicate ACK? Retransmit data if need */
 
               if (++TCP_WBNACK(wrb) == TCP_FAST_RETRANSMISSION_THRESH)
+#endif
                 {
 #ifdef CONFIG_NET_TCP_SELECTIVE_ACK
                   if ((conn->flags & TCP_SACK) &&
@@ -614,11 +618,12 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
                       /* Do fast retransmit */
 
                       rexmitno = ackno;
-#endif
-
+#ifndef CONFIG_NET_TCP_CC_NEWRENO
                       /* Reset counter */
 
                       TCP_WBNACK(wrb) = 0;
+#endif
+#endif
                     }
                 }
             }
@@ -690,6 +695,7 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
       FAR sq_entry_t *entry;
       FAR sq_entry_t *next;
       size_t sndlen;
+      int ret;
 
       /* According to RFC 6298 (5.4), retransmit the earliest segment
        * that has not been acknowledged by the TCP receiver.
@@ -737,13 +743,26 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
 
           tcp_setsequence(conn->sndseq, TCP_WBSEQNO(wrb));
 
-          devif_iob_send(dev, TCP_WBIOB(wrb), sndlen,
-                         0, tcpip_hdrsize(conn));
-          if (dev->d_sndlen == 0)
+          ret = devif_iob_send(dev, TCP_WBIOB(wrb), sndlen,
+                               0, tcpip_hdrsize(conn));
+          if (ret <= 0)
             {
               return flags;
             }
 
+#ifdef CONFIG_NET_TCP_CC_NEWRENO
+          /* After Fast retransmitted, set ssthresh to the maximum of
+           * the unacked and the 2*SMSS, and enter to Fast Recovery.
+           * ssthresh = max (FlightSize / 2, 2*SMSS) referring to rfc5681
+           * cwnd=ssthresh + 3*SMSS  referring to rfc5681
+           */
+
+          if (conn->flags & TCP_INFT)
+            {
+              tcp_cc_update(conn, NULL);
+            }
+#endif
+
           /* Reset the retransmission timer. */
 
           tcp_update_retrantimer(conn, conn->rto);
@@ -802,6 +821,19 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
               right = ofosegs[i].right;
             }
         }
+
+#ifdef CONFIG_NET_TCP_CC_NEWRENO
+          /* After Fast retransmitted, set ssthresh to the maximum of
+           * the unacked and the 2*SMSS, and enter to Fast Recovery.
+           * ssthresh = max (FlightSize / 2, 2*SMSS) referring to rfc5681
+           * cwnd=ssthresh + 3*SMSS  referring to rfc5681
+           */
+
+          if (conn->flags & TCP_INFT)
+            {
+              tcp_cc_update(conn, NULL);
+            }
+#endif
     }
   else
 #endif
@@ -965,10 +997,16 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
        */
 
       seq = TCP_WBSEQNO(wrb) + TCP_WBSENT(wrb);
+
+#ifdef CONFIG_NET_TCP_CC_NEWRENO
+      snd_wnd_edge = conn->snd_wl2 + MIN(conn->snd_wnd, conn->cwnd);
+#else
       snd_wnd_edge = conn->snd_wl2 + conn->snd_wnd;
+#endif
       if (TCP_SEQ_LT(seq, snd_wnd_edge))
         {
           uint32_t remaining_snd_wnd;
+          int ret;
 
           sndlen = TCP_WBPKTLEN(wrb) - TCP_WBSENT(wrb);
           if (sndlen > conn->mss)
@@ -1012,9 +1050,9 @@ static uint16_t psock_send_eventhandler(FAR struct net_driver_s *dev,
            * won't actually happen until the polling cycle completes).
            */
 
-          devif_iob_send(dev, TCP_WBIOB(wrb), sndlen,
-                         TCP_WBSENT(wrb), tcpip_hdrsize(conn));
-          if (dev->d_sndlen == 0)
+          ret = devif_iob_send(dev, TCP_WBIOB(wrb), sndlen,
+                               TCP_WBSENT(wrb), tcpip_hdrsize(conn));
+          if (ret <= 0)
             {
               return flags;
             }
@@ -1231,7 +1269,7 @@ ssize_t psock_tcp_send(FAR struct socket *psock, FAR const void *buf,
       goto errout;
     }
 
-  conn = (FAR struct tcp_conn_s *)psock->s_conn;
+  conn = psock->s_conn;
 
   if (!_SS_ISCONNECTED(conn->sconn.s_flags))
     {
