@@ -59,6 +59,11 @@ volatile uint32_t g_system_ticks = INITIAL_SYSTEM_TIMER_TICKS;
 struct timespec   g_basetime;
 #endif
 
+#ifdef CONFIG_CLOCK_ADJTIME
+long long g_clk_adj_usec;
+long long g_clk_adj_count;
+#endif
+
 /****************************************************************************
  * Private Functions
  ****************************************************************************/
@@ -222,6 +227,12 @@ void clock_initialize(void)
 
   clock_inittime(NULL);
 #endif
+
+#if defined(CONFIG_CLOCK_ADJTIME)
+  g_clk_adj_count = 0;
+  g_clk_adj_usec = 0;
+#endif
+
 #endif
 }
 
@@ -292,7 +303,7 @@ void clock_synchronize(FAR const struct timespec *tp)
  *
  ****************************************************************************/
 
-#if defined(CONFIG_RTC) && !defined(CONFIG_SCHED_TICKLESS)
+#if defined(CONFIG_RTC) && !defined(CONFIG_SCHED_TICKLESS) && !defined(CONFIG_CLOCK_TIMEKEEPING)
 void clock_resynchronize(FAR struct timespec *rtc_diff)
 {
   struct timespec rtc_time;
@@ -403,6 +414,40 @@ skip:
 }
 #endif
 
+/****************************************************************************
+ * Name: clock_set_adjust
+ *
+ * Description:
+ *   This function is called from adjtime() defined in clock_adjtime.c if
+ *   CONFIG_CLOCK_ADJTIME is enabled. It sets global variables g_clk_adj_usec
+ *   and g_clk_adj_count which are used for clock adjustment.
+ *
+ * Input Parameters:
+ *   adj_usec  - period adjustment in usec
+ *   adj_count - number of clock ticks over which we apply adj_usec
+ *
+ ****************************************************************************/
+
+#ifdef CONFIG_CLOCK_ADJTIME
+void clock_set_adjust(long long adj_usec, long long adj_count,
+                      long long *adj_usec_old, long long *adj_count_old)
+{
+  /* Get old adjust values. */
+
+  *adj_usec_old  = g_clk_adj_usec;
+  *adj_count_old = g_clk_adj_count;
+
+  /* Set new adjust values. */
+
+  g_clk_adj_usec  = adj_usec;
+  g_clk_adj_count = adj_count;
+
+  /* And change timer period. */
+
+  up_adj_timer_period(g_clk_adj_usec);
+}
+#endif
+
 /****************************************************************************
  * Name: clock_timer
  *
@@ -419,5 +464,25 @@ void clock_timer(void)
   /* Increment the per-tick system counter */
 
   g_system_ticks++;
+
+#ifdef CONFIG_CLOCK_ADJTIME
+  /* Do we apply timer adjustment? */
+
+  if (g_clk_adj_count > 0)
+    {
+      /* Yes, decrement the count each tick. */
+
+      g_clk_adj_count--;
+
+      /* Check if clock adjusment is finished. */
+
+      if (g_clk_adj_count == 0)
+        {
+          /* Yes... reset timer period. */
+
+          up_adj_timer_period(0);
+        }
+    }
+#endif
 }
 #endif
