@@ -25,6 +25,7 @@
 #include <nuttx/config.h>
 
 #include <nuttx/arch.h>
+#include <nuttx/binfmt/binfmt.h>
 #include <nuttx/board.h>
 #include <nuttx/irq.h>
 #include <nuttx/tls.h>
@@ -49,6 +50,8 @@
  * Pre-processor Definitions
  ****************************************************************************/
 
+#define DEADLOCK_MAX 8
+
 #ifndef CONFIG_BOARD_RESET_ON_ASSERT
 #  define CONFIG_BOARD_RESET_ON_ASSERT 0
 #endif
@@ -69,7 +72,15 @@
  * Private Data
  ****************************************************************************/
 
-static uint8_t g_last_regs[XCPTCONTEXT_SIZE];
+static uint8_t g_last_regs[XCPTCONTEXT_SIZE] aligned_data(16);
+
+#ifdef CONFIG_BOARD_COREDUMP
+static struct lib_syslogstream_s  g_syslogstream;
+static struct lib_hexdumpstream_s g_hexstream;
+#  ifdef CONFIG_BOARD_COREDUMP_COMPRESSION
+static struct lib_lzfoutstream_s  g_lzfstream;
+#  endif
+#endif
 
 static FAR const char *g_policy[4] =
 {
@@ -144,86 +155,126 @@ static void dump_stack(FAR const char *tag, uintptr_t sp,
   uintptr_t top = base + size;
 
   _alert("%s Stack:\n", tag);
-  _alert("sp:     %p\n", (FAR void *)sp);
   _alert("  base: %p\n", (FAR void *)base);
   _alert("  size: %08zu\n", size);
 
-  if (sp >= base && sp < top)
+  if (!force)
     {
+      _alert("    sp: %p\n", (FAR void *)sp);
       stack_dump(sp, top);
     }
   else
     {
-      _alert("ERROR: %s Stack pointer is not within the stack\n", tag);
-
-      if (force)
-        {
 #ifdef CONFIG_STACK_COLORATION
-          size_t remain = size - used;
+      size_t remain = size - used;
 
-          base += remain;
-          size -= remain;
+      base += remain;
+      size -= remain;
 #endif
 
 #if CONFIG_ARCH_STACKDUMP_MAX_LENGTH > 0
-          if (size > CONFIG_ARCH_STACKDUMP_MAX_LENGTH)
-            {
-              size = CONFIG_ARCH_STACKDUMP_MAX_LENGTH;
-            }
+      if (size > CONFIG_ARCH_STACKDUMP_MAX_LENGTH)
+        {
+          size = CONFIG_ARCH_STACKDUMP_MAX_LENGTH;
+        }
 #endif
 
-          stack_dump(base, base + size);
-        }
+      stack_dump(base, base + size);
     }
 }
 
 /****************************************************************************
- * Name: showstacks
+ * Name: dump_stacks
  ****************************************************************************/
 
-static void show_stacks(FAR struct tcb_s *rtcb)
+static void dump_stacks(FAR struct tcb_s *rtcb, uintptr_t sp)
 {
-  uintptr_t sp = up_getsp();
-
-  /* Get the limits on the interrupt stack memory */
+#if CONFIG_ARCH_INTERRUPTSTACK > 0
+  uintptr_t intstack_base = up_get_intstackbase();
+  size_t intstack_size = CONFIG_ARCH_INTERRUPTSTACK;
+  uintptr_t intstack_top = intstack_base + intstack_size;
+  FAR void *intstack_sp = NULL;
+#endif
+#ifdef CONFIG_ARCH_KERNEL_STACK
+  uintptr_t kernelstack_base = (uintptr_t)rtcb->xcp.kstack;
+  size_t kernelstack_size = CONFIG_ARCH_KERNEL_STACKSIZE;
+  uintptr_t kernelstack_top = kernelstack_base + kernelstack_size;
+  FAR void *kernelstack_sp = NULL;
+#endif
+  uintptr_t tcbstack_base = (uintptr_t)rtcb->stack_base_ptr;
+  size_t tcbstack_size = (size_t)rtcb->adj_stack_size;
+  uintptr_t tcbstack_top = tcbstack_base + tcbstack_size;
+  FAR void *tcbstack_sp = NULL;
+  bool force = false;
 
 #if CONFIG_ARCH_INTERRUPTSTACK > 0
-  dump_stack("IRQ", sp,
-             up_get_intstackbase(),
-             CONFIG_ARCH_INTERRUPTSTACK,
-#  ifdef CONFIG_STACK_COLORATION
-             up_check_intstack(),
-#  else
-             0,
-#  endif
-             up_interrupt_context());
-  if (up_interrupt_context())
+  if (sp >= intstack_base && sp < intstack_top)
     {
-      sp = up_getusrsp();
+      intstack_sp = (FAR void *)sp;
+      tcbstack_sp = (FAR void *)up_getusrsp(rtcb->xcp.regs);
+    }
+  else
+#endif
+#ifdef CONFIG_ARCH_KERNEL_STACK
+  if (sp >= kernelstack_base && sp < kernelstack_top)
+    {
+      kernelstack_sp = (FAR void *)sp;
+    }
+  else
+#endif
+  if (sp >= tcbstack_base && sp < tcbstack_top)
+    {
+      tcbstack_sp = (FAR void *)sp;
+    }
+  else
+    {
+      force = true;
+      _alert("ERROR: Stack pointer is not within the stack\n");
+    }
+
+#if CONFIG_ARCH_INTERRUPTSTACK > 0
+  if (intstack_sp != NULL || force)
+    {
+      dump_stack("IRQ",
+                 (uintptr_t)intstack_sp,
+                 intstack_base,
+                 intstack_size,
+#ifdef CONFIG_STACK_COLORATION
+                 up_check_intstack(),
+#else
+                 0,
+#endif
+                 force
+                );
     }
 #endif
 
-  dump_stack("User", sp,
-             (uintptr_t)rtcb->stack_base_ptr,
-             (size_t)rtcb->adj_stack_size,
-#ifdef CONFIG_STACK_COLORATION
-             up_check_tcbstack(rtcb),
-#else
-             0,
-#endif
 #ifdef CONFIG_ARCH_KERNEL_STACK
-             false
-#else
-             true
+  if (kernelstack_sp != NULL || force)
+    {
+      dump_stack("Kernel",
+                 (uintptr_t)kernelstack_sp,
+                 kernelstack_base,
+                 kernelstack_size,
+                 0, force
+                );
+    }
 #endif
-            );
 
-#ifdef CONFIG_ARCH_KERNEL_STACK
-  dump_stack("Kernel", sp,
-             (uintptr_t)rtcb->xcp.kstack,
-             CONFIG_ARCH_KERNEL_STACKSIZE,
-             0, false);
+  if (tcbstack_sp != NULL || force)
+    {
+      dump_stack("User",
+                 (uintptr_t)tcbstack_sp,
+                 tcbstack_base,
+                 tcbstack_size,
+#ifdef CONFIG_STACK_COLORATION
+                 up_check_tcbstack(rtcb),
+#else
+                 0,
 #endif
+                 force
+                );
+    }
 }
 
 #endif
@@ -343,10 +394,10 @@ static void dump_backtrace(FAR struct tcb_s *tcb, FAR void *arg)
 #endif
 
 /****************************************************************************
- * Name: showtasks
+ * Name: dump_tasks
  ****************************************************************************/
 
-static void show_tasks(void)
+static void dump_tasks(void)
 {
 #if CONFIG_ARCH_INTERRUPTSTACK > 0 && defined(CONFIG_STACK_COLORATION)
   size_t stack_used = up_check_intstack();
@@ -369,7 +420,7 @@ static void show_tasks(void)
 #endif
          " PRI POLICY   TYPE    NPX"
          " STATE   EVENT"
-         "      SIGMASK"
+         "      SIGMASK        "
          "  STACKBASE"
          "  STACKSIZE"
 #ifdef CONFIG_STACK_COLORATION
@@ -415,28 +466,111 @@ static void show_tasks(void)
 #endif
 }
 
+/****************************************************************************
+ * Name: dump_core
+ ****************************************************************************/
+
+#ifdef CONFIG_BOARD_COREDUMP
+static void dump_core(pid_t pid)
+{
+  FAR void *stream;
+  int logmask;
+
+  logmask = setlogmask(LOG_ALERT);
+
+  _alert("Start coredump:\n");
+
+  /* Initialize hex output stream */
+
+  lib_syslogstream(&g_syslogstream, LOG_EMERG);
+
+  stream = &g_syslogstream;
+
+  lib_hexdumpstream(&g_hexstream, stream);
+
+  stream = &g_hexstream;
+
+#  ifdef CONFIG_BOARD_COREDUMP_COMPRESSION
+
+  /* Initialize LZF compression stream */
+
+  lib_lzfoutstream(&g_lzfstream, stream);
+  stream = &g_lzfstream;
+
+#  endif
+
+  /* Do core dump */
+
+  core_dump(NULL, stream, pid);
+
+#  ifdef CONFIG_BOARD_COREDUMP_COMPRESSION
+  _alert("Finish coredump (Compression Enabled).\n");
+#  else
+  _alert("Finish coredump.\n");
+#  endif
+
+  setlogmask(logmask);
+}
+#endif
+
+/****************************************************************************
+ * Name: dump_deadlock
+ ****************************************************************************/
+
+#ifdef CONFIG_ARCH_DEADLOCKDUMP
+static void dump_deadlock(void)
+{
+  pid_t deadlock[DEADLOCK_MAX];
+  size_t i = nxsched_collect_deadlock(deadlock, DEADLOCK_MAX);
+
+  if (i > 0)
+    {
+      _alert("Deadlock detected\n");
+      while (i-- > 0)
+        {
+#ifdef CONFIG_SCHED_BACKTRACE
+          sched_dumpstack(deadlock[i]);
+#else
+          _alert("deadlock pid: %d\n", deadlock[i])
+#endif
+        }
+    }
+}
+#endif
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
 
-void _assert(FAR const char *filename, int linenum, FAR const char *msg)
+/****************************************************************************
+ * Name: _assert
+ ****************************************************************************/
+
+void _assert(FAR const char *filename, int linenum,
+             FAR const char *msg, FAR void *regs)
 {
   FAR struct tcb_s *rtcb = running_task();
   struct utsname name;
-  bool fatal = false;
+  bool fatal = true;
+
+  /* try to save current context if regs is null */
+
+  if (regs == NULL)
+    {
+      up_saveusercontext(g_last_regs);
+      regs = g_last_regs;
+    }
 
   /* Flush any buffered SYSLOG data (from prior to the assertion) */
 
   syslog_flush();
 
 #if CONFIG_BOARD_RESET_ON_ASSERT < 2
-  if (up_interrupt_context() ||
-      (rtcb->flags & TCB_FLAG_TTYPE_MASK) == TCB_FLAG_TTYPE_KERNEL)
+  if (!up_interrupt_context() &&
+      (rtcb->flags & TCB_FLAG_TTYPE_MASK) != TCB_FLAG_TTYPE_KERNEL)
     {
-      fatal = true;
+      fatal = false;
     }
-#else
-  fatal = true;
 #endif
 
   panic_notifier_call_chain(fatal ? PANIC_KERNEL : PANIC_TASK, rtcb);
@@ -473,18 +607,10 @@ void _assert(FAR const char *filename, int linenum, FAR const char *msg)
 
   /* Register dump */
 
-  if (up_interrupt_context())
-    {
-      up_dump_register(NULL);
-    }
-  else
-    {
-      up_saveusercontext(g_last_regs);
-      up_dump_register(g_last_regs);
-    }
+  up_dump_register(regs);
 
 #ifdef CONFIG_ARCH_STACKDUMP
-  show_stacks(rtcb);
+  dump_stacks(rtcb, up_getusrsp(regs));
 #endif
 
   /* Flush any buffered SYSLOG data */
@@ -493,7 +619,13 @@ void _assert(FAR const char *filename, int linenum, FAR const char *msg)
 
   if (fatal)
     {
-      show_tasks();
+      dump_tasks();
+
+#ifdef CONFIG_ARCH_DEADLOCKDUMP
+      /* Deadlock Dump */
+
+      dump_deadlock();
+#endif
 
 #ifdef CONFIG_ARCH_USBDUMP
       /* Dump USB trace data */
@@ -502,7 +634,17 @@ void _assert(FAR const char *filename, int linenum, FAR const char *msg)
 #endif
 
 #ifdef CONFIG_BOARD_CRASHDUMP
-      board_crashdump(up_getsp(), rtcb, filename, linenum, msg);
+      board_crashdump(up_getsp(), rtcb, filename, linenum, msg, regs);
+
+#elif defined(CONFIG_BOARD_COREDUMP)
+      /* Dump core information */
+
+#  ifdef CONFIG_BOARD_COREDUMP_FULL
+      dump_core(INVALID_PROCESS_ID);
+#  else
+      dump_core(rtcb->pid);
+#  endif
+
 #endif
 
       /* Flush any buffered SYSLOG data */
