@@ -148,13 +148,15 @@ int sched_lock(void)
 
   if (rtcb != NULL && !up_interrupt_context())
     {
+      irqstate_t flags;
+
       /* Catch attempts to increment the lockcount beyond the range of the
        * integer type.
        */
 
       DEBUGASSERT(rtcb->lockcount < MAX_LOCK_COUNT);
 
-      irqstate_t flags = enter_critical_section();
+      flags = enter_critical_section();
 
       /* We must hold the lock on this CPU before we increment the lockcount
        * for the first time. Holding the lock is sufficient to lockout
