@@ -26,6 +26,7 @@
 #include <nuttx/compiler.h>
 
 #include <time.h>
+#include <sys/param.h>
 #include <assert.h>
 #include <debug.h>
 
@@ -69,14 +70,6 @@
 #  define KEEP_ALIVE_TICKS MSEC2TICK(80)
 #endif
 
-#ifndef MIN
-#  define MIN(a,b) (((a) < (b)) ? (a) : (b))
-#endif
-
-#ifndef MAX
-#  define MAX(a,b) (((a) > (b)) ? (a) : (b))
-#endif
-
 /****************************************************************************
  * Public Data
  ****************************************************************************/
