@@ -84,38 +84,30 @@ nxsem_allocholder(FAR sem_t *sem, FAR struct tcb_s *htcb)
        * put it into the semaphore's holder list
        */
 
-      g_freeholders    = pholder->flink;
-      pholder->flink   = sem->hhead;
-      sem->hhead       = pholder;
+      g_freeholders  = pholder->flink;
+      pholder->flink = sem->hhead;
+      sem->hhead     = pholder;
     }
 #else
-  if (sem->holder[0].htcb == NULL)
+  if (sem->holder.htcb == NULL)
     {
-      pholder          = &sem->holder[0];
-    }
-  else if (sem->holder[1].htcb == NULL)
-    {
-      pholder          = &sem->holder[1];
+      pholder = &sem->holder;
     }
 #endif
   else
     {
       serr("ERROR: Insufficient pre-allocated holders\n");
-      pholder          = NULL;
-      DEBUGPANIC();
+      PANIC();
     }
 
-  if (pholder != NULL)
-    {
-      pholder->sem    = sem;
-      pholder->htcb   = htcb;
-      pholder->counts = 0;
+  pholder->sem    = sem;
+  pholder->htcb   = htcb;
+  pholder->counts = 0;
 
-      /* Put it into the task's list */
+  /* Put it into the task's list */
 
-      pholder->tlink  = htcb->holdsem;
-      htcb->holdsem   = pholder;
-    }
+  pholder->tlink  = htcb->holdsem;
+  htcb->holdsem   = pholder;
 
   return pholder;
 }
@@ -149,19 +141,15 @@ nxsem_findholder(FAR sem_t *sem, FAR struct tcb_s *htcb)
         }
     }
 #else
-  int i;
+  /* We have one hard-allocated holder structures in sem_t */
 
-  /* We have two hard-allocated holder structures in sem_t */
+  pholder = &sem->holder;
 
-  for (i = 0; i < 2; i++)
+  if (pholder->htcb == htcb)
     {
-      pholder = &sem->holder[i];
-      if (pholder->htcb == htcb)
-        {
-          /* Got it! */
+      /* Got it! */
 
-          return pholder;
-        }
+      return pholder;
     }
 #endif
 
@@ -178,7 +166,7 @@ static inline FAR struct semholder_s *
 nxsem_findorallocateholder(FAR sem_t *sem, FAR struct tcb_s *htcb)
 {
   FAR struct semholder_s *pholder = nxsem_findholder(sem, htcb);
-  if (!pholder)
+  if (pholder == NULL)
     {
       pholder = nxsem_allocholder(sem, htcb);
     }
@@ -240,6 +228,7 @@ static inline void nxsem_freeholder(FAR sem_t *sem,
  * Name: nxsem_freecount0holder
  ****************************************************************************/
 
+#if CONFIG_SEM_PREALLOCHOLDERS > 0
 static int nxsem_freecount0holder(FAR struct semholder_s *pholder,
                                   FAR sem_t *sem, FAR void *arg)
 {
@@ -255,6 +244,7 @@ static int nxsem_freecount0holder(FAR struct semholder_s *pholder,
 
   return 0;
 }
+#endif
 
 /****************************************************************************
  * Name: nxsem_foreachholder
@@ -282,22 +272,17 @@ static int nxsem_foreachholder(FAR sem_t *sem, holderhandler_t handler,
       ret = handler(pholder, sem, arg);
     }
 #else
-  int i;
+  /* We have one hard-allocated holder structures in sem_t */
 
-  /* We have two hard-allocated holder structures in sem_t */
+  pholder = &sem->holder;
 
-  for (i = 0; i < 2 && ret == 0; i++)
+  /* The hard-allocated containers may hold a NULL holder */
+
+  if (pholder->htcb != NULL)
     {
-      pholder = &sem->holder[i];
+      /* Call the handler */
 
-      /* The hard-allocated containers may hold a NULL holder */
-
-      if (pholder->htcb != NULL)
-        {
-          /* Call the handler */
-
-          ret = handler(pholder, sem, arg);
-        }
+      ret = handler(pholder, sem, arg);
     }
 #endif
 
@@ -392,24 +377,13 @@ static int nxsem_dumpholder(FAR struct semholder_s *pholder, FAR sem_t *sem,
 #endif
 
 /****************************************************************************
- * Name: nxsem_restoreholderprio
+ * Name: nxsem_restore_priority
  ****************************************************************************/
 
-static int nxsem_restoreholderprio(FAR struct semholder_s *pholder,
-                                   FAR sem_t *sem, FAR void *arg)
+static void nxsem_restore_priority(FAR struct tcb_s *htcb)
 {
-  FAR struct tcb_s *htcb = pholder->htcb;
   int hpriority;
 
-  /* Release the holder if all counts have been given up
-   * before reprioritizing causes a context switch.
-   */
-
-  if (pholder->counts <= 0)
-    {
-      nxsem_freeholder(sem, pholder);
-    }
-
   /* We attempt to restore thread priority to its base priority.  If
    * there is any thread with the higher priority waiting for the
    * semaphore held by htcb then this value will be overwritten.
@@ -424,6 +398,8 @@ static int nxsem_restoreholderprio(FAR struct semholder_s *pholder,
 
   if (htcb->sched_priority != hpriority)
     {
+      FAR struct semholder_s *pholder;
+
       /* Try to find the highest priority across all the threads that are
        * waiting for any semaphore held by htcb.
        */
@@ -447,10 +423,33 @@ static int nxsem_restoreholderprio(FAR struct semholder_s *pholder,
 
       nxsched_set_priority(htcb, hpriority);
     }
+}
+
+/****************************************************************************
+ * Name: nxsem_restoreholderprio
+ ****************************************************************************/
+
+static int nxsem_restoreholderprio(FAR struct semholder_s *pholder,
+                                   FAR sem_t *sem, FAR void *arg)
+{
+  FAR struct tcb_s *htcb = pholder->htcb;
+
+  /* Release the holder if all counts have been given up
+   * before reprioritizing causes a context switch.
+   */
+
+  if (pholder->counts <= 0)
+    {
+      nxsem_freeholder(sem, pholder);
+    }
+
+  nxsem_restore_priority(htcb);
 
   return 0;
 }
 
+#if CONFIG_SEM_PREALLOCHOLDERS > 0
+
 /****************************************************************************
  * Name: nxsem_restoreholderprio_others
  *
@@ -495,96 +494,7 @@ static int nxsem_restoreholderprio_self(FAR struct semholder_s *pholder,
   return 0;
 }
 
-/****************************************************************************
- * Name: nxsem_restore_baseprio_irq
- *
- * Description:
- *   This function is called after an interrupt handler posts a count on
- *   the semaphore.  It will check if we need to drop the priority of any
- *   threads holding a count on the semaphore.  Their priority could have
- *   been boosted while they held the count.
- *
- * Input Parameters:
- *   stcb - The TCB of the task that was just started (if any).  If the
- *     post action caused a count to be given to another thread, then stcb
- *     is the TCB that received the count.  Note, just because stcb received
- *     the count, it does not mean that it is higher priority than other
- *     threads.
- *   sem - A reference to the semaphore being posted.
- *     - If the semaphore count is <0 then there are still threads waiting
- *       for a count.  stcb should be non-null and will be higher priority
- *       than all of the other threads still waiting.
- *     - If it is ==0 then stcb refers to the thread that got the last count;
- *       no other threads are waiting.
- *     - If it is >0 then there should be no threads waiting for counts and
- *       stcb should be null.
- *
- * Returned Value:
- *   None
- *
- * Assumptions:
- *   The scheduler is locked.
- *
- ****************************************************************************/
-
-static inline void nxsem_restore_baseprio_irq(FAR struct tcb_s *stcb,
-                                              FAR sem_t *sem)
-{
-  /* Drop the priority of all holder threads */
-
-  nxsem_foreachholder(sem, nxsem_restoreholderprio, stcb);
-}
-
-/****************************************************************************
- * Name: nxsem_restore_baseprio_task
- *
- * Description:
- *   This function is called after the current running task releases a
- *   count on the semaphore.  It will check if we need to drop the priority
- *   of any threads holding a count on the semaphore.  Their priority could
- *   have been boosted while they held the count.
- *
- * Input Parameters:
- *   stcb - The TCB of the task that was just started (if any).  If the
- *     post action caused a count to be given to another thread, then stcb
- *     is the TCB that received the count.  Note, just because stcb received
- *     the count, it does not mean that it is higher priority than other
- *     threads.
- *   sem - A reference to the semaphore being posted.
- *     - If the semaphore count is <0 then there are still threads waiting
- *       for a count.  stcb should be non-null and will be higher priority
- *       than all of the other threads still waiting.
- *     - If it is ==0 then stcb refers to the thread that got the last count;
- *       no other threads are waiting.
- *     - If it is >0 then there should be no threads waiting for counts and
- *       stcb should be null.
- *
- * Returned Value:
- *   None
- *
- * Assumptions:
- *   The scheduler is locked.
- *
- ****************************************************************************/
-
-static inline void nxsem_restore_baseprio_task(FAR struct tcb_s *stcb,
-                                               FAR sem_t *sem)
-{
-  /* The currently executed thread should be the lower priority
-   * thread that just posted the count and caused this action.
-   * However, we cannot drop the priority of the currently running
-   * thread -- because that will cause it to be suspended.
-   *
-   * So, do this in two passes.  First, reprioritizing all holders
-   * except for the running thread.
-   */
-
-  nxsem_foreachholder(sem, nxsem_restoreholderprio_others, stcb);
-
-  /* Now, find an reprioritize only the ready to run task */
-
-  nxsem_foreachholder(sem, nxsem_restoreholderprio_self, stcb);
-}
+#endif
 
 /****************************************************************************
  * Public Functions
@@ -671,7 +581,7 @@ void nxsem_destroyholder(FAR sem_t *sem)
 #else
   /* There may be an issue if there are multiple holders of the semaphore. */
 
-  DEBUGASSERT(sem->holder[0].htcb == NULL || sem->holder[1].htcb == NULL);
+  DEBUGASSERT(sem->holder.htcb == NULL);
 
 #endif
 
@@ -713,7 +623,7 @@ void nxsem_add_holder_tcb(FAR struct tcb_s *htcb, FAR sem_t *sem)
       /* Find or allocate a container for this new holder */
 
       pholder = nxsem_findorallocateholder(sem, htcb);
-      if (pholder != NULL && pholder->counts < SEM_VALUE_MAX)
+      if (pholder->counts < SEM_VALUE_MAX)
         {
           /* Increment the number of counts held by this holder */
 
@@ -769,7 +679,11 @@ void nxsem_boost_priority(FAR sem_t *sem)
    * count.
    */
 
+#if CONFIG_SEM_PREALLOCHOLDERS > 0
   nxsem_foreachholder(sem, nxsem_boostholderprio, rtcb);
+#else
+  nxsem_boostholderprio(&sem->holder, sem, rtcb);
+#endif
 }
 
 /****************************************************************************
@@ -792,61 +706,47 @@ void nxsem_boost_priority(FAR sem_t *sem)
 void nxsem_release_holder(FAR sem_t *sem)
 {
   FAR struct tcb_s *rtcb = this_task();
-  FAR struct semholder_s *pholder;
-  FAR struct semholder_s *candidate = NULL;
-  unsigned int total = 0;
+  uint8_t prioinherit = sem->flags & SEM_PRIO_MASK;
 
-  /* Find the container for this holder */
+  /* If priority inheritance is disabled for this thread or it is IDLE
+   * thread, then do not add the holder.
+   * If there are never holders of the semaphore, the priority
+   * inheritance is effectively disabled.
+   */
+
+  if (!is_idle_task(rtcb) && prioinherit == SEM_PRIO_INHERIT)
+    {
+      FAR struct semholder_s *pholder;
+
+      DEBUGASSERT(!up_interrupt_context());
+
+      /* Find the container for this holder */
 
 #if CONFIG_SEM_PREALLOCHOLDERS > 0
-  for (pholder = sem->hhead; pholder != NULL; pholder = pholder->flink)
+      for (pholder = sem->hhead; pholder != NULL; pholder = pholder->flink)
+        {
+          DEBUGASSERT(pholder->counts > 0);
+
+          if (pholder->htcb == rtcb)
+            {
+              /* Decrement the counts on this holder -- the holder will be
+               * freed later in nxsem_restore_baseprio.
+               */
+
+              pholder->counts--;
+              return;
+            }
+        }
+
+      /* The current task is not a holder */
+
+      DEBUGPANIC();
 #else
-  int i;
-
-  /* We have two hard-allocated holder structures in sem_t */
-
-  for (i = 0; i < 2; i++)
+      pholder = &sem->holder;
+      DEBUGASSERT(pholder->htcb == rtcb);
+      nxsem_freeholder(sem, pholder);
 #endif
-    {
-#if CONFIG_SEM_PREALLOCHOLDERS == 0
-      pholder = &sem->holder[i];
-      if (pholder->htcb == NULL)
-        {
-          continue;
-        }
-#endif
-
-      DEBUGASSERT(pholder->counts > 0);
-
-      if (pholder->htcb == rtcb)
-        {
-          /* Decrement the counts on this holder -- the holder will be freed
-           * later in nxsem_restore_baseprio.
-           */
-
-          pholder->counts--;
-          return;
-        }
-
-      total++;
-      candidate = pholder;
     }
-
-  /* The current task is not a holder */
-
-  if (total == 1)
-    {
-      /* If the semaphore has only one holder, we can decrement the counts
-       * simply.
-       */
-
-      candidate->counts--;
-      return;
-    }
-
-  /* TODO:
-   *   How do we choose the holder to decrement it's counts?
-   */
 }
 
 /****************************************************************************
@@ -891,6 +791,8 @@ void nxsem_restore_baseprio(FAR struct tcb_s *stcb, FAR sem_t *sem)
               (sem->semcount <= 0 && stcb != NULL));
 #endif
 
+  DEBUGASSERT(!up_interrupt_context());
+
   /* Perform the following actions only if a new thread was given a count.
    * The thread that received the count should be the highest priority
    * of all threads waiting for a count from the semaphore.  So in that
@@ -900,25 +802,33 @@ void nxsem_restore_baseprio(FAR struct tcb_s *stcb, FAR sem_t *sem)
 
   if (stcb != NULL)
     {
-      /* Handler semaphore counts posted from an interrupt handler
-       * differently from interrupts posted from threads.  The primary
-       * difference is that if the semaphore is posted from a thread, then
-       * the poster thread is a player in the priority inheritance scheme.
-       * The interrupt handler externally injects the new count without
-       * otherwise participating itself.
+#if CONFIG_SEM_PREALLOCHOLDERS > 0
+      /* The currently executed thread should be the lower priority
+       * thread that just posted the count and caused this action.
+       * However, we cannot drop the priority of the currently running
+       * thread -- because that will cause it to be suspended.
+       *
+       * So, do this in two passes.  First, reprioritizing all holders
+       * except for the running thread.
        */
 
-      if (up_interrupt_context())
-        {
-          nxsem_restore_baseprio_irq(stcb, sem);
-        }
-      else
-        {
-          nxsem_restore_baseprio_task(stcb, sem);
-        }
+      nxsem_foreachholder(sem, nxsem_restoreholderprio_others, stcb);
+
+      /* Now, find an reprioritize only the ready to run task */
+
+      nxsem_foreachholder(sem, nxsem_restoreholderprio_self, stcb);
+#else
+      /* New owner is already the highest priority since the wait queue
+       * is priority-based, no need to adjust its priority, only restore
+       * the older owner when posted the count.
+       */
+
+      nxsem_restore_priority(this_task());
+#endif
     }
   else
     {
+#if CONFIG_SEM_PREALLOCHOLDERS > 0
       /* Remove the holder from the list if it's counts is zero. */
 
       nxsem_foreachholder(sem, nxsem_freecount0holder, NULL);
@@ -926,6 +836,7 @@ void nxsem_restore_baseprio(FAR struct tcb_s *stcb, FAR sem_t *sem)
       /* If there are no tasks waiting for available counts, then all holders
        * should be at their base priority.
        */
+#endif
 
 #ifdef CONFIG_DEBUG_ASSERTIONS
       nxsem_foreachholder(sem, nxsem_verifyholder, NULL);
