@@ -59,6 +59,8 @@
  *     CONFIG_LIBC_ENVPATH is defined, this may be either a relative or
  *     or an absolute path.  Otherwise, it must be an absolute path.
  *
+ *   actions - The spawn file actions
+ *
  *   attr - If the value of the 'attr' parameter is NULL, the all default
  *     values for the POSIX spawn attributes will be used.  Otherwise, the
  *     attributes will be set according to the spawn flags.  The
@@ -84,6 +86,7 @@
  ****************************************************************************/
 
 static int nxposix_spawn_exec(FAR pid_t *pidp, FAR const char *path,
+                              FAR const posix_spawn_file_actions_t *actions,
                               FAR const posix_spawnattr_t *attr,
                               FAR char * const argv[],
                               FAR char * const envp[])
@@ -108,7 +111,7 @@ static int nxposix_spawn_exec(FAR pid_t *pidp, FAR const char *path,
 
   /* Start the task */
 
-  pid = exec_spawn(path, argv, envp, symtab, nsymbols, attr);
+  pid = exec_spawn(path, argv, envp, symtab, nsymbols, actions, attr);
   if (pid < 0)
     {
       ret = -pid;
@@ -140,81 +143,6 @@ errout:
   return ret;
 }
 
-/****************************************************************************
- * Name: nxposix_spawn_proxy
- *
- * Description:
- *   Perform file_actions, then execute the task from the file system.
- *
- *   Do we really need this proxy task?  Isn't that wasteful?
- *
- *   Q: Why not use a starthook so that there is callout from nxtask_start()
- *      to perform these operations after the file is loaded from
- *      the file system?
- *   A: That existing nxtask_starthook() implementation cannot be used in
- *      this context; any of nxtask_starthook() will also conflict with
- *      binfmt's use of the start hook to call C++ static initializers.
- *      task_restart() would also be an issue.
- *
- * Input Parameters:
- *   Standard task start-up parameters
- *
- * Returned Value:
- *   Standard task return value.
- *
- ****************************************************************************/
-
-static int nxposix_spawn_proxy(int argc, FAR char *argv[])
-{
-  int ret;
-
-  /* Perform file actions and/or set a custom signal mask.  We get here only
-   * if the file_actions parameter to posix_spawn[p] was non-NULL and/or the
-   * option to change the signal mask was selected.
-   */
-
-  DEBUGASSERT(g_spawn_parms.file_actions ||
-              (g_spawn_parms.attr &&
-               (g_spawn_parms.attr->flags & POSIX_SPAWN_SETSIGMASK) != 0));
-
-  /* Set the attributes and perform the file actions as appropriate */
-
-  ret = spawn_proxyattrs(g_spawn_parms.attr, g_spawn_parms.file_actions);
-  if (ret == OK)
-    {
-      /* Start the task */
-
-      ret = nxposix_spawn_exec(g_spawn_parms.pid, g_spawn_parms.u.posix.path,
-                               g_spawn_parms.attr, g_spawn_parms.argv,
-                               g_spawn_parms.envp);
-
-#ifdef CONFIG_SCHED_HAVE_PARENT
-      if (ret == OK)
-        {
-          /* Change of the parent of the task we just spawned to our parent.
-           * What should we do in the event of a failure?
-           */
-
-          int tmp = task_reparent(0, *g_spawn_parms.pid);
-          if (tmp < 0)
-            {
-              serr("ERROR: task_reparent() failed: %d\n", tmp);
-            }
-        }
-#endif
-    }
-
-  /* Post the semaphore to inform the parent task that we have completed
-   * what we need to do.
-   */
-
-  g_spawn_parms.result = ret;
-#ifndef CONFIG_SCHED_WAITPID
-  nxsem_post(&g_spawn_execsem);
-#endif
-  return OK;
-}
-
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
@@ -311,125 +239,15 @@ int posix_spawn(FAR pid_t *pid, FAR const char *path,
                 FAR const posix_spawnattr_t *attr,
                 FAR char * const argv[], FAR char * const envp[])
 {
-  struct sched_param param;
-  int proxy;
-#ifdef CONFIG_SCHED_WAITPID
-  int status;
-#endif
-  int ret;
-
-  DEBUGASSERT(path);
-
   sinfo("pid=%p path=%s file_actions=%p attr=%p argv=%p\n",
         pid, path, file_actions, attr, argv);
 
-  /* If there are no file actions to be performed and there is no change to
-   * the signal mask, then start the new child task directly from the parent
-   * task.
-   */
-
-  if ((file_actions == NULL || *file_actions == NULL) &&
-      (attr == NULL || (attr->flags & POSIX_SPAWN_SETSIGMASK) == 0))
+  if (attr != NULL)
     {
-      return nxposix_spawn_exec(pid, path, attr, argv, envp);
+      spawn_proxyattrs(attr);
     }
 
-  /* Otherwise, we will have to go through an intermediary/proxy task in
-   * order to perform the I/O redirection.  This would be a natural place
-   * to fork().  However, true fork() behavior requires an MMU and most
-   * implementations of vfork() are not capable of these operations.
-   *
-   * Even without fork(), we can still do the job, but parameter passing is
-   * messier.  Unfortunately, there is no (clean) way to pass binary values
-   * as a task parameter, so we will use a semaphore-protected global
-   * structure.
-   */
-
-  /* Get exclusive access to the global parameter structure */
-
-  ret = nxmutex_lock(&g_spawn_parmlock);
-  if (ret < 0)
-    {
-      serr("ERROR: nxmutex_lock failed: %d\n", ret);
-      return -ret;
-    }
-
-  /* Populate the parameter structure */
-
-  g_spawn_parms.result       = ENOSYS;
-  g_spawn_parms.pid          = pid;
-  g_spawn_parms.file_actions = file_actions ? *file_actions : NULL;
-  g_spawn_parms.attr         = attr;
-  g_spawn_parms.argv         = argv;
-  g_spawn_parms.envp         = envp;
-  g_spawn_parms.u.posix.path = path;
-
-  /* Get the priority of this (parent) task */
-
-  ret = nxsched_get_param(0, &param);
-  if (ret < 0)
-    {
-      serr("ERROR: nxsched_get_param failed: %d\n", ret);
-      nxmutex_unlock(&g_spawn_parmlock);
-      return -ret;
-    }
-
-  /* Disable pre-emption so that the proxy does not run until waitpid
-   * is called.  This is probably unnecessary since the nxposix_spawn_proxy
-   * has the same priority as this thread; it should be schedule behind
-   * this task in the ready-to-run list.
-   */
-
-#ifdef CONFIG_SCHED_WAITPID
-  sched_lock();
-#endif
-
-  /* Start the intermediary/proxy task at the same priority as the parent
-   * task.
-   */
-
-  proxy = nxthread_create("nxposix_spawn_proxy", TCB_FLAG_TTYPE_KERNEL,
-                          param.sched_priority, NULL,
-                          CONFIG_POSIX_SPAWN_PROXY_STACKSIZE,
-                          nxposix_spawn_proxy, NULL, environ);
-  if (proxy < 0)
-    {
-      ret = -proxy;
-      serr("ERROR: Failed to start nxposix_spawn_proxy: %d\n", ret);
-      goto errout_with_lock;
-    }
-
-  /* Wait for the proxy to complete its job */
-
-#ifdef CONFIG_SCHED_WAITPID
-  /* REVISIT: This should not call waitpid() directly.  waitpid is a
-   * cancellation point and modifies the errno value.  It is inappropriate
-   * for use within the OS.
-   */
-
-  ret = nxsched_waitpid((pid_t)proxy, &status, 0);
-  if (ret < 0)
-    {
-      serr("ERROR: waitpid() failed: %d\n", ret);
-      goto errout_with_lock;
-    }
-#else
-  ret = nxsem_wait_uninterruptible(&g_spawn_execsem);
-  if (ret < 0)
-    {
-      serr("ERROR: nxsem_wait_uninterruptible() failed: %d\n", ret);
-      goto errout_with_lock;
-    }
-#endif
-
-  /* Get the result and relinquish our access to the parameter structure */
-
-  ret = g_spawn_parms.result;
-
-errout_with_lock:
-#ifdef CONFIG_SCHED_WAITPID
-  sched_unlock();
-#endif
-  nxmutex_unlock(&g_spawn_parmlock);
-  return ret;
+  return nxposix_spawn_exec(pid, path,
+                            file_actions != NULL ?
+                            *file_actions : NULL, attr, argv, envp);
 }
