@@ -1078,3 +1078,22 @@ int bt_conn_le_conn_update(FAR struct bt_conn_s *conn, uint16_t min,
 
   return bt_hci_cmd_send(BT_HCI_OP_LE_CONN_UPDATE, buf);
 }
+
+/****************************************************************************
+ * Name: bt_conn_initialize
+ *
+ * Description:
+ *   Initialize this module's private data.
+ *
+ * Input Parameters:
+ *   None
+ *
+ * Returned Value:
+ *   None
+ *
+ ****************************************************************************/
+
+void bt_conn_initialize(void)
+{
+  memset(g_conns, 0, sizeof(g_conns));
+}
